@@ -40,9 +40,7 @@ extern "C" {
 static knh_StringData_t StringData[] = {
 	{"", TSN_EMPTY}, 
 	{"\t", TSN_TAB}, 
-	{"\n", TSN_LF}, 
-	{" ", TSN_SPC}, 
-	{"EOL", TSN_EOL}, 
+	{""KONOHA_OS_LINEFEED"", TSN_EOL}, 
 	{"BEGIN", TSN_BEGIN}, 
 	{"END", TSN_END}, 
 	{"void", TSN_void}, 
@@ -70,7 +68,6 @@ static knh_StringData_t StringData[] = {
 static char* StructName[] =  {
 	"Object",
 	"Any",
-	"Nue",
 	"Boolean",
 	"Number",
 	"Int",
@@ -96,11 +93,9 @@ static char* StructName[] =  {
 	"ClassMap",
 	"Closure",
 	"AffineConv",
-	"IntUnit",
-	"FloatUnit",
 	"Regex",
 	"BytesConv",
-	"StringUnit",
+	"ClassSpec",
 	"InputStream",
 	"OutputStream",
 	"Socket",
@@ -139,16 +134,6 @@ static knh_StructData_t StructData[] = {
 		(knh_fstruct_newClass)knh_Any_newClass,
 		knh_Any_getkey
 	},
-	{
-		"Nue", STRUCT_Nue, FLAG_Nue, 0,
-		(knh_fstruct_init)knh_Nue_init,
-		(knh_fstruct_copy)knh_Nue_copy, 
-		(knh_fstruct_traverse)knh_Nue_traverse,
-		(knh_fstruct_compareTo)knh_Nue_compareTo,
-		(knh_fstruct_hashCode)knh_Nue_hashCode,
-		(knh_fstruct_newClass)knh_Nue_newClass,
-		knh_Nue_getkey
-	},
 	{
 		"Boolean", STRUCT_Boolean, FLAG_Boolean, 0,
 		(knh_fstruct_init)knh_Boolean_init,
@@ -380,7 +365,7 @@ static knh_StructData_t StructData[] = {
 		knh_ClassMap_getkey
 	},
 	{
-		"Closure", STRUCT_Closure, FLAG_Closure, sizeof(knh_Closure_struct),
+		"Closure", STRUCT_Closure, FLAG_Closure, 0,
 		(knh_fstruct_init)knh_Closure_init,
 		(knh_fstruct_copy)knh_Closure_copy, 
 		(knh_fstruct_traverse)knh_Closure_traverse,
@@ -399,26 +384,6 @@ static knh_StructData_t StructData[] = {
 		(knh_fstruct_newClass)knh_AffineConv_newClass,
 		knh_AffineConv_getkey
 	},
-	{
-		"IntUnit", STRUCT_IntUnit, FLAG_IntUnit, sizeof(knh_IntUnit_struct),
-		(knh_fstruct_init)knh_IntUnit_init,
-		(knh_fstruct_copy)knh_IntUnit_copy, 
-		(knh_fstruct_traverse)knh_IntUnit_traverse,
-		(knh_fstruct_compareTo)knh_IntUnit_compareTo,
-		(knh_fstruct_hashCode)knh_IntUnit_hashCode,
-		(knh_fstruct_newClass)knh_IntUnit_newClass,
-		knh_IntUnit_getkey
-	},
-	{
-		"FloatUnit", STRUCT_FloatUnit, FLAG_FloatUnit, sizeof(knh_FloatUnit_struct),
-		(knh_fstruct_init)knh_FloatUnit_init,
-		(knh_fstruct_copy)knh_FloatUnit_copy, 
-		(knh_fstruct_traverse)knh_FloatUnit_traverse,
-		(knh_fstruct_compareTo)knh_FloatUnit_compareTo,
-		(knh_fstruct_hashCode)knh_FloatUnit_hashCode,
-		(knh_fstruct_newClass)knh_FloatUnit_newClass,
-		knh_FloatUnit_getkey
-	},
 	{
 		"Regex", STRUCT_Regex, FLAG_Regex, 0,
 		(knh_fstruct_init)knh_Regex_init,
@@ -440,14 +405,14 @@ static knh_StructData_t StructData[] = {
 		knh_BytesConv_getkey
 	},
 	{
-		"StringUnit", STRUCT_StringUnit, FLAG_StringUnit, sizeof(knh_StringUnit_struct),
-		(knh_fstruct_init)knh_StringUnit_init,
-		(knh_fstruct_copy)knh_StringUnit_copy, 
-		(knh_fstruct_traverse)knh_StringUnit_traverse,
-		(knh_fstruct_compareTo)knh_StringUnit_compareTo,
-		(knh_fstruct_hashCode)knh_StringUnit_hashCode,
-		(knh_fstruct_newClass)knh_StringUnit_newClass,
-		knh_StringUnit_getkey
+		"ClassSpec", STRUCT_ClassSpec, FLAG_ClassSpec, sizeof(knh_ClassSpec_struct),
+		(knh_fstruct_init)knh_ClassSpec_init,
+		(knh_fstruct_copy)knh_ClassSpec_copy, 
+		(knh_fstruct_traverse)knh_ClassSpec_traverse,
+		(knh_fstruct_compareTo)knh_ClassSpec_compareTo,
+		(knh_fstruct_hashCode)knh_ClassSpec_hashCode,
+		(knh_fstruct_newClass)knh_ClassSpec_newClass,
+		knh_ClassSpec_getkey
 	},
 	{
 		"InputStream", STRUCT_InputStream, FLAG_InputStream, sizeof(knh_InputStream_struct),
@@ -603,23 +568,20 @@ static knh_StructData_t StructData[] = {
 };
 
 static knh_ClassData_t ClassData[] = {
-	{"konoha.Object", CLASS_Object, CFLAG_Object, STRUCT_Object, CLASS_Object, 0, 34, 0, knh_Object_fdefault},
-	{"konoha.Any", CLASS_Any, CFLAG_Any, STRUCT_Any, CLASS_Object, 0, 0, 0, knh_Any_fdefault},
+	{"konoha.Object", CLASS_Object, CFLAG_Object, STRUCT_Object, CLASS_Object, 0, 33, 0, knh_Object_fdefault},
+	{"konoha.Any", CLASS_Any, CFLAG_Any, STRUCT_Any, CLASS_Object, 0, 1, 0, knh_Any_fdefault},
 	{"konoha.This", CLASS_This, CFLAG_This, STRUCT_This, CLASS_Any, 0, 0, 0, knh_This_fdefault},
 	{"konoha.Any0", CLASS_Any0, CFLAG_Any0, STRUCT_Any0, CLASS_Any, 0, 0, 0, knh_Any0_fdefault},
 	{"konoha.Any1", CLASS_Any1, CFLAG_Any1, STRUCT_Any1, CLASS_Any, 0, 0, 0, knh_Any1_fdefault},
 	{"konoha.Any2", CLASS_Any2, CFLAG_Any2, STRUCT_Any2, CLASS_Any, 0, 0, 0, knh_Any2_fdefault},
 	{"konoha.Any3", CLASS_Any3, CFLAG_Any3, STRUCT_Any3, CLASS_Any, 0, 0, 0, knh_Any3_fdefault},
-	{"konoha.Nue", CLASS_Nue, CFLAG_Nue, STRUCT_Nue, CLASS_Object, 0, 1, 0, knh_Nue_fdefault},
+	{"konoha.AnyVar", CLASS_AnyVar, CFLAG_AnyVar, STRUCT_AnyVar, CLASS_Any, 0, 0, 0, knh_AnyVar_fdefault},
 	{"konoha.Boolean", CLASS_Boolean, CFLAG_Boolean, STRUCT_Boolean, CLASS_Object, 0, 3, 0, knh_Boolean_fdefault},
 	{"konoha.Number", CLASS_Number, CFLAG_Number, STRUCT_Number, CLASS_Object, 0, 0, 0, knh_Number_fdefault},
 	{"konoha.Int", CLASS_Int, CFLAG_Int, STRUCT_Int, CLASS_Number, 0, 31, 0, knh_Int_fdefault},
-	{"konoha.IntX", CLASS_IntX, CFLAG_IntX, STRUCT_IntX, CLASS_Number, 0, 0, 0, knh_IntX_fdefault},
 	{"konoha.Float", CLASS_Float, CFLAG_Float, STRUCT_Float, CLASS_Number, 0, 22, 0, knh_Float_fdefault},
-	{"konoha.FloatX", CLASS_FloatX, CFLAG_FloatX, STRUCT_FloatX, CLASS_Number, 0, 0, 0, knh_FloatX_fdefault},
 	{"konoha.String", CLASS_String, CFLAG_String, STRUCT_String, CLASS_Object, 0, 36, 0, knh_String_fdefault},
-	{"konoha.StringX", CLASS_StringX, CFLAG_StringX, STRUCT_StringX, CLASS_String, 0, 0, 0, knh_StringX_fdefault},
-	{"konoha.Bytes", CLASS_Bytes, CFLAG_Bytes, STRUCT_Bytes, CLASS_Object, 0, 10, 0, knh_Bytes_fdefault},
+	{"konoha.Bytes", CLASS_Bytes, CFLAG_Bytes, STRUCT_Bytes, CLASS_Object, 0, 11, 0, knh_Bytes_fdefault},
 	{"konoha.Tuple2", CLASS_Tuple2, CFLAG_Tuple2, STRUCT_Tuple2, CLASS_Object, 0, 0, 0, knh_Tuple2_fdefault},
 	{"konoha.Range", CLASS_Range, CFLAG_Range, STRUCT_Range, CLASS_Object, 0, 0, 0, knh_Range_fdefault},
 	{"konoha.Array", CLASS_Array, CFLAG_Array, STRUCT_Array, CLASS_Object, 0, 31, 0, knh_Array_fdefault},
@@ -631,19 +593,17 @@ static knh_ClassData_t ClassData[] = {
 	{"konoha.HashMap", CLASS_HashMap, CFLAG_HashMap, STRUCT_HashMap, CLASS_Object, 0, 8, 0, knh_HashMap_fdefault},
 	{"konoha.HashSet", CLASS_HashSet, CFLAG_HashSet, STRUCT_HashSet, CLASS_Object, 0, 0, 0, knh_HashSet_fdefault},
 	{"konoha.DictIdx", CLASS_DictIdx, CFLAG_DictIdx, STRUCT_DictIdx, CLASS_Object, 0, 6, 0, knh_DictIdx_fdefault},
-	{"konoha.Class", CLASS_Class, CFLAG_Class, STRUCT_Class, CLASS_Object, 0, 12, 0, knh_Class_fdefault},
+	{"konoha.Class", CLASS_Class, CFLAG_Class, STRUCT_Class, CLASS_Object, 0, 13, 0, knh_Class_fdefault},
 	{"konoha.ClassStruct", CLASS_ClassStruct, CFLAG_ClassStruct, STRUCT_ClassStruct, CLASS_Object, 0, 0, 0, knh_ClassStruct_fdefault},
 	{"konoha.MethodField", CLASS_MethodField, CFLAG_MethodField, STRUCT_MethodField, CLASS_Object, 0, 0, 0, knh_MethodField_fdefault},
-	{"konoha.Method", CLASS_Method, CFLAG_Method, STRUCT_Method, CLASS_Object, 0, 16, 0, knh_Method_fdefault},
-	{"konoha.Mapper", CLASS_Mapper, CFLAG_Mapper, STRUCT_Mapper, CLASS_Object, 0, 12, 0, knh_Mapper_fdefault},
+	{"konoha.Method", CLASS_Method, CFLAG_Method, STRUCT_Method, CLASS_Object, 0, 14, 0, knh_Method_fdefault},
+	{"konoha.Mapper", CLASS_Mapper, CFLAG_Mapper, STRUCT_Mapper, CLASS_Object, 0, 13, 0, knh_Mapper_fdefault},
 	{"konoha.ClassMap", CLASS_ClassMap, CFLAG_ClassMap, STRUCT_ClassMap, CLASS_Object, 0, 1, 0, knh_ClassMap_fdefault},
-	{"konoha.Closure", CLASS_Closure, CFLAG_Closure, STRUCT_Closure, CLASS_Object, 0, 3, 0, knh_Closure_fdefault},
+	{"konoha.Closure", CLASS_Closure, CFLAG_Closure, STRUCT_Closure, CLASS_Object, 0, 1, 0, knh_Closure_fdefault},
 	{"konoha.AffineConv", CLASS_AffineConv, CFLAG_AffineConv, STRUCT_AffineConv, CLASS_Object, 0, 0, 0, knh_AffineConv_fdefault},
-	{"konoha.IntUnit", CLASS_IntUnit, CFLAG_IntUnit, STRUCT_IntUnit, CLASS_Object, 0, 1, 0, knh_IntUnit_fdefault},
-	{"konoha.FloatUnit", CLASS_FloatUnit, CFLAG_FloatUnit, STRUCT_FloatUnit, CLASS_Object, 0, 0, 0, knh_FloatUnit_fdefault},
 	{"konoha.Regex", CLASS_Regex, CFLAG_Regex, STRUCT_Regex, CLASS_Object, 0, 3, 0, knh_Regex_fdefault},
 	{"konoha.BytesConv", CLASS_BytesConv, CFLAG_BytesConv, STRUCT_BytesConv, CLASS_Object, 0, 0, 0, knh_BytesConv_fdefault},
-	{"konoha.StringUnit", CLASS_StringUnit, CFLAG_StringUnit, STRUCT_StringUnit, CLASS_Object, 0, 0, 0, knh_StringUnit_fdefault},
+	{"konoha.ClassSpec", CLASS_ClassSpec, CFLAG_ClassSpec, STRUCT_ClassSpec, CLASS_Object, 0, 0, 0, knh_ClassSpec_fdefault},
 	{"konoha.InputStream", CLASS_InputStream, CFLAG_InputStream, STRUCT_InputStream, CLASS_Object, 0, 9, 0, knh_InputStream_fdefault},
 	{"konoha.OutputStream", CLASS_OutputStream, CFLAG_OutputStream, STRUCT_OutputStream, CLASS_Object, 0, 17, 0, knh_OutputStream_fdefault},
 	{"konoha.Socket", CLASS_Socket, CFLAG_Socket, STRUCT_Socket, CLASS_Object, 0, 5, 0, knh_Socket_fdefault},
@@ -651,10 +611,10 @@ static knh_ClassData_t ClassData[] = {
 	{"konoha.ResultSet", CLASS_ResultSet, CFLAG_ResultSet, STRUCT_ResultSet, CLASS_Object, 0, 9, 0, knh_ResultSet_fdefault},
 	{"konoha.Exception", CLASS_Exception, CFLAG_Exception, STRUCT_Exception, CLASS_Object, 0, 9, 0, knh_Exception_fdefault},
 	{"konoha.ExceptionHandler", CLASS_ExceptionHandler, CFLAG_ExceptionHandler, STRUCT_ExceptionHandler, CLASS_Object, 0, 0, 0, knh_ExceptionHandler_fdefault},
-	{"konoha.Script", CLASS_Script, CFLAG_Script, STRUCT_Script, CLASS_Object, 0, 8, 0, knh_Script_fdefault},
+	{"konoha.Script", CLASS_Script, CFLAG_Script, STRUCT_Script, CLASS_Object, 0, 7, 0, knh_Script_fdefault},
 	{"konoha.NameSpace", CLASS_NameSpace, CFLAG_NameSpace, STRUCT_NameSpace, CLASS_Object, 0, 0, 0, knh_NameSpace_fdefault},
-	{"konoha.System", CLASS_System, CFLAG_System, STRUCT_System, CLASS_Object, 0, 16, 0, knh_System_fdefault},
-	{"konoha.Context", CLASS_Context, CFLAG_Context, STRUCT_Context, CLASS_Object, 0, 16, 0, knh_Context_fdefault},
+	{"konoha.System", CLASS_System, CFLAG_System, STRUCT_System, CLASS_Object, 0, 18, 0, knh_System_fdefault},
+	{"konoha.Context", CLASS_Context, CFLAG_Context, STRUCT_Context, CLASS_Object, 0, 15, 0, knh_Context_fdefault},
 	{"konoha.Token", CLASS_Token, CFLAG_Token, STRUCT_Token, CLASS_Object, 0, 3, 0, knh_Token_fdefault},
 	{"konoha.Stmt", CLASS_Stmt, CFLAG_Stmt, STRUCT_Stmt, CLASS_Object, 0, 2, 0, knh_Stmt_fdefault},
 	{"konoha.Asm", CLASS_Asm, CFLAG_Asm, STRUCT_Asm, CLASS_Object, 0, 0, 0, knh_Asm_fdefault},
@@ -663,11 +623,11 @@ static knh_ClassData_t ClassData[] = {
 };
 
 static knh_GenericsData_t GenericsData[] = {
-	{"String[]", ACLASS_String, CLASS_Array, CLASS_String, CLASS_Nue},
-	{"String..", ICLASS_String, CLASS_Iterator, CLASS_String, CLASS_Nue},
-	{"Any1..", ICLASS_Any1, CLASS_Iterator, CLASS_Any1, CLASS_Nue},
-	{"Int..", ICLASS_Int, CLASS_Iterator, CLASS_Int, CLASS_Nue},
-	{"This..", ICLASS_This, CLASS_Iterator, CLASS_This, CLASS_Nue},
+	{"String..", ICLASS_String, CLASS_Iterator, CLASS_String, CLASS_unknown},
+	{"Any1..", ICLASS_Any1, CLASS_Iterator, CLASS_Any1, CLASS_unknown},
+	{"Int..", ICLASS_Int, CLASS_Iterator, CLASS_Int, CLASS_unknown},
+	{"This..", ICLASS_This, CLASS_Iterator, CLASS_This, CLASS_unknown},
+	{"String[]", ACLASS_String, CLASS_Array, CLASS_String, CLASS_unknown},
 	{NULL}
 };
 
@@ -709,15 +669,12 @@ static knh_FieldNameData_t FieldNameData[] = {
 	{"autoFlush", FIELDN_autoFlush},
 	{"b", FIELDN_b},
 	{"bag", FIELDN_bag},
-	{"base", FIELDN_base},
 	{"bits", FIELDN_bits},
 	{"buf", FIELDN_buf},
 	{"byte", FIELDN_byte},
 	{"bytes", FIELDN_bytes},
 	{"c", FIELDN_c},
-	{"callcc", FIELDN_callcc},
 	{"ch", FIELDN_ch},
-	{"changeChannel", FIELDN_changeChannel},
 	{"char", FIELDN_char},
 	{"class", FIELDN_class},
 	{"clear", FIELDN_clear},
@@ -731,6 +688,8 @@ static knh_FieldNameData_t FieldNameData[] = {
 	{"defined", FIELDN_defined},
 	{"delegate", FIELDN_delegate},
 	{"delim", FIELDN_delim},
+	{"dirname", FIELDN_dirname},
+	{"domain", FIELDN_domain},
 	{"dump", FIELDN_dump},
 	{"e", FIELDN_e},
 	{"empty", FIELDN_empty},
@@ -745,21 +704,19 @@ static knh_FieldNameData_t FieldNameData[] = {
 	{"exec", FIELDN_exec},
 	{"exit", FIELDN_exit},
 	{"f", FIELDN_f},
-	{"file", FIELDN_file},
+	{"filename", FIELDN_filename},
 	{"float", FIELDN_float},
 	{"floatToIntBits", FIELDN_floatToIntBits},
 	{"flush", FIELDN_flush},
 	{"format", FIELDN_format},
 	{"formatted", FIELDN_formatted},
-	{"funcname", FIELDN_funcname},
+	{"func", FIELDN_func},
 	{"g", FIELDN_g},
-	{"gCMarked", FIELDN_gCMarked},
 	{"gc", FIELDN_gc},
 	{"h", FIELDN_h},
 	{"hasDir", FIELDN_hasDir},
 	{"hasFile", FIELDN_hasFile},
-	{"hasMetaData", FIELDN_hasMetaData},
-	{"hook", FIELDN_hook},
+	{"hasLib", FIELDN_hasLib},
 	{"host", FIELDN_host},
 	{"i", FIELDN_i},
 	{"in", FIELDN_in},
@@ -773,7 +730,6 @@ static knh_FieldNameData_t FieldNameData[] = {
 	{"intToFloatBits", FIELDN_intToFloatBits},
 	{"invoke", FIELDN_invoke},
 	{"isAbstract", FIELDN_isAbstract},
-	{"isAspect", FIELDN_isAspect},
 	{"isAutoFlush", FIELDN_isAutoFlush},
 	{"isClosed", FIELDN_isClosed},
 	{"isConst", FIELDN_isConst},
@@ -782,8 +738,8 @@ static knh_FieldNameData_t FieldNameData[] = {
 	{"isFatal", FIELDN_isFatal},
 	{"isFinal", FIELDN_isFinal},
 	{"isFormatted", FIELDN_isFormatted},
-	{"isGCMarked", FIELDN_isGCMarked},
 	{"isGenerated", FIELDN_isGenerated},
+	{"isICast", FIELDN_isICast},
 	{"isIgnoreCase", FIELDN_isIgnoreCase},
 	{"isImmutable", FIELDN_isImmutable},
 	{"isInteractive", FIELDN_isInteractive},
@@ -795,22 +751,20 @@ static knh_FieldNameData_t FieldNameData[] = {
 	{"isModified", FIELDN_isModified},
 	{"isNotNull", FIELDN_isNotNull},
 	{"isNull", FIELDN_isNull},
-	{"isNullBase", FIELDN_isNullBase},
 	{"isPartial", FIELDN_isPartial},
 	{"isPrivate", FIELDN_isPrivate},
 	{"isPublic", FIELDN_isPublic},
 	{"isRelease", FIELDN_isRelease},
+	{"isShared", FIELDN_isShared},
 	{"isSignificant", FIELDN_isSignificant},
-	{"isStatement", FIELDN_isStatement},
 	{"isStatic", FIELDN_isStatic},
+	{"isStmt", FIELDN_isStmt},
 	{"isStoringBuffer", FIELDN_isStoringBuffer},
 	{"isStrict", FIELDN_isStrict},
-	{"isSynchronized", FIELDN_isSynchronized},
 	{"isSynonym", FIELDN_isSynonym},
 	{"isTemporal", FIELDN_isTemporal},
 	{"isTotal", FIELDN_isTotal},
-	{"isTrusted", FIELDN_isTrusted},
-	{"isUnsigned", FIELDN_isUnsigned},
+	{"isUndefined", FIELDN_isUndefined},
 	{"isVarArgs", FIELDN_isVarArgs},
 	{"isVerbose", FIELDN_isVerbose},
 	{"isVirtual", FIELDN_isVirtual},
@@ -823,17 +777,20 @@ static knh_FieldNameData_t FieldNameData[] = {
 	{"lastIndexOf", FIELDN_lastIndexOf},
 	{"lastIndexOf:IgnoreCase", FIELDN_lastIndexOf__IgnoreCase},
 	{"len", FIELDN_len},
+	{"lib", FIELDN_lib},
 	{"likely", FIELDN_likely},
-	{"linenum", FIELDN_linenum},
+	{"lines", FIELDN_lines},
+	{"listDir", FIELDN_listDir},
 	{"listProperties", FIELDN_listProperties},
 	{"m", FIELDN_m},
 	{"main", FIELDN_main},
 	{"man", FIELDN_man},
 	{"match", FIELDN_match},
+	{"memcpy", FIELDN_memcpy},
+	{"mkdir", FIELDN_mkdir},
 	{"mode", FIELDN_mode},
 	{"modified", FIELDN_modified},
 	{"msg", FIELDN_msg},
-	{"mtd", FIELDN_mtd},
 	{"n", FIELDN_n},
 	{"name", FIELDN_name},
 	{"new", FIELDN_new},
@@ -841,9 +798,12 @@ static knh_FieldNameData_t FieldNameData[] = {
 	{"new:array2D", FIELDN_new__array2D},
 	{"new:array3D", FIELDN_new__array3D},
 	{"new:init", FIELDN_new__init},
+	{"newname", FIELDN_newname},
 	{"next", FIELDN_next},
 	{"o", FIELDN_o},
+	{"off", FIELDN_off},
 	{"offset", FIELDN_offset},
+	{"oldname", FIELDN_oldname},
 	{"opAdd", FIELDN_opAdd},
 	{"opAddr", FIELDN_opAddr},
 	{"opDiv", FIELDN_opDiv},
@@ -902,11 +862,13 @@ static knh_FieldNameData_t FieldNameData[] = {
 	{"readLine", FIELDN_readLine},
 	{"register", FIELDN_register},
 	{"remove", FIELDN_remove},
+	{"rename", FIELDN_rename},
 	{"replace", FIELDN_replace},
 	{"reverse", FIELDN_reverse},
 	{"s", FIELDN_s},
 	{"script", FIELDN_script},
 	{"seed", FIELDN_seed},
+	{"shared", FIELDN_shared},
 	{"shuffle", FIELDN_shuffle},
 	{"size", FIELDN_size},
 	{"sort", FIELDN_sort},
@@ -915,7 +877,6 @@ static knh_FieldNameData_t FieldNameData[] = {
 	{"startsWith", FIELDN_startsWith},
 	{"startsWith:IgnoreCase", FIELDN_startsWith__IgnoreCase},
 	{"status", FIELDN_status},
-	{"stdErr", FIELDN_stdErr},
 	{"storingBuffer", FIELDN_storingBuffer},
 	{"strict", FIELDN_strict},
 	{"string", FIELDN_string},
@@ -931,6 +892,7 @@ static knh_FieldNameData_t FieldNameData[] = {
 	{"typeof", FIELDN_typeof},
 	{"u", FIELDN_u},
 	{"unlikely", FIELDN_unlikely},
+	{"unlink", FIELDN_unlink},
 	{"urn", FIELDN_urn},
 	{"v", FIELDN_v},
 	{"value", FIELDN_value},
@@ -992,6 +954,7 @@ static knh_MethodFieldData_t MethodFieldData[] = {
 	{MFN_void_nInt_nInt_nInt_nFloat, 4+ 0, TYPE_void, NNTYPE_Int, FIELDN_x, NNTYPE_Int, FIELDN_y, NNTYPE_Int, FIELDN_z, NNTYPE_Float, FIELDN_v},
 	{MFN_nBytes_Int, 1+ 0, NNTYPE_Bytes, TYPE_Int, FIELDN_i},
 	{MFN_void_nBytes_Int_Int, 3+ 0, TYPE_void, NNTYPE_Bytes, FIELDN_b, TYPE_Int, FIELDN_n, TYPE_Int, FIELDN_l},
+	{MFN_void_Int_nBytes_Int_Int, 4+ 0, TYPE_void, TYPE_Int, FIELDN_o, NNTYPE_Bytes, FIELDN_s, TYPE_Int, FIELDN_o, TYPE_Int, FIELDN_l},
 	{MFN_nConnection_nString, 1+ 0, NNTYPE_Connection, NNTYPE_String, FIELDN_u},
 	{MFN_nResultSet_nString, 1+ 0, NNTYPE_ResultSet, NNTYPE_String, FIELDN_q},
 	{MFN_void_nString, 1+ 0, TYPE_void, NNTYPE_String, FIELDN_q},
@@ -1011,6 +974,8 @@ static knh_MethodFieldData_t MethodFieldData[] = {
 	{MFN_void_String_, 1+ 0, TYPE_void, TYPE_String, FIELDN_v},
 	{MFN_Any2_Any1, 1+ 0, TYPE_Any2, TYPE_Any1, FIELDN_k},
 	{MFN_void_nAny1_Any2, 2+ 0, TYPE_void, NNTYPE_Any1, FIELDN_k, TYPE_Any2, FIELDN_v},
+	{MFN_void_String, 1+ 0, TYPE_void, TYPE_String, FIELDN_s},
+	{MFN_String_String, 1+ 0, TYPE_String, TYPE_String, FIELDN_p},
 	{MFN_nBoolean, 0+ 0, NNTYPE_Boolean},
 	{MFN_nBoolean_nInt, 1+ 0, NNTYPE_Boolean, NNTYPE_Int, FIELDN_v},
 	{MFN_nInt_nInt_, 1+ 0, NNTYPE_Int, NNTYPE_Int, FIELDN_v},
@@ -1025,6 +990,7 @@ static knh_MethodFieldData_t MethodFieldData[] = {
 	{MFN_nBoolean_nClass, 1+ 0, NNTYPE_Boolean, NNTYPE_Class, FIELDN_c},
 	{MFN_nClass, 0+ 0, NNTYPE_Class},
 	{MFN_iThis, 0+ 0, NNITYPE_This},
+	{MFN_nArray, 0+ 0, NNTYPE_Array},
 	{MFN_InputStream_nString_String, 2+ 0, TYPE_InputStream, NNTYPE_String, FIELDN_u, TYPE_String, FIELDN_m},
 	{MFN_nInt_nBytes_Int_Int, 3+ 0, NNTYPE_Int, NNTYPE_Bytes, FIELDN_b, TYPE_Int, FIELDN_o, TYPE_Int, FIELDN_l},
 	{MFN_nString, 0+ 0, NNTYPE_String},
@@ -1033,7 +999,6 @@ static knh_MethodFieldData_t MethodFieldData[] = {
 	{MFN_nSocket_nString_nInt, 2+ 0, NNTYPE_Socket, NNTYPE_String, FIELDN_h, NNTYPE_Int, FIELDN_p},
 	{MFN_nInputStream, 0+ 0, NNTYPE_InputStream},
 	{MFN_nOutputStream, 0+ 0, NNTYPE_OutputStream},
-	{MFN_void_String, 1+ 0, TYPE_void, TYPE_String, FIELDN_e},
 	{MFN_nString_nBytes_String, 2+ 0, NNTYPE_String, NNTYPE_Bytes, FIELDN_b, TYPE_String, FIELDN_e},
 	{MFN_nBytes_String, 1+ 0, NNTYPE_Bytes, TYPE_String, FIELDN_e},
 	{MFN_nString_Any, 1+ 0, NNTYPE_String, TYPE_Any, FIELDN_v},
@@ -1046,20 +1011,21 @@ static knh_MethodFieldData_t MethodFieldData[] = {
 	{MFN_Regex_nString_String, 2+ 0, TYPE_Regex, NNTYPE_String, FIELDN_p, TYPE_String, FIELDN_o},
 	{MFN_nBoolean_nRegex, 1+ 0, NNTYPE_Boolean, NNTYPE_Regex, FIELDN_r},
 	{MFN_aString_Regex, 1+ 0, ATYPE_String, TYPE_Regex, FIELDN_p},
-	{MFN_void_InputStream, 1+ 0, TYPE_void, TYPE_InputStream, FIELDN_i},
-	{MFN_void_OutputStream, 1+ 0, TYPE_void, TYPE_OutputStream, FIELDN_o},
+	{MFN_nBoolean_nString_String, 2+ 0, NNTYPE_Boolean, NNTYPE_String, FIELDN_l, TYPE_String, FIELDN_f},
 	{MFN_void_Int, 1+ 0, TYPE_void, TYPE_Int, FIELDN_s},
+	{MFN_aString_String, 1+ 0, ATYPE_String, TYPE_String, FIELDN_d},
+	{MFN_nBoolean_String_String, 2+ 0, NNTYPE_Boolean, TYPE_String, FIELDN_o, TYPE_String, FIELDN_n},
 	{MFN_Any_nString, 1+ 0, TYPE_Any, NNTYPE_String, FIELDN_k},
 	{MFN_void_nString_Any, 2+ 0, TYPE_void, NNTYPE_String, FIELDN_k, TYPE_Any, FIELDN_v},
 	{MFN_naString_String, 1+ 0, NNATYPE_String, TYPE_String, FIELDN_k},
+	{MFN_void_InputStream, 1+ 0, TYPE_void, TYPE_InputStream, FIELDN_i},
+	{MFN_void_OutputStream, 1+ 0, TYPE_void, TYPE_OutputStream, FIELDN_i},
 	{MFN_nException_String_Any, 2+ 0, NNTYPE_Exception, TYPE_String, FIELDN_m, TYPE_Any, FIELDN_b},
-	{MFN_nThis_nAny_nMethod, 2+ 0, NNTYPE_This, NNTYPE_Any, FIELDN_b, NNTYPE_Method, FIELDN_m},
 	{MFN_Any0_Any1_Any2_Any3, 3+ 0, TYPE_Any0, TYPE_Any1, FIELDN_x, TYPE_Any2, FIELDN_y, TYPE_Any3, FIELDN_z},
 	{MFN_nException_String_String_Object, 3+ 0, NNTYPE_Exception, TYPE_String, FIELDN_e, TYPE_String, FIELDN_m, TYPE_Object, FIELDN_b},
 	{MFN_String, 0+ 0, TYPE_String},
 	{MFN_nThis, 0+ 0, NNTYPE_This},
-	{MFN_void_String_String_Int, 3+ 0, TYPE_void, TYPE_String, FIELDN_s, TYPE_String, FIELDN_f, TYPE_Int, FIELDN_l},
-	{MFN_String_String, 1+ 0, TYPE_String, TYPE_String, FIELDN_p},
+	{MFN_aString, 0+ 0, ATYPE_String},
 	{MFN_Boolean_nClass, 1+ 0, TYPE_Boolean, NNTYPE_Class, FIELDN_c},
 	{MFN_void_Boolean, 1+ 0, TYPE_void, TYPE_Boolean, FIELDN_b},
 	{-1, 0}
@@ -1146,6 +1112,7 @@ static knh_MethodData_t MethodData[] = {
 	{knh__Bytes_get, 0, CLASS_Bytes, METHODN_get, 0, MFN_nInt_nInt, (void*)NULL},
 	{knh__Bytes_set, 0, CLASS_Bytes, METHODN_set, 0, MFN_void_nInt_nInt, (void*)NULL},
 	{knh__Bytes_opFill, 0, CLASS_Bytes, METHODN_opFill, 0, MFN_void_nInt, (void*)NULL},
+	{knh__Bytes_memcpy, 0, CLASS_Bytes, METHODN_memcpy, 0, MFN_void_Int_nBytes_Int_Int, (void*)NULL},
 	{knh_fmethod_movableText, 0, CLASS_Bytes, METHODN__k, 0, MFN_void_OutputStream_Any, (void*)knh_Bytes__k},
 	{knh_fmethod_movableText, 0, CLASS_Bytes, METHODN__dump, 0, MFN_void_OutputStream_Any, (void*)knh_Bytes__dump},
 	{knh__Connection_new, 0, CLASS_Connection, METHODN_new, 0, MFN_nConnection_nString, (void*)NULL},
@@ -1189,6 +1156,10 @@ static knh_MethodData_t MethodData[] = {
 	{knh_fmethod_movableText, 0, CLASS_HashMap, METHODN__k, 0, MFN_void_OutputStream_Any, (void*)knh_HashMap__k},
 	{knh_fmethod_movableText, 0, CLASS_HashMap, METHODN__dump, 0, MFN_void_OutputStream_Any, (void*)knh_HashMap__dump},
 	{knh__HashMap_opItr, 0, CLASS_HashMap, METHODN_opItr, 0, MFN_iAny1, (void*)NULL},
+	{knh__Script_isStmt, 0, CLASS_Script, METHODN_isStmt, 0, MFN_nBoolean_nString, (void*)NULL},
+	{knh__Script_eval, 0, CLASS_Script, METHODN_eval, 0, MFN_void_String, (void*)NULL},
+	{knh__Script_readLine, 0, CLASS_Script, METHODN_readLine, 0, MFN_String_String, (void*)NULL},
+	{knh__Script_addHistory, 0, CLASS_Script, METHODN_addHistory, 0, MFN_void_nString, (void*)NULL},
 	{knh__Boolean_opNot, KNH_FLAG_MF_CONST, CLASS_Boolean, METHODN_opNot, 0, MFN_nBoolean, (void*)NULL},
 	{knh__Int_opAdd, KNH_FLAG_MF_CONST, CLASS_Int, METHODN_opAdd, 0, MFN_nInt_nInt, (void*)NULL},
 	{knh__Int_opNeg, KNH_FLAG_MF_CONST, CLASS_Int, METHODN_opNeg, 0, MFN_nInt, (void*)NULL},
@@ -1255,7 +1226,7 @@ static knh_MethodData_t MethodData[] = {
 	{knh__Object_opIs, KNH_FLAG_MF_CONST, CLASS_Object, METHODN_opIs, 0, MFN_nBoolean_Any, (void*)NULL},
 	{knh__Object_opIsa, KNH_FLAG_MF_CONST, CLASS_Object, METHODN_opIsa, 0, MFN_nBoolean_nClass, (void*)NULL},
 	{knh__Object_opTo, KNH_FLAG_MF_CONST, CLASS_Object, METHODN_opTo, 0, MFN_nBoolean_nClass, (void*)NULL},
-	{knh__Object_getClass, KNH_FLAG_MF_CONST, CLASS_Object, METHODN_getClass, 0, MFN_nClass, (void*)NULL},
+	{knh__Object_getClass, KNH_FLAG_MF_CONST|KNH_FLAG_MF_NULLBASE, CLASS_Object, METHODN_getClass, 0, MFN_nClass, (void*)NULL},
 	{knh__Object_isNull, KNH_FLAG_MF_CONST|KNH_FLAG_MF_NULLBASE, CLASS_Object, METHODN_isNull, 0, MFN_nBoolean, (void*)NULL},
 	{knh__Object_isNotNull, KNH_FLAG_MF_CONST|KNH_FLAG_MF_NULLBASE, CLASS_Object, METHODN_isNotNull, 0, MFN_nBoolean, (void*)NULL},
 	{knh__Object_opItr, 0, CLASS_Object, METHODN_opItr, 0, MFN_iThis, (void*)NULL},
@@ -1264,7 +1235,8 @@ static knh_MethodData_t MethodData[] = {
 	{knh_fmethod_movableText, 0, CLASS_Object, METHODN__k, 0, MFN_void_OutputStream_Any, (void*)knh_Object__k},
 	{knh_fmethod_movableText, 0, CLASS_Object, METHODN__dump, 0, MFN_void_OutputStream_Any, (void*)knh_Object__dump},
 	{knh_fmethod_movableText, 0, CLASS_Object, METHODN__empty, 0, MFN_void_OutputStream_Any, (void*)knh_Object__empty},
-	{knh_fmethod_movableText, 0, CLASS_Nue, METHODN__k, 0, MFN_void_OutputStream_Any, (void*)knh_Nue__k},
+	{knh_fmethod_movableText, 0, CLASS_Any, METHODN__k, 0, MFN_void_OutputStream_Any, (void*)knh_Any__k},
+	{knh__Class_domain, 0, CLASS_Class, METHODN_domain, 0, MFN_nArray, (void*)NULL},
 	{knh_fmethod_movableText, 0, CLASS_Class, METHODN__s, 0, MFN_void_OutputStream_Any, (void*)knh_Class__s},
 	{knh_fmethod_movableText, 0, CLASS_Class, METHODN__k, 0, MFN_void_OutputStream_Any, (void*)knh_Class__k},
 	{knh_fmethod_movableText, 0, CLASS_Class, METHODN__dump, 0, MFN_void_OutputStream_Any, (void*)knh_Class__dump},
@@ -1334,15 +1306,17 @@ static knh_MethodData_t MethodData[] = {
 	{knh__String_match, KNH_FLAG_MF_CONST, CLASS_String, METHODN_match, 0, MFN_aString_Regex, (void*)NULL},
 	{knh_fmethod_movableText, 0, CLASS_Regex, METHODN__s, 0, MFN_void_OutputStream_Any, (void*)knh_Regex__s},
 	{knh__System_getIn, KNH_FLAG_MF_STATIC, CLASS_System, METHODN_getIn, 0, MFN_nInputStream, (void*)NULL},
-	{knh__System_setIn, KNH_FLAG_MF_STATIC, CLASS_System, METHODN_setIn, 0, MFN_void_InputStream, (void*)NULL},
 	{knh__System_getOut, KNH_FLAG_MF_STATIC, CLASS_System, METHODN_getOut, 0, MFN_nOutputStream, (void*)NULL},
-	{knh__System_setOut, KNH_FLAG_MF_STATIC, CLASS_System, METHODN_setOut, 0, MFN_void_OutputStream, (void*)NULL},
 	{knh__System_getErr, KNH_FLAG_MF_STATIC, CLASS_System, METHODN_getErr, 0, MFN_nOutputStream, (void*)NULL},
-	{knh__System_setErr, KNH_FLAG_MF_STATIC, CLASS_System, METHODN_setStdErr, 0, MFN_void_OutputStream, (void*)NULL},
+	{knh__System_hasLib, KNH_FLAG_MF_STATIC, CLASS_System, METHODN_hasLib, 0, MFN_nBoolean_nString_String, (void*)NULL},
 	{knh__System_setRandomSeed, KNH_FLAG_MF_STATIC, CLASS_System, METHODN_setRandomSeed, 0, MFN_void_Int, (void*)NULL},
 	{knh__System_exit, KNH_FLAG_MF_STATIC, CLASS_System, METHODN_exit, 0, MFN_void_Int, (void*)NULL},
+	{knh__System_listDir, 0, CLASS_System, METHODN_listDir, 0, MFN_aString_String, (void*)NULL},
 	{knh__System_hasDir, 0, CLASS_System, METHODN_hasDir, 0, MFN_nBoolean_nString, (void*)NULL},
 	{knh__System_hasFile, 0, CLASS_System, METHODN_hasFile, 0, MFN_nBoolean_nString, (void*)NULL},
+	{knh__System_mkdir, 0, CLASS_System, METHODN_mkdir, 0, MFN_nBoolean_nString, (void*)NULL},
+	{knh__System_unlink, 0, CLASS_System, METHODN_unlink, 0, MFN_nBoolean_nString, (void*)NULL},
+	{knh__System_rename, 0, CLASS_System, METHODN_rename, 0, MFN_nBoolean_String_String, (void*)NULL},
 	{knh__Context_getProperty, 0, CLASS_Context, METHODN_getProperty, 0, MFN_Any_nString, (void*)NULL},
 	{knh__Context_setProperty, 0, CLASS_Context, METHODN_setProperty, 0, MFN_void_nString_Any, (void*)NULL},
 	{knh__Context_listProperties, 0, CLASS_Context, METHODN_listProperties, 0, MFN_naString_String, (void*)NULL},
@@ -1359,9 +1333,7 @@ static knh_MethodData_t MethodData[] = {
 	{knh_fmethod_movableText, 0, CLASS_System, METHODN__dump, 0, MFN_void_OutputStream_Any, (void*)knh_System__dump},
 	{knh__Context_setEncoding, 0, CLASS_Context, METHODN_setEncoding, 0, MFN_void_String, (void*)NULL},
 	{knh_fmethod_movableText, 0, CLASS_ClassMap, METHODN__dump, 0, MFN_void_OutputStream_Any, (void*)knh_ClassMap__dump},
-	{knh__Closure_new, 0, CLASS_Closure, METHODN_new, 0, MFN_nThis_nAny_nMethod, (void*)NULL},
 	{knh__Closure_invoke, 0, CLASS_Closure, METHODN_invoke, 0, MFN_Any0_Any1_Any2_Any3, (void*)NULL},
-	{knh__Closure_callcc, 0, CLASS_Closure, METHODN_callcc, 0, MFN_Any0_Any1_Any2_Any3, (void*)NULL},
 	{knh__Connection_close, 0, CLASS_Connection, METHODN_close, 0, MFN_void, (void*)NULL},
 	{knh__DictIdx_size, 0, CLASS_DictIdx, METHODN_size, 0, MFN_nInt, (void*)NULL},
 	{knh__DictIdx_index, 0, CLASS_DictIdx, METHODN_index, 0, MFN_nInt_nString, (void*)NULL},
@@ -1394,26 +1366,20 @@ static knh_MethodData_t MethodData[] = {
 	{knh_fmethod_movableText, 0, CLASS_Token, METHODN__s, 0, MFN_void_OutputStream_Any, (void*)knh_Token__s},
 	{knh_fmethod_movableText, 0, CLASS_Token, METHODN__k, 0, MFN_void_OutputStream_Any, (void*)knh_Token__k},
 	{knh_fmethod_movableText, 0, CLASS_Token, METHODN__dump, 0, MFN_void_OutputStream_Any, (void*)knh_Token__dump},
-	{knh__Script_changeChannel, 0, CLASS_Script, METHODN_changeChannel, 0, MFN_void_nInt, (void*)NULL},
-	{knh__Script_hook, 0, CLASS_Script, METHODN_hook, 0, MFN_void_nString, (void*)NULL},
-	{knh__Script_eval, 0, CLASS_Script, METHODN_eval, 0, MFN_void_String_String_Int, (void*)NULL},
-	{knh__Script_isStatement, 0, CLASS_Script, METHODN_isStatement, 0, MFN_nBoolean_nString, (void*)NULL},
-	{knh__Script_readLine, 0, CLASS_Script, METHODN_readLine, 0, MFN_String_String, (void*)NULL},
-	{knh__Script_addHistory, 0, CLASS_Script, METHODN_addHistory, 0, MFN_void_nString, (void*)NULL},
+	{knh__Script_getLines, 0, CLASS_Script, METHODN_getLines, 0, MFN_aString, (void*)NULL},
 	{knh__System_gc, KNH_FLAG_MF_STATIC, CLASS_System, METHODN_gc, 0, MFN_void, (void*)NULL},
 	{knh__System_stackdump, 0, CLASS_System, METHODN_stackdump, 0, MFN_void, (void*)NULL},
 	{knh__Object_opInstanceof, 0, CLASS_Object, METHODN_opInstanceof, 0, MFN_Boolean_nClass, (void*)NULL},
 	{knh__Object_isRelease, 0, CLASS_Object, METHODN_isRelease, 0, MFN_nBoolean, (void*)NULL},
 	{knh__Object_isDebug, 0, CLASS_Object, METHODN_isDebug, 0, MFN_nBoolean, (void*)NULL},
 	{knh__Object_isImmutable, 0, CLASS_Object, METHODN_isImmutable, 0, MFN_nBoolean, (void*)NULL},
-	{knh__Object_hasMetaData, 0, CLASS_Object, METHODN_hasMetaData, 0, MFN_nBoolean, (void*)NULL},
-	{knh__Object_isSynchronized, 0, CLASS_Object, METHODN_isSynchronized, 0, MFN_nBoolean, (void*)NULL},
+	{knh__Object_isUndefined, 0, CLASS_Object, METHODN_isUndefined, 0, MFN_nBoolean, (void*)NULL},
 	{knh__Object_isModified, 0, CLASS_Object, METHODN_isModified, 0, MFN_nBoolean, (void*)NULL},
 	{knh__Object_setModified, 0, CLASS_Object, METHODN_setModified, 0, MFN_void_Boolean, (void*)NULL},
-	{knh__Object_isGCMarked, 0, CLASS_Object, METHODN_isGCMarked, 0, MFN_nBoolean, (void*)NULL},
-	{knh__Object_setGCMarked, 0, CLASS_Object, METHODN_setGCMarked, 0, MFN_void_Boolean, (void*)NULL},
 	{knh__Object_isFormatted, 0, CLASS_Object, METHODN_isFormatted, 0, MFN_nBoolean, (void*)NULL},
 	{knh__Object_setFormatted, 0, CLASS_Object, METHODN_setFormatted, 0, MFN_void_Boolean, (void*)NULL},
+	{knh__Object_isShared, 0, CLASS_Object, METHODN_isShared, 0, MFN_nBoolean, (void*)NULL},
+	{knh__Object_setShared, 0, CLASS_Object, METHODN_setShared, 0, MFN_void_Boolean, (void*)NULL},
 	{knh__DictMap_isIgnoreCase, 0, CLASS_DictMap, METHODN_isIgnoreCase, 0, MFN_nBoolean, (void*)NULL},
 	{knh__DictSet_isIgnoreCase, 0, CLASS_DictSet, METHODN_isIgnoreCase, 0, MFN_nBoolean, (void*)NULL},
 	{knh__Class_isRelease, 0, CLASS_Class, METHODN_isRelease, 0, MFN_nBoolean, (void*)NULL},
@@ -1430,10 +1396,9 @@ static knh_MethodData_t MethodData[] = {
 	{knh__Method_isFinal, 0, CLASS_Method, METHODN_isFinal, 0, MFN_nBoolean, (void*)NULL},
 	{knh__Method_isConst, 0, CLASS_Method, METHODN_isConst, 0, MFN_nBoolean, (void*)NULL},
 	{knh__Method_isStatic, 0, CLASS_Method, METHODN_isStatic, 0, MFN_nBoolean, (void*)NULL},
-	{knh__Method_isNullBase, 0, CLASS_Method, METHODN_isNullBase, 0, MFN_nBoolean, (void*)NULL},
 	{knh__Method_isVarArgs, 0, CLASS_Method, METHODN_isVarArgs, 0, MFN_nBoolean, (void*)NULL},
-	{knh__Method_isAspect, 0, CLASS_Method, METHODN_isAspect, 0, MFN_nBoolean, (void*)NULL},
 	{knh__Mapper_isInterface, 0, CLASS_Mapper, METHODN_isInterface, 0, MFN_nBoolean, (void*)NULL},
+	{knh__Mapper_isICast, 0, CLASS_Mapper, METHODN_isICast, 0, MFN_nBoolean, (void*)NULL},
 	{knh__Mapper_isSignificant, 0, CLASS_Mapper, METHODN_isSignificant, 0, MFN_nBoolean, (void*)NULL},
 	{knh__Mapper_isFinal, 0, CLASS_Mapper, METHODN_isFinal, 0, MFN_nBoolean, (void*)NULL},
 	{knh__Mapper_isSynonym, 0, CLASS_Mapper, METHODN_isSynonym, 0, MFN_nBoolean, (void*)NULL},
@@ -1444,7 +1409,6 @@ static knh_MethodData_t MethodData[] = {
 	{knh__Mapper_isTemporal, 0, CLASS_Mapper, METHODN_isTemporal, 0, MFN_nBoolean, (void*)NULL},
 	{knh__Mapper_isLocal, 0, CLASS_Mapper, METHODN_isLocal, 0, MFN_nBoolean, (void*)NULL},
 	{knh__Mapper_isDerived, 0, CLASS_Mapper, METHODN_isDerived, 0, MFN_nBoolean, (void*)NULL},
-	{knh__IntUnit_isUnsigned, 0, CLASS_IntUnit, METHODN_isUnsigned, 0, MFN_nBoolean, (void*)NULL},
 	{knh__OutputStream_isAutoFlush, 0, CLASS_OutputStream, METHODN_isAutoFlush, 0, MFN_nBoolean, (void*)NULL},
 	{knh__OutputStream_setAutoFlush, 0, CLASS_OutputStream, METHODN_setAutoFlush, 0, MFN_void_Boolean, (void*)NULL},
 	{knh__OutputStream_isStoringBuffer, 0, CLASS_OutputStream, METHODN_isStoringBuffer, 0, MFN_nBoolean, (void*)NULL},
@@ -1458,7 +1422,6 @@ static knh_MethodData_t MethodData[] = {
 	{knh__Context_setDebug, 0, CLASS_Context, METHODN_setDebug, 0, MFN_void_Boolean, (void*)NULL},
 	{knh__Context_isVerbose, 0, CLASS_Context, METHODN_isVerbose, 0, MFN_nBoolean, (void*)NULL},
 	{knh__Context_setVerbose, 0, CLASS_Context, METHODN_setVerbose, 0, MFN_void_Boolean, (void*)NULL},
-	{knh__Context_isTrusted, 0, CLASS_Context, METHODN_isTrusted, 0, MFN_nBoolean, (void*)NULL},
 	{knh__Context_isInteractive, 0, CLASS_Context, METHODN_isInteractive, 0, MFN_nBoolean, (void*)NULL},
 	{NULL, 0}
 };
