@@ -38,7 +38,6 @@
 /* configure */
 
 #define KNH_USING_RCGC         1
-//#define KNH_USING_MARKGC       1
 #define KNH_USING_FASTMALLOC   1
 #define KNH_USING_UNBOXFIELD   1
 
@@ -51,20 +50,18 @@
 #else
 #define KONOHA_NAME     "konoha"
 #define KONOHA_MAJOR_VERSION  0
-#define KONOHA_MINER_VERSION  5
-#define KONOHA_PATCH_VERSION  2
-#define KONOHA_VERSION  "0.5.2"
+#define KONOHA_MIDDLE_VERSION 5
+#define KONOHA_MINER_VERSION  0
+#define KONOHA_VERSION  "0.5"
 #endif
 
-#define LIBKONOHA_VERSION "0.5"
-
-#define KONOHA_XCODE    "kurume"
-#define KONOHA_URL	"http://konoha.sourceforge.jp/"
+#define KONOHA_XCODE    "toguro"
+#define KONOHA_URL		"http://konoha.sourceforge.jp/"
 #define KONOHA_URLBASE  "kttp://konoha.sourceforge.jp/wiki/"
 
-#ifndef KONOHA_DIST
-#define KONOHA_DIST     "source"
-#endif
+#define KONOHA_BUILDID_LATEST  625
+#define KONOHA_BUILDID_TOOOLD  625
+#define KNH_CHECK_VERSION(id)  ((id) <= KONOHA_BUILDID_TOOOLD)
 
 /* ======================================================================== */
 /* COMMON */
@@ -135,9 +132,15 @@
 #ifdef __GNUC__
 	#define KONOHA_CC_VERSION "GCC " __VERSION__
 	#if defined(__i386__) || (defined(__x86_64__) && !defined(__amd64__))
-		#undef fastcall
-		#define fastcall __attribute__((fastcall))
-		#define KNH_CC_FASTCALL   fastcall
+		#if defined(KONOHA_ON_LKM)
+			#if (LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,25))
+				#define KNH_CC_FASTCALL   fastcall
+			#else
+				#define KNH_CC_FASTCALL   __attribute__((fastcall))
+			#endif
+		#else
+			#define KNH_CC_FASTCALL   __attribute__((fastcall))
+		#endif
 		#define likely(x)       __builtin_expect(!!(x), 1)
 		#define unlikely(x)     __builtin_expect(!!(x), 0)
 	#else
@@ -207,10 +210,6 @@
 #define KONOHA_PLATFORM KONOHA_OS "_32"
 #endif
 
-#ifdef __SSE2__
-#define KNH_USING_SSE2
-#endif
-
 /* ======================================================================== */
 /* [OPTIONS_DEPS] */
 
@@ -229,4 +228,5 @@
 /* ======================================================================== */
 /* ======================================================================== */
 
+
 #endif /*KONOHA_CONFIG_H_*/
