@@ -1,7 +1,7 @@
 /****************************************************************************
  * KONOHA COPYRIGHT, LICENSE NOTICE, AND DISCRIMER
  *
- * Copyright (c) 2005-2008, Kimio Kuramitsu <kimio at ynu.ac.jp>
+ * Copyright (c) 2005-2009, Kimio Kuramitsu <kimio at ynu.ac.jp>
  *           (c) 2008-      Konoha Software Foundation
  * All rights reserved.
  *
@@ -28,11 +28,16 @@
 #ifndef KONOHA_DEV_H_
 #define KONOHA_DEV_H_
 
+#ifdef KONOHA_ON_LKM
+#include"konoha_lkm.h"
+#else
 #include<stdio.h>
 #include<ctype.h>
 #include<assert.h>
-#include<stdarg.h>
 #include<setjmp.h>
+#endif
+
+#include<stdarg.h>
 
 #ifndef LIBKONOHA
 #ifdef HAVE_CONFIG_H
@@ -44,6 +49,10 @@
 
 #include"konoha_deps.h"
 
+#ifdef KONOHA_ON_TB
+#include"konoha_tb.h"
+#endif
+
 #include"gen/konoha_class_.h"
 #include"gen/konoha_knhapi_.h"
 
