@@ -16,10 +16,10 @@
 #define _(s)          s
 
 #define KNH_NOTICE(ctx, fmt, ...) \
-	konoha_says(ctx, KONOHA_NOTICE, fmt, ## __VA_ARGS__) \
+	knh_says(ctx, KONOHA_NOTICE, fmt, ## __VA_ARGS__) \
 
 #define KNH_WARNING(ctx, fmt, ...) \
-	konoha_says(ctx, KONOHA_WARNING, fmt, ## __VA_ARGS__) \
+	knh_says(ctx, KONOHA_WARNING, fmt, ## __VA_ARGS__) \
 
 #define KNH_EXIT(fmt, ...) {\
 		fprintf(stderr, "konoha: Emergency Exit at %s\n\t", __FUNCTION__); \
@@ -35,4 +35,12 @@
 
 /* ------------------------------------------------------------------------ */
 
+#ifdef KONOHA_MACROS
+#define T(msg)          new_StringX__T(ctx, CLASS_String, msg)
+#define S(msg)          new_String(ctx, B(msg), NULL)
+#endif
+
+/* ------------------------------------------------------------------------ */
+
+
 #endif /*KONOHA_MACRO_H_*/
