@@ -1,7 +1,7 @@
 /****************************************************************************
  * KONOHA COPYRIGHT, LICENSE NOTICE, AND DISCRIMER
  *
- * Copyright (c) 2005-2008, Kimio Kuramitsu <kimio at ynu.ac.jp>
+ * Copyright (c) 2005-2009, Kimio Kuramitsu <kimio at ynu.ac.jp>
  *           (c) 2008-      Konoha Software Foundation
  * All rights reserved.
  *
@@ -47,7 +47,7 @@ static METHOD knh__Object_new__init(Ctx *ctx, knh_sfp_t *sfp)
 	Object *o = sfp[0].o;
 //	knh_class_t cid = knh_Object_cid(o);
 //	knh_sfp_t *v = sfp + 1;
-//	int ac = knh_sfp_argc(ctx, v);
+//	int ac = knh_stack_argc(ctx, v);
 //	size_t i;
 //	for(i = 0; i < ac; i+= 2) {
 //		if(IS_bString(v[i].s)) {
@@ -190,7 +190,7 @@ static METHOD knh__Object_opTo(Ctx *ctx, knh_sfp_t *sfp)
 }
 
 /* ------------------------------------------------------------------------ */
-/* @method[CONST] Class! Object.getClass() */
+/* @method[CONST|NULLBASE] Class! Object.getClass() */
 
 static METHOD knh__Object_getClass(Ctx *ctx, knh_sfp_t *sfp)
 {
@@ -240,7 +240,7 @@ void knh_Object__s(Ctx *ctx, Object *b, OutputStream *w, String *m)
 		knh_write(ctx, w, STEXT("null"));
 	}
 	else {
-		knh_write__s(ctx,w, CLASSN(knh_Object_cid(b)));
+		knh_write_char(ctx,w, CLASSN(knh_Object_cid(b)));
 		knh_write(ctx,w, STEXT(":"));
 		knh_write__p(ctx,w, (void*)b);
 	}
@@ -270,9 +270,15 @@ static METHOD knh__Object__p(Ctx *ctx, knh_sfp_t *sfp)
 static
 void knh_Object__k(Ctx *ctx, Object *o, OutputStream *w, String *m)
 {
-	if(o->h.bcid == CLASS_Object) {
+	if(knh_Object_isUndefined(o)) {
+		knh_write(ctx, w, STEXT("undefined"));
+	}
+	else if(IS_NULL(o)) {
+		knh_write(ctx, w, STEXT("null"));
+	}
+	else if(o->h.bcid == CLASS_Object) {
 		size_t bsize = ctx->share->ClassTable[o->h.cid].bsize;
-		knh_write__s(ctx, w, CLASSN(o->h.cid));
+		knh_write_char(ctx, w, CLASSN(o->h.cid));
 		if(bsize > 0) {
 			size_t i;
 			Object **v = (Object**)o->ref;
@@ -286,18 +292,18 @@ void knh_Object__k(Ctx *ctx, Object *o, OutputStream *w, String *m)
 				}
 				knh_printf(ctx, w, "%s: ", FIELDN(cf->fn));
 #ifdef KNH_USING_UNBOXFIELD
-				if(cf->type == NNTYPE_Int) {
+				if(IS_ubxint(cf->type)) {
 					knh_int_t *data = (knh_int_t*)(v + i);
-					knh_write__i(ctx, w, data[0]);
+					knh_write_ifmt(ctx, w, KNH_INT_FMT, data[0]);
 					continue;
 				}
-				else if(cf->type == NNTYPE_Float) {
+				else if(IS_ubxfloat(cf->type)) {
 					knh_float_t *data = (knh_float_t*)(v + i);
-					knh_write__f(ctx, w, data[0]);
+					knh_write_ffmt(ctx, w, KNH_FLOAT_FMT, data[0]);
 					continue;
 				}
-				else if(cf->type == NNTYPE_Boolean) {
-					knh_boolean_t *data = (knh_boolean_t*)(v + i);
+				else if(IS_ubxboolean(cf->type)) {
+					knh_bool_t *data = (knh_bool_t*)(v + i);
 					if(data[0]) knh_write(ctx, w, STEXT("true"));
 					else knh_write(ctx, w, STEXT("false"));
 					continue;
@@ -328,7 +334,7 @@ void knh_Object__dump(Ctx *ctx, Object *b, OutputStream *w, String *m)
 //	}
 //	knh_intptr_t i, c = 0;
 //	knh_putc(ctx, w, '[');
-//	for(i = 0; i < ctx->share->ClassTable[cid].bsize; i++) {
+//	for(i = 0; i < ClassTable(cid).bsize; i++) {
 //		knh_cfield_t *cf = knh_Class_fieldAt(ctx, cid, i);
 //		if(cf->fn == FIELDN_NONAME || KNH_FLAG_IS(cf->flag, KNH_FLAG_CFF_HIDDEN)) continue;
 //		if(c > 0) {
@@ -362,23 +368,23 @@ static
 MAPPER knh_Object_String(Ctx *ctx, knh_sfp_t *sfp)
 {
 	Object *o = sfp[0].o;
-	Method *mtd = konoha_lookupFormatter(ctx, knh_Object_cid(o), METHODN__s);
+	Method *mtd = knh_lookupFormatter(ctx, knh_Object_cid(o), METHODN__s);
 	if(IS_NULL(mtd)) {
 		KNH_MAPPED(ctx, sfp, TS_EMPTY);
 	}
-	knh_cwb_t cwb = new_cwb(ctx);
+	knh_cwb_t cwbbuf, *cwb = knh_cwb_open(ctx, &cwbbuf);
 	KNH_MOV(ctx, sfp[2].o, o);
-	KNH_MOV(ctx, sfp[3].o, cwb.w);
+	KNH_MOV(ctx, sfp[3].o, cwb->w);
 	KNH_MOV(ctx, sfp[4].o, KNH_NULL);
 	KNH_SCALL(ctx, sfp, 1, mtd, 2);
 	KNH_MAPPED(ctx, sfp, new_String__cwb(ctx, cwb));
 }
 
 /* ------------------------------------------------------------------------ */
-/* @method void Nue.%k(OutputStream w, String m) */
+/* @method void Any.%k(OutputStream w, String m) */
 
 static
-void knh_Nue__k(Ctx *ctx, Nue *b, OutputStream *w, String *m)
+void knh_Any__k(Ctx *ctx, knh_Any_t *o, OutputStream *w, String *m)
 {
 	knh_write(ctx, w, knh_String_tobytes(TS_null));
 }
@@ -386,6 +392,15 @@ void knh_Nue__k(Ctx *ctx, Nue *b, OutputStream *w, String *m)
 /* ======================================================================== */
 /* [Class] */
 
+/* @method Array! Class.domain() */
+
+static
+METHOD knh__Class_domain(Ctx *ctx, knh_sfp_t *sfp)
+{
+	KNH_RETURN(ctx, sfp, knh_getClassDomain(ctx, (sfp[0].c)->cid));
+}
+
+/* ------------------------------------------------------------------------ */
 /* @method void Class.%s(OutputStream w, String m) */
 
 static
@@ -400,7 +415,7 @@ void knh_Class__s(Ctx *ctx, Class *o, OutputStream *w, String *m)
 static
 void knh_Class__k(Ctx *ctx, Class *o, OutputStream *w, String *m)
 {
-	knh_write_type(ctx, w, o->type);
+	knh_write_cid(ctx, w, o->cid);
 }
 
 /* ------------------------------------------------------------------------ */
@@ -484,18 +499,18 @@ char *knh_methodop_tochar(knh_methodn_t mn)
 static
 void knh_Class_NAME__man(Ctx *ctx, knh_class_t cid, OutputStream *w)
 {
-	knh_write__s(ctx, w, _("Class"));
+	knh_write_char(ctx, w, _("Class"));
 	knh_write_EOL(ctx, w);
 
 	knh_write_TAB(ctx, w);
-	knh_write(ctx, w, knh_String_tobytes(ctx->share->ClassTable[cid].lname));
+	knh_write(ctx, w, knh_String_tobytes(ClassTable(cid).lname));
 	knh_write_EOL(ctx, w);
 
-	while(ctx->share->ClassTable[cid].supcid != CLASS_Object) {
-		cid = ctx->share->ClassTable[cid].supcid;
+	while(ClassTable(cid).supcid != CLASS_Object) {
+		cid = ClassTable(cid).supcid;
 		knh_write_TAB(ctx, w);
 		knh_write(ctx, w, STEXT("extends "));
-		knh_write(ctx, w, knh_String_tobytes(ctx->share->ClassTable[cid].lname));
+		knh_write(ctx, w, knh_String_tobytes(ClassTable(cid).lname));
 		knh_write_EOL(ctx, w);
 	}
 }
@@ -505,20 +520,23 @@ void knh_Class_NAME__man(Ctx *ctx, knh_class_t cid, OutputStream *w)
 static
 void knh_Method__man(Ctx *ctx, Method *o, OutputStream *w, knh_class_t cid)
 {
-	if(knh_Method_isAbstract(o)) {
-		knh_write(ctx, w, STEXT("@abstract"));
-		knh_putc(ctx, w, ' ');
+	if(!knh_Context_isVerbose(ctx)) {
+		if(knh_Method_isPrivate(o)) return;
 	}
 
-	if(knh_Method_rztype(o) == TYPE_void) {
-		knh_write(ctx, w, knh_String_tobytes(TS_void));
-	}else{
-		knh_write_type(ctx, w, knh_pmztype_totype(ctx, knh_Method_rztype(o), cid));
+	if(knh_Method_isAbstract(o)) {
+		knh_write(ctx, w, STEXT("@Abstract"));
+		knh_putc(ctx, w, ' ');
 	}
+//	if(knh_Method_rztype(o) == TYPE_void) {
+//		knh_write(ctx, w, knh_String_tobytes(TS_void));
+//	}else{
+		knh_write_type(ctx, w, knh_pmztype_totype(ctx, knh_Method_rztype(o), cid));
+//	}
 	knh_putc(ctx, w, ' ');
 
 	if(knh_Method_isStatic(o)) {
-		knh_write__s(ctx, w, CTXCLASSN(cid));
+		knh_write_char(ctx, w, CTXCLASSN(cid));
 		knh_putc(ctx, w, '.');
 	}
 	knh_write_mn(ctx, w, DP(o)->mn);
@@ -558,7 +576,7 @@ void knh_Class__man(Ctx *ctx, Class *o, OutputStream *w, String *m)
 	KNH_LPUSH(ctx, dm);
 	size_t i = 0;
 	while(1) {
-		Array *a = ctx->share->ClassTable[cid].cstruct->methods;
+		Array *a = ClassTable(cid).cstruct->methods;
 		for(i = 0; i < knh_Array_size(a); i++) {
 			Method *mtd = (Method*)knh_Array_n(a, i);
 			char *op = knh_methodop_tochar(DP(mtd)->mn);
@@ -573,12 +591,12 @@ void knh_Class__man(Ctx *ctx, Class *o, OutputStream *w, String *m)
 			else {
 				knh_bytes_t name = B(op);
 				if(IS_NULL(knh_DictMap_get__b(ctx,  dm, name))) {
-					knh_DictMap_set(ctx, dm, new_String__T(ctx, op), UP(mtd));
+					knh_DictMap_set(ctx, dm, T(op), UP(mtd));
 				}
 			}
 		}
 		if(cid == CLASS_Object) break;
-		cid = ctx->share->ClassTable[cid].supcid;
+		cid = ClassTable(cid).supcid;
 	}
 
 	int cnt = 0;
@@ -594,7 +612,7 @@ void knh_Class__man(Ctx *ctx, Class *o, OutputStream *w, String *m)
 			knh_DictMap_removeAt(ctx, dm, i);
 			if(DP(mtd)->cid == CLASS_Object && cid != CLASS_Object) continue;
 			if(hasCaption == 0) {
-				knh_write__s(ctx, w, _("Operator"));
+				knh_write_char(ctx, w, _("Operator"));
 				knh_write_EOL(ctx, w);
 				hasCaption = 1;
 			}
@@ -603,7 +621,7 @@ void knh_Class__man(Ctx *ctx, Class *o, OutputStream *w, String *m)
 				isBOL = 0;
 			}
 			knh_snprintf(bufmn, sizeof(bufmn), "%10s  ", op);
-			knh_write__s(ctx, w, bufmn);
+			knh_write_char(ctx, w, bufmn);
 			if(cnt % 5 == 4) {
 				knh_write_EOL(ctx, w);
 				isBOL = 1;
@@ -624,9 +642,9 @@ void knh_Class__man(Ctx *ctx, Class *o, OutputStream *w, String *m)
 		if(IS_Method(mtd)) {
 			if(METHODN_IS_MOVTEXT(DP(mtd)->mn)) continue;
 			if(DP(mtd)->cid == CLASS_Object && cid != CLASS_Object) continue;
-//			if(DP(mtd)->cid != ctx->share->ClassTable[cid].bcid) continue;
+//			if(DP(mtd)->cid != ClassTable(cid).bcid) continue;
 			if(hasCaption == 0) {
-				knh_write__s(ctx, w, _("Method"));
+				knh_write_char(ctx, w, _("Method"));
 				knh_write_EOL(ctx, w);
 				hasCaption = 1;
 			}
@@ -645,7 +663,7 @@ void knh_Class__man(Ctx *ctx, Class *o, OutputStream *w, String *m)
 			if(!METHODN_IS_MOVTEXT(DP(mtd)->mn)) continue;
 			if(DP(mtd)->cid == CLASS_Object && cid != CLASS_Object) continue;
 			if(hasCaption == 0) {
-				knh_write__s(ctx, w, _("Formatter"));
+				knh_write_char(ctx, w, _("Formatter"));
 				knh_write_EOL(ctx, w);
 				knh_write_TAB(ctx, w);
 				hasCaption = 1;
@@ -662,7 +680,7 @@ void knh_Class__man(Ctx *ctx, Class *o, OutputStream *w, String *m)
 	}
 	knh_write_EOL(ctx, w);
 
-	knh_ClassMap__man(ctx, ctx->share->ClassTable[cid].cmap, w, cid);
+	knh_ClassMap__man(ctx, ClassTable(cid).cmap, w, cid);
 	KNH_LOCALBACK(ctx, lsfp);
 }
 
@@ -673,7 +691,7 @@ void knh_Class__man(Ctx *ctx, Class *o, OutputStream *w, String *m)
 
 void knh_Method__s(Ctx *ctx, Method *o, OutputStream *w, String *m)
 {
-	knh_write__s(ctx, w, CTXCLASSN(DP(o)->cid));
+	knh_write_char(ctx, w, CTXCLASSN(DP(o)->cid));
 	knh_putc(ctx, w, '.');
 	knh_write_mn(ctx, w, DP(o)->mn);
 }
@@ -687,11 +705,9 @@ void knh_Method__k(Ctx *ctx, Method *o, OutputStream *w, String *m)
 		knh_write(ctx, w, STEXT("@abstract"));
 		knh_putc(ctx, w, ' ');
 	}
-
 	if(knh_Method_rztype(o) == TYPE_void) {
 		knh_write(ctx, w, knh_String_tobytes(TS_void));
 	}else{
-//		knh_write__type(ctx, w, knh_pmztype_totype(ctx, knh_Method_rtype(o), DP(o)->cid));
 		knh_write_type(ctx, w, knh_Method_rztype(o));
 	}
 	knh_putc(ctx, w, ' ');
@@ -703,7 +719,6 @@ void knh_Method__k(Ctx *ctx, Method *o, OutputStream *w, String *m)
 			knh_write_delim(ctx, w);
 		}
 		knh_mparam_t mf = knh_Method_param(o, i);
-		//knh_write__type(ctx, w, knh_pmztype_totype(ctx, mf.type, DP(o)->cid));
 		knh_write_type(ctx, w, mf.type);
 		knh_putc(ctx, w, ' ');
 		knh_write(ctx, w, B(FIELDN(mf.fn)));
