@@ -51,6 +51,7 @@ knh_struct_t knh_StructTable_newId(Ctx *ctx)
 	}
 	newid = ctx->share->StructTableSize;
 	((knh_SharedData_t*)ctx->share)->StructTableSize += 1;
+
 	L_UNLOCK:;
 	KNH_UNLOCK(ctx, LOCK_SYSTBL, NULL);
 	return newid;
@@ -76,57 +77,26 @@ knh_class_t knh_ClassTable_newId(Ctx *ctx)
 
 /* ------------------------------------------------------------------------ */
 
-knh_Class_t *new_Class(Ctx *ctx, knh_class_t cid)
+static knh_Class_t *new_Class(Ctx *ctx, knh_class_t cid)
 {
-	DBG2_ASSERT_cid(cid);
-	knh_ClassTable_t *t = pClassTable(cid);
-	if(t->class_cid == NULL) {
-		knh_Class_t *o = (knh_Class_t*)new_hObject(ctx, FLAG_Class, CLASS_Class, CLASS_Class);
-		o->cid = cid;
-		o->type = cid;
-		KNH_INITv(t->class_cid, o);
-	}
-	return t->class_cid;
+	knh_Class_t *o = (knh_Class_t*)new_hObject(ctx, FLAG_Class, CLASS_Class, CLASS_Class);
+	o->cid = cid;
+	o->type = cid;
+	return o;
 }
 
 /* ------------------------------------------------------------------------ */
 
-knh_Class_t *new_Type(Ctx *ctx, knh_type_t type)
+void knh_setClassName(Ctx *ctx, knh_class_t cid, String *lname)
 {
-	knh_class_t cid = CLASS_type(type);
-	DBG2_ASSERT_cid(cid);
+	KNH_ASSERT_cid(cid);
 	knh_ClassTable_t *t = pClassTable(cid);
-	if(IS_NATYPE(type)) {
-		if(t->class_natype == NULL) {
-			knh_Class_t *o = (knh_Class_t*)new_hObject(ctx, FLAG_Class, CLASS_Class, CLASS_Class);
-			o->cid = cid;
-			o->type = type;
-			KNH_INITv(t->class_natype, o);
-		}
-		return t->class_natype;
-	}
-	else {
-		if(t->class_cid == NULL) {
-			knh_Class_t *o = (knh_Class_t*)new_hObject(ctx, FLAG_Class, CLASS_Class, CLASS_Class);
-			o->cid = cid;
-			o->type = type;
-			KNH_INITv(t->class_cid, o);
-		}
-		return t->class_cid;
-	}
-}
-
-/* ------------------------------------------------------------------------ */
-
-void knh_setClassName(Ctx *ctx, knh_class_t cid, knh_String_t *lname)
-{
-	DBG2_ASSERT_cid(cid);
-	knh_ClassTable_t *t = pClassTable(cid);
-	//KNH_NOTICE(ctx, _("added new class: %s"), __tochar(lname));
+	//KNH_NOTICE(ctx, _("added new class: %s"), knh_String_tochar(lname));
 	KNH_ASSERT(ClassTable(cid).class_cid == NULL);
+	KNH_INITv(t->class_cid, new_Class(ctx, cid));
 	KNH_INITv(t->lname, lname);
 	{
-		knh_bytes_t n = __tobytes(lname);
+		knh_bytes_t n = knh_String_tobytes(lname);
 		knh_index_t idx = knh_bytes_index(n, '{');
 		if(idx != -1) {
 			KNH_INITv(t->sname, lname);
@@ -149,7 +119,7 @@ void knh_setClassName(Ctx *ctx, knh_class_t cid, knh_String_t *lname)
 	}
 
 	L_ADDDICT:
-	//DBG2_P("NEW CLASS: %s, %s", __tochar(lname), __tochar(t->sname));
+	//DBG2_P("NEW CLASS: %s, %s", knh_String_tochar(lname), knh_String_tochar(t->sname));
 	KNH_LOCK(ctx, LOCK_SYSTBL, NULL);
 	{
 		knh_DictSet_append(ctx, DP(ctx->sys)->ClassNameDictSet, lname, cid+1);
@@ -158,7 +128,6 @@ void knh_setClassName(Ctx *ctx, knh_class_t cid, knh_String_t *lname)
 		}
 	}
 	KNH_UNLOCK(ctx, LOCK_SYSTBL, NULL);
-
 }
 
 /* ------------------------------------------------------------------------ */
@@ -169,21 +138,20 @@ char *knh_ClassTable_CLASSN(Ctx *ctx, knh_class_t cid)
 	if(cid == CLASS_unknown) {
 		return "unknown";
 	}
+	KNH_ASSERT_cid(cid);
+	DBG2_ASSERT(ClassTable(cid).sname != NULL);
+	if(knh_String_startsWith(ClassTable(cid).lname, STEXT("konoha."))) {
+		return knh_String_tochar(ClassTable(cid).sname);
+	}
 	else {
-		DBG2_ASSERT_cid(cid);
-		DBG2_ASSERT(ClassTable(cid).sname != NULL);
-		if(knh_String_startsWith(ClassTable(cid).lname, STEXT("konoha."))) {
-			return __tochar(ClassTable(cid).sname);
-		}
-		else {
-			return __tochar(ClassTable(cid).lname);
-		}
+		return knh_String_tochar(ClassTable(cid).lname);
 	}
 }
 
 /* ------------------------------------------------------------------------ */
 /* [default] */
 
+static
 knh_Object_t *knh_fdefault__CONST(Ctx *ctx, knh_class_t cid)
 {
 	return ClassTable(cid).cspec;
@@ -191,13 +159,13 @@ knh_Object_t *knh_fdefault__CONST(Ctx *ctx, knh_class_t cid)
 
 /* ------------------------------------------------------------------------ */
 
-knh_Object_t *knh_fdefault__NEWVALUE(Ctx *ctx, knh_class_t cid)
+static
+knh_Object_t *knh_fdefault__INIT(Ctx *ctx, knh_class_t cid)
 {
+	KNH_ASSERT(IS_NULL(ClassTable(cid).cspec));
+	Object *v = new_Object_init(ctx, ClassTable(cid).oflag | KNH_FLAG_OF_IMMUTABLE | KNH_FLAG_OF_UNDEFINED, cid, 0);
 	knh_ClassTable_t *t = pClassTable(cid);
-	Object *v = new_Object_init(ctx, ClassTable(cid).oflag | FLAG_Object_Immutable | FLAG_Object_Undefined, cid, 0);
-	DBG2_P("create new default value of %s", CLASSN(cid));
-	KNH_ASSERT(t->cspec == NULL);
-	KNH_INITv(t->cspec, v);
+	KNH_SETv(ctx, t->cspec, v);
 	t->fdefault = knh_fdefault__CONST;
 	return t->cspec;
 }
@@ -206,52 +174,62 @@ knh_Object_t *knh_fdefault__NEWVALUE(Ctx *ctx, knh_class_t cid)
 
 void knh_setClassDefaultValue(Ctx *ctx, knh_class_t cid, Object *value, knh_fdefault fdefault)
 {
-	DBG2_ASSERT_cid(cid);
+	KNH_ASSERT_cid(cid);
 	knh_ClassTable_t *t = pClassTable(cid);
-	if(value == NULL) {
-		if(fdefault == NULL) fdefault = knh_fdefault__NEWVALUE;
+	if(t->cspec == NULL) {
+		KNH_INITv(t->cspec, value);
 	}
 	else {
-		KNH_ASSERT(t->cspec == NULL);
-		KNH_INITv(t->cspec, value);
-		if(fdefault == NULL) fdefault = knh_fdefault__CONST;
+		KNH_SETv(ctx, t->cspec, value);
+	}
+	if(fdefault == NULL) {
+		if(IS_NULL(value) && cid != CLASS_Any) {
+			fdefault = knh_fdefault__INIT;
+		}
+		else {
+			fdefault = knh_fdefault__CONST;
+		}
 	}
 	t->fdefault = fdefault;
 }
 
 /* ------------------------------------------------------------------------ */
 
+#define _KNH_DEF(ctx, cid)  knh_getClassDefaultValue(ctx, cid)
+
 Object *knh_getClassDefaultValue(Ctx *ctx, knh_class_t cid)
 {
-	DBG2_ASSERT_cid(cid);
+	KNH_ASSERT_cid(cid);
 	return ClassTable(cid).fdefault(ctx, cid);
 }
 
-///* ------------------------------------------------------------------------ */
-//
-//Object *knh_getDefaultValue(Ctx *ctx, knh_type_t type)
-//{
-//	if(IS_NNTYPE(type)) {
-//		knh_class_t cid = CLASS_type(type);
-//		DBG2_ASSERT_cid(cid);
-//		return ClassTable(cid).fdefault(ctx, cid);
-//	}
-//	else {
-//		return KNH_NULL;
-//	}
-//}
+/* ------------------------------------------------------------------------ */
+
+#define _KNH_DEF(ctx, cid)  knh_getClassDefaultValue(ctx, cid)
+
+Object *knh_getDefaultValue(Ctx *ctx, knh_type_t type)
+{
+	if(IS_NNTYPE(type)) {
+		knh_class_t cid = CLASS_type(type);
+		KNH_ASSERT_cid(cid);
+		return ClassTable(cid).fdefault(ctx, cid);
+	}
+	else {
+		return KNH_NULL;
+	}
+}
 
 /* ======================================================================== */
 /* [PARAM] */
 
 void knh_setClassParam(Ctx *ctx, knh_class_t cid, knh_class_t p1, knh_class_t p2)
 {
-	DBG2_ASSERT_cid(cid);
+	KNH_ASSERT_cid(cid);
 	knh_ClassTable_t *t = pClassTable(cid);
 	t->p1 = p1;
 	t->p2 = p2;
 	if(!knh_class_isCyclic(cid)) {
-		if(knh_class_isCyclic(p1) || (p2 != CLASS_Tvoid && knh_class_isCyclic(p2))) {
+		if(knh_class_isCyclic(p1) || knh_class_isCyclic(p2)) {
 			knh_class_setCyclic(cid, 1);
 		}
 	}
@@ -260,23 +238,17 @@ void knh_setClassParam(Ctx *ctx, knh_class_t cid, knh_class_t p1, knh_class_t p2
 /* ------------------------------------------------------------------------ */
 
 knh_class_t
-knh_addGenericsClass(Ctx *ctx, knh_class_t cid, knh_String_t *name, knh_class_t bcid, knh_class_t p1, knh_class_t p2)
+knh_addGenericsClass(Ctx *ctx, knh_class_t cid, String *name, knh_class_t bcid, knh_class_t p1, knh_class_t p2)
 {
-	knh_ClassTable_t *t;
-	knh_flag_t mask = 0;
 	if(cid == CLASS_newid) {
 		cid = knh_ClassTable_newId(ctx);
 	}else {
 		//KNH_ASSERT(cid + 1 == ctx->share->ClassTableSize);
 		((knh_SharedData_t*)ctx->share)->ClassTableSize = cid;
 	}
-	/* knh_ClassTable_t */t = pClassTable(cid);
-	DBG2_ASSERT(bcid < cid);
-	KNH_ASSERT(t->sname == NULL);
-	if(knh_class_isTypeVariable(CLASS_type(p1)) ||
-			(p2 != CLASS_Tvoid && knh_class_isTypeVariable(CLASS_type(p2)))) {
-		mask = FLAG_Class_TypeVariable;
-	}
+	knh_ClassTable_t *t = pClassTable(cid);
+	KNH_ASSERT(bcid < cid);
+	KNH_ASSERT(t->class_cid == NULL);
 
 	t->cflag  = ClassTable(bcid).cflag;
 	t->oflag  = ClassTable(bcid).oflag;
@@ -295,17 +267,18 @@ knh_addGenericsClass(Ctx *ctx, knh_class_t cid, knh_String_t *name, knh_class_t
 	knh_setClassName(ctx, cid, name);
 	KNH_INITv(t->cstruct, ClassTable(bcid).cstruct);
 	KNH_INITv(t->cmap, new_ClassMap0(ctx, 0));
+	knh_setClassDefaultValue(ctx, cid, KNH_NULL, NULL);
 	knh_setClassParam(ctx, cid, p1, p2);
-	StructTable(bcid).fnewClass(ctx, cid);
+	ctx->share->StructTable[bcid].fnewClass(ctx, cid);
 	return cid;
 }
 
 /* ======================================================================== */
-/* [ClassField] */
+/* [ClassStruct] */
 
-knh_ClassField_t* new_ClassField0(Ctx *ctx, int field_size, int method_size)
+ClassStruct* new_ClassStruct0(Ctx *ctx, int field_size, int method_size)
 {
-	knh_ClassField_t* cs = (knh_ClassField_t*)new_Object_bcid(ctx, CLASS_ClassField, field_size);
+	knh_ClassStruct_t* cs = (knh_ClassStruct_t*)new_Object_bcid(ctx, CLASS_ClassStruct, field_size);
 	KNH_SETv(ctx, cs->methods, new_Array0(ctx, method_size));
 	return cs;
 }
@@ -316,11 +289,11 @@ knh_ClassField_t* new_ClassField0(Ctx *ctx, int field_size, int method_size)
 knh_index_t knh_Class_indexOfField(Ctx *ctx, knh_class_t cid, knh_fieldn_t fn)
 {
 	L_TAIL:;
-	DBG2_ASSERT_cid(cid);
+	KNH_ASSERT_cid(cid);
 	DBG2_P("******* cid=%s, offset=%d, supcid=%s", CLASSN(cid), ClassTable(cid).offset, CLASSN(ClassTable(cid).supcid));
 	{
 		knh_index_t idx = -1;
-		knh_ClassField_t *cs = ClassTable(cid).cstruct;
+		ClassStruct *cs = ClassTable(cid).cstruct;
 		if(cs->fields != NULL) {
 			for(idx = 0; idx < cs->fsize; idx++) {
 				if(cs->fields[idx].fn == fn) {
@@ -341,20 +314,21 @@ knh_index_t knh_Class_queryField(Ctx *ctx, knh_class_t cid, knh_fieldn_t fnq)
 {
 	knh_fieldn_t fn = FIELDN_UNMASK(fnq);
 	L_TAIL:;
-	DBG2_ASSERT_cid(cid);
+	KNH_ASSERT_cid(cid);
 	{
-		knh_ClassField_t *cs = ClassTable(cid).cstruct;
+		ClassStruct *cs = ClassTable(cid).cstruct;
 		knh_index_t idx = -1;
 		if(FIELDN_IS_SUPER(fnq)) {
 			fnq = fn;
 			goto L_SUPER;
 		}
 		if(cs->fields != NULL) {
-			for(idx = cs->fsize - 1; idx >= 0; idx--) {
+			for(idx = 0; idx < cs->fsize; idx++) {
 				if(cs->fields[idx].fn == fn) {
 					return ClassTable(cid).offset + idx;
 				}
 			}
+			idx = -1;
 		}
 		L_SUPER:;
 		if(ClassTable(cid).offset == 0) return -1;
@@ -367,11 +341,11 @@ knh_index_t knh_Class_queryField(Ctx *ctx, knh_class_t cid, knh_fieldn_t fnq)
 
 knh_cfield_t *knh_Class_fieldAt(Ctx *ctx, knh_class_t cid, size_t n)
 {
-	DBG2_ASSERT_cid(cid);
+	KNH_ASSERT_cid(cid);
 	KNH_ASSERT(/*0 <= n &&*/ n < ClassTable(cid).size);
 	L_TAIL:;
 	{
-		knh_ClassField_t *cs = ClassTable(cid).cstruct;;
+		ClassStruct *cs = ClassTable(cid).cstruct;;
 		size_t offset = ClassTable(cid).offset;
 		if(offset <= n) {
 			if(cs->fields == NULL) {
@@ -381,18 +355,35 @@ knh_cfield_t *knh_Class_fieldAt(Ctx *ctx, knh_class_t cid, size_t n)
 				return &(cs->fields[n - offset]);
 			}
 		}
-		DBG2_ASSERT_cid(cid);
+		KNH_ASSERT_cid(cid);
 		cid = ClassTable(cid).supcid;
 	}
 	goto L_TAIL;
 }
 
+
+/* ------------------------------------------------------------------------ */
+/* [movabletext] */
+
+void knh_cfield_dump(Ctx *ctx, knh_cfield_t *f, size_t offset, size_t fsize, OutputStream *w)
+{
+	size_t idx = 0;
+	for(idx = 0; idx < fsize; idx++) {
+		if(f[idx].fn == FIELDN_NONAME) {
+			knh_printf(ctx, w, "[%d] -\n", (offset+idx));
+			continue;
+		}
+		if(f[idx].fn == FIELDN_register || f[idx].type == CLASS_unknown) continue;
+		knh_printf(ctx, w, "[%d] %T %N = %O\n", (offset+idx), f[idx].type, f[idx].fn, f[idx].value);
+	}
+}
+
 /* ======================================================================== */
 /* [ClassMap] */
 
-knh_ClassMap_t* new_ClassMap0(Ctx *ctx, knh_ushort_t capacity)
+ClassMap* new_ClassMap0(Ctx *ctx, knh_ushort_t capacity)
 {
-	if(capacity < KNH_FASTMALLOC_BSIZE) capacity = KNH_FASTMALLOC_BSIZE;
+	if(capacity < KNH_FASTMALLOC_SIZE) capacity = KNH_FASTMALLOC_BSIZE;
 	knh_ClassMap_t *o = (knh_ClassMap_t*)new_Object_bcid(ctx, CLASS_ClassMap, capacity);
 	knh_ClassMap_setSorted(o, 1);
 	return o;
@@ -400,30 +391,18 @@ knh_ClassMap_t* new_ClassMap0(Ctx *ctx, knh_ushort_t capacity)
 
 /* ------------------------------------------------------------------------ */
 
-Object *knh_ClassMap_fdefault(Ctx *ctx, knh_class_t cid)
+int knh_ClassMap_isDefault(Ctx *ctx, ClassMap *o)
 {
-	if(unlikely(ClassTable(CLASS_Any).cmap == NULL)) {
-		knh_ClassTable_t *t = pClassTable(CLASS_Any);
-		KNH_INITv(t->cmap, new_ClassMap0(ctx, 0));
-	}
-	return (Object*)ClassTable(CLASS_Any).cmap;
+	return (o == ctx->share->ClassTable[CLASS_Any].cmap);
 }
 
 /* ------------------------------------------------------------------------ */
 
-static
-int knh_ClassMap_isDefault(Ctx *ctx, knh_ClassMap_t *o)
+void knh_ClassTable_readyClassMap(Ctx *ctx, knh_class_t cid)
 {
-	return (o == (knh_ClassMap_t*)knh_ClassMap_fdefault(ctx, CLASS_Any));
-}
-
-/* ------------------------------------------------------------------------ */
-
-void knh_readyClassMap(Ctx *ctx, knh_class_t cid)
-{
-	DBG2_ASSERT_cid(cid);
+	KNH_ASSERT_cid(cid);
 	KNH_ASSERT(cid != CLASS_Any);
-	if(knh_ClassMap_isDefault(ctx, ClassTable(cid).cmap)) {
+	if(ClassTable(cid).cmap == ClassTable(CLASS_Any).cmap) {
 		knh_ClassTable_t *t = pClassTable(cid);
 		KNH_SETv(ctx, t->cmap, new_ClassMap0(ctx, 4));
 	}
@@ -431,24 +410,23 @@ void knh_readyClassMap(Ctx *ctx, knh_class_t cid)
 
 /* ------------------------------------------------------------------------ */
 
-void knh_ClassMap_resize(Ctx *ctx, knh_ClassMap_t *cmap, size_t newsize)
+void knh_ClassMap_resize(Ctx *ctx, ClassMap *o, size_t newsize)
 {
+	Mapper **newlist = (Mapper**)KNH_MALLOC(ctx, newsize * sizeof(Mapper*));
+	knh_bzero(newlist, newsize);
 	knh_intptr_t i;
-	knh_Mapper_t **newlist =
-		(knh_Mapper_t**)KNH_MALLOC(ctx, newsize * sizeof(knh_Mapper_t*));
-	knh_bzero(newlist, newsize * sizeof(knh_Mapper_t*));
-	for(i = 0; i < (cmap)->size; i++) {
-		newlist[i] = (cmap)->maplist[i];
+	for(i = 0; i < DP(o)->size; i++) {
+		newlist[i] = DP(o)->maplist[i];
 	}
-	KNH_FREE(ctx, (cmap)->maplist, (cmap)->capacity * sizeof(knh_Mapper_t*));
-	(cmap)->maplist = newlist;
-	(cmap)->capacity = newsize;
+	KNH_FREE(ctx, DP(o)->maplist, DP(o)->capacity * sizeof(Mapper*));
+	DP(o)->maplist = newlist;
+	DP(o)->capacity = newsize;
 }
 
 /* ------------------------------------------------------------------------ */
 
 int
-knh_ClassMap_util_cmp(const knh_Mapper_t *m1, const knh_Mapper_t *m2)
+knh_ClassMap_util_cmp(const Mapper *m1, const Mapper *m2)
 {
 	int res = DP(m1)->flag - DP(m2)->flag;
 	return (res == 0) ? DP(m2)->tcid - DP(m1)->tcid : res;
@@ -456,61 +434,115 @@ knh_ClassMap_util_cmp(const knh_Mapper_t *m1, const knh_Mapper_t *m2)
 
 /* ------------------------------------------------------------------------ */
 
-void knh_ClassMap_sort(Ctx *ctx, knh_ClassMap_t *cmap)
+void knh_ClassMap_sort(Ctx *ctx, ClassMap *o)
 {
-	if(!knh_ClassMap_isSorted(cmap)) {
-		knh_qsort((cmap)->maplist, (cmap)->size, sizeof(knh_Mapper_t*),
+	if(!knh_ClassMap_isSorted(o)) {
+		knh_qsort(DP(o)->maplist, DP(o)->size, sizeof(Mapper*),
 					(int (*)(const void*, const void*))knh_ClassMap_util_cmp);
-		knh_ClassMap_setSorted(cmap, 1);
+		knh_ClassMap_setSorted(o, 1);
 	}
 }
 
 /* ------------------------------------------------------------------------ */
 
-void knh_ClassMap_add(Ctx *ctx, knh_ClassMap_t *cmap, knh_Mapper_t *mpr)
+void knh_ClassMap_add(Ctx *ctx, ClassMap *o, Mapper *map)
 {
-	DBG2_ASSERT(!knh_ClassMap_isDefault(ctx, cmap));
-	if((cmap)->size == (cmap)->capacity) {
-		knh_ClassMap_resize(ctx, cmap, (cmap)->capacity + 8);
+	KNH_ASSERT(!knh_ClassMap_isDefault(ctx, o));
+	if(DP(o)->size == DP(o)->capacity) {
+		knh_ClassMap_resize(ctx, o, DP(o)->capacity + 8);
 	}
-	DBG2_ASSERT((cmap)->size < (cmap)->capacity);
-	DBG2_ASSERT((cmap)->maplist[(cmap)->size] == NULL);
-	KNH_INITv((cmap)->maplist[(cmap)->size], mpr);
-	(cmap)->size++;
+	KNH_ASSERT(DP(o)->maplist[DP(o)->size] == NULL);
+	KNH_INITv(DP(o)->maplist[DP(o)->size], map);
+	DP(o)->size++;
 }
 
 /* ======================================================================== */
+/* [movabletext] */
+
+/* @method void ClassMap.%dump(OutputStream w, String m) */
+
+void knh_ClassMap__dump(Ctx *ctx, ClassMap *o, OutputStream *w, String *m)
+{
+	size_t i;
+	for(i = 0; i < DP(o)->size; i++) {
+		knh_printf(ctx, w, "[%d]\t", i);
+		knh_Mapper__k(ctx, DP(o)->maplist[i], w, m);
+		knh_write_EOL(ctx, w);
+	}
+}
+
+/* ======================================================================== */
+
+void knh_ClassMap__man(Ctx *ctx, ClassMap *o, OutputStream *w, knh_class_t cid)
+{
+	int i;
+	int hasCaption = 0, from = 0;
+	L_TAIL:;
+	knh_ClassMap_sort(ctx, o);
+	for(i = 0; i < DP(o)->size; i++) {
+		if(hasCaption == 0) {
+			knh_write_char(ctx, w, _("Mapper"));
+			knh_write_EOL(ctx, w);
+			hasCaption = 1;
+		}
+		if(from == 0) {
+			knh_write_TAB(ctx, w);
+			knh_write_cid(ctx, w, cid);
+			knh_write_EOL(ctx, w);
+			from = 1;
+		}
+		Mapper *mpr = DP(o)->maplist[i];
+		knh_write_TAB(ctx, w); knh_write_TAB(ctx, w);
+		if(knh_Mapper_isSynonym(mpr)) {
+			knh_write_char(ctx, w, "=== ");
+		}
+		else if(knh_Mapper_isTotal(mpr)) {
+			knh_write_char(ctx, w, "==> ");
+		}
+		else {
+			knh_write_char(ctx, w, "--> ");
+		}
+		knh_write_cid(ctx, w, DP(mpr)->tcid);
+		knh_write_EOL(ctx, w);
+	}
+	if(ClassTable(cid).supcid != CLASS_Object) {
+		cid = ClassTable(cid).supcid;
+		o = ClassTable(cid).cmap;
+		from = 0;
+		goto L_TAIL;
+	}
+}
+
+/* ------------------------------------------------------------------------ */
 /* [DataPool] */
 /* ------------------------------------------------------------------------ */
 
-static knh_Array_t* knh_Class_domain(Ctx *ctx)
+static Array* knh_Class_domain(Ctx *ctx)
 {
-	knh_Array_t *a = new_Array(ctx, CLASS_Class, 0);
-	int cid = 0;
-	for(cid = 0; cid < ctx->share->StructTableSize; cid++) {
-		if(knh_class_isPrivate(cid) || knh_class_isTypeVariable(cid)) continue;
-		knh_Array_add(ctx, a, UP(new_Class(ctx, cid)));
+	Array *a = new_Array(ctx, CLASS_Class, 0);
+	int i = 0;
+	for(i = 0; i < ctx->share->StructTableSize; i++) {
+		knh_Array_add(ctx, a, UP(ClassTable(i).class_cid));
 	}
-	for(cid = ctx->share->ClassTableSize; cid < KNH_TCLASS_SIZE; cid++) {
-		if(knh_class_isPrivate(cid) || knh_class_isTypeVariable(cid)) continue;
-		knh_Array_add(ctx, a, UP(new_Class(ctx, cid)));
+	for(i = ctx->share->ClassTableSize; i < KNH_TCLASS_SIZE; i++) {
+		knh_Array_add(ctx, a, UP(ClassTable(i).class_cid));
 	}
 	return a;
 }
 
 /* ------------------------------------------------------------------------ */
 
-static knh_Array_t* knh_Method_domain(Ctx *ctx)
+static Array* knh_Method_domain(Ctx *ctx)
 {
-	knh_Array_t *a = new_Array(ctx, CLASS_Method, 0);
+	Array *a = new_Array(ctx, CLASS_Method, 0);
 	int i = 0;
 	for(i = 0; i < ctx->share->StructTableSize; i++) {
-		knh_ClassField_t *cs = ClassTable(i).cstruct;
+		ClassStruct *cs = ClassTable(i).cstruct;
 		DBG2_ASSERT(cs != NULL);
 		knh_Array_addArray(ctx, a, cs->methods);
 	}
 	for(i = ctx->share->ClassTableSize; i < KNH_TCLASS_SIZE; i++) {
-		knh_ClassField_t *cs = ClassTable(i).cstruct;
+		ClassStruct *cs = ClassTable(i).cstruct;
 		DBG2_ASSERT(cs != NULL);
 		knh_Array_addArray(ctx, a, cs->methods);
 	}
@@ -519,9 +551,9 @@ static knh_Array_t* knh_Method_domain(Ctx *ctx)
 
 /* ------------------------------------------------------------------------ */
 
-knh_Array_t* knh_getClassDomain(Ctx *ctx, knh_class_t cid)
+Array* knh_getClassDomain(Ctx *ctx, knh_class_t cid)
 {
-	knh_Array_t *a = NULL;
+	Array *a = NULL;
 	switch(cid) {
 	case CLASS_Class:
 		a = knh_Class_domain(ctx);
