@@ -37,13 +37,13 @@ extern "C" {
 
 
 static
-knh_class_t knh_addSpecializedType(Ctx *ctx, knh_class_t cid, knh_class_t bcid, knh_ClassSpec_t *u);
+knh_class_t knh_addSpecializedType(Ctx *ctx, knh_class_t cid, knh_class_t bcid, ClassSpec *u);
 
 /* ======================================================================== */
 /* [Int] */
 
 static
-int knh_fichk__range(knh_ClassSpec_t *u, knh_int_t v)
+int knh_fichk__range(ClassSpec *u, knh_int_t v)
 {
 	return (DP(u)->imin <= v && v <= DP(u)->imax);
 }
@@ -51,7 +51,7 @@ int knh_fichk__range(knh_ClassSpec_t *u, knh_int_t v)
 /* ------------------------------------------------------------------------ */
 
 static
-int knh_fichk__umax(knh_ClassSpec_t *u, knh_uint_t v)
+int knh_fichk__umax(ClassSpec *u, knh_uint_t v)
 {
 	return (v <= DP(u)->umax);
 }
@@ -59,7 +59,7 @@ int knh_fichk__umax(knh_ClassSpec_t *u, knh_uint_t v)
 /* ------------------------------------------------------------------------ */
 
 static
-int knh_fichk__urange(knh_ClassSpec_t *u, knh_uint_t v)
+int knh_fichk__urange(ClassSpec *u, knh_uint_t v)
 {
 	return (DP(u)->umin <= v && v <= DP(u)->umax);
 }
@@ -67,7 +67,7 @@ int knh_fichk__urange(knh_ClassSpec_t *u, knh_uint_t v)
 /* ------------------------------------------------------------------------ */
 
 static
-int knh_ficmp__unsigned(knh_ClassSpec_t *u, knh_uint_t v1, knh_uint_t v2)
+int knh_ficmp__unsigned(ClassSpec *u, knh_uint_t v1, knh_uint_t v2)
 {
 	if(v1 == v2) return 0;
 	return (v1 > v2) ? 1 : -1;
@@ -77,7 +77,7 @@ int knh_ficmp__unsigned(knh_ClassSpec_t *u, knh_uint_t v1, knh_uint_t v2)
 /* [Float] */
 
 static
-int knh_ffchk__range(knh_ClassSpec_t *u, knh_float_t v)
+int knh_ffchk__range(ClassSpec *u, knh_float_t v)
 {
 	return (DP(u)->fmin <= v && v <= DP(u)->fmax);
 }
@@ -85,7 +85,7 @@ int knh_ffchk__range(knh_ClassSpec_t *u, knh_float_t v)
 /* ------------------------------------------------------------------------ */
 
 static
-int knh_ffcmp__step(knh_ClassSpec_t *u, knh_float_t v1, knh_float_t v2)
+int knh_ffcmp__step(ClassSpec *u, knh_float_t v1, knh_float_t v2)
 {
 	knh_float_t v = v1 - v2;
 	if(v >= DP(u)->fstep) return 1;
@@ -96,7 +96,7 @@ int knh_ffcmp__step(knh_ClassSpec_t *u, knh_float_t v1, knh_float_t v2)
 /* ------------------------------------------------------------------------ */
 
 static
-void knh_ClassSpec_initIntRange(Ctx *ctx, knh_ClassSpec_t *u, knh_int_t min, knh_int_t max)
+void knh_ClassSpec_initIntRange(Ctx *ctx, ClassSpec *u, knh_int_t min, knh_int_t max)
 {
 	DP(u)->imin = min;
 	DP(u)->fmin = (knh_float_t)min;
@@ -128,7 +128,7 @@ void knh_ClassSpec_initIntRange(Ctx *ctx, knh_ClassSpec_t *u, knh_int_t min, knh
 
 /* ------------------------------------------------------------------------ */
 
-void knh_write_intx(Ctx *ctx, knh_OutputStream_t *w, knh_ClassSpec_t *u, knh_int_t v)
+void knh_write_intx(Ctx *ctx, OutputStream *w, ClassSpec *u, knh_int_t v)
 {
 	char buf[KNH_INT_FMTSIZ];
 	char *fmt = KNH_INT_FMT;
@@ -137,7 +137,7 @@ void knh_write_intx(Ctx *ctx, knh_OutputStream_t *w, knh_ClassSpec_t *u, knh_int
 	}
 	knh_snprintf(buf, sizeof(buf), fmt, v);
 	knh_write(ctx, w, B(buf));
-	knh_bytes_t tag = __tobytes(DP(u)->tag);
+	knh_bytes_t tag = knh_String_tobytes(DP(u)->tag);
 	if(tag.len > 0) {
 		knh_putc(ctx, w, '[');
 		knh_write(ctx, w, tag);
@@ -148,7 +148,7 @@ void knh_write_intx(Ctx *ctx, knh_OutputStream_t *w, knh_ClassSpec_t *u, knh_int
 /* ------------------------------------------------------------------------ */
 
 static
-void knh_ClassSpec_initFloatRange(Ctx *ctx, knh_ClassSpec_t *u, knh_float_t min, knh_float_t max, knh_float_t step)
+void knh_ClassSpec_initFloatRange(Ctx *ctx, ClassSpec *u, knh_float_t min, knh_float_t max, knh_float_t step)
 {
 	DP(u)->fmin = min;
 	DP(u)->fmax = max;
@@ -188,7 +188,7 @@ void knh_ClassSpec_initFloatRange(Ctx *ctx, knh_ClassSpec_t *u, knh_float_t min,
 
 /* ------------------------------------------------------------------------ */
 
-void knh_write_floatx(Ctx *ctx, knh_OutputStream_t *w, knh_ClassSpec_t *u, knh_float_t v)
+void knh_write_floatx(Ctx *ctx, OutputStream *w, ClassSpec *u, knh_float_t v)
 {
 	char *FMT = KNH_FLOAT_FMT;
 #if !defined(KNH_USING_NOFLOT)
@@ -207,7 +207,7 @@ void knh_write_floatx(Ctx *ctx, knh_OutputStream_t *w, knh_ClassSpec_t *u, knh_f
 	}
 #endif
 	knh_write_ffmt(ctx, w, FMT, v);
-	knh_bytes_t tag = __tobytes(DP(u)->tag);
+	knh_bytes_t tag = knh_String_tobytes(DP(u)->tag);
 	if(tag.len > 0) {
 		knh_putc(ctx, w, '[');
 		knh_write(ctx, w, tag);
@@ -234,7 +234,7 @@ static MAPPER knh_FloatX_IntX(Ctx *ctx, knh_sfp_t *sfp)
 /* ------------------------------------------------------------------------ */
 
 static
-int knh_ClassSpec_isVocab(knh_ClassSpec_t *u)
+int knh_ClassSpec_isVocab(ClassSpec *u)
 {
 	return IS_DictIdx(DP(u)->vocabDictIdx);
 }
@@ -242,7 +242,7 @@ int knh_ClassSpec_isVocab(knh_ClassSpec_t *u)
 /* ------------------------------------------------------------------------ */
 
 static
-knh_String_t *knh_ClassSpec_getVocabAt(Ctx *ctx, knh_ClassSpec_t *u, size_t n)
+String *knh_ClassSpec_getVocabAt(Ctx *ctx, ClassSpec *u, size_t n)
 {
 	return knh_DictIdx_get__fast(DP(u)->vocabDictIdx, n - DP(u)->imin);
 }
@@ -250,16 +250,16 @@ knh_String_t *knh_ClassSpec_getVocabAt(Ctx *ctx, knh_ClassSpec_t *u, size_t n)
 /* ------------------------------------------------------------------------ */
 
 static
-knh_int_t knh_ClassSpec_getVocabIdx(Ctx *ctx, knh_ClassSpec_t *u, knh_String_t *s)
+knh_int_t knh_ClassSpec_getVocabIdx(Ctx *ctx, ClassSpec *u, String *s)
 {
-	return knh_DictIdx_index(ctx, DP(u)->vocabDictIdx, __tobytes(s)) + DP(u)->imin;
+	return knh_DictIdx_index(ctx, DP(u)->vocabDictIdx, knh_String_tobytes(s)) + DP(u)->imin;
 }
 
 /* ------------------------------------------------------------------------ */
 
 static MAPPER knh_IntX_Vocab(Ctx *ctx, knh_sfp_t *sfp)
 {
-	knh_ClassSpec_t *u = (knh_ClassSpec_t*)DP(sfp[1].mpr)->mapdata;
+	ClassSpec *u = (ClassSpec*)DP(sfp[1].mpr)->mapdata;
 	KNH_ASSERT(IS_ClassSpec(u));
 	KNH_MAPPED(ctx, sfp, knh_ClassSpec_getVocabAt(ctx, u, sfp[0].ivalue));
 }
@@ -268,14 +268,14 @@ static MAPPER knh_IntX_Vocab(Ctx *ctx, knh_sfp_t *sfp)
 
 static MAPPER knh_Vocab_IntX(Ctx *ctx, knh_sfp_t *sfp)
 {
-	knh_ClassSpec_t *u = (knh_ClassSpec_t*)DP(sfp[1].mpr)->mapdata;
+	ClassSpec *u = (ClassSpec*)DP(sfp[1].mpr)->mapdata;
 	KNH_ASSERT(IS_ClassSpec(u));
 	KNH_MAPPED_Int(ctx, sfp, knh_ClassSpec_getVocabIdx(ctx, u, sfp[0].s));
 }
 
 /* ------------------------------------------------------------------------ */
 
-void knh_ClassSpec_reuse(Ctx *ctx, knh_ClassSpec_t *u, knh_class_t cid)
+void knh_ClassSpec_reuse(Ctx *ctx, ClassSpec *u, knh_class_t cid)
 {
 	knh_class_t bcid = ClassTable(cid).bcid;
 	if(bcid == CLASS_Int) {
@@ -286,16 +286,16 @@ void knh_ClassSpec_reuse(Ctx *ctx, knh_ClassSpec_t *u, knh_class_t cid)
 		KNH_ASSERT(DP(u)->ivalue == NULL);
 		KNH_INITv(DP(u)->ivalue, new_IntX__fast(ctx, cid, v));
 		if(DP(u)->fvalue != NULL) {
-			knh_Mapper_t *mpr = new_Mapper(ctx, FLAG_Mapper_Affine, DP(u)->ucid, cid, knh_FloatX_IntX, (Object*)u);
+			Mapper *mpr = new_Mapper(ctx, KNH_FLAG_MMF_AFFINE, DP(u)->ucid, cid, knh_FloatX_IntX, (Object*)u);
 			knh_addMapper(ctx, mpr);
-			mpr = new_Mapper(ctx, FLAG_Mapper_Affine, cid, DP(u)->ucid, knh_IntX_FloatX, (Object*)u);
+			mpr = new_Mapper(ctx, KNH_FLAG_MMF_AFFINE, cid, DP(u)->ucid, knh_IntX_FloatX, (Object*)u);
 			knh_addMapper(ctx, mpr);
 		}
 		if(DP(u)->svalue != NULL) {
 			if(knh_ClassSpec_isVocab(u)) {
-				knh_Mapper_t *mpr = new_Mapper(ctx, FLAG_Mapper_Affine, cid, DP(u)->ucid, knh_IntX_Vocab, (Object*)u);
+				Mapper *mpr = new_Mapper(ctx, KNH_FLAG_MMF_AFFINE, cid, DP(u)->ucid, knh_IntX_Vocab, (Object*)u);
 				knh_addMapper(ctx, mpr);
-				mpr = new_Mapper(ctx, FLAG_Mapper_Affine, DP(u)->ucid, cid, knh_Vocab_IntX, (Object*)u);
+				mpr = new_Mapper(ctx, KNH_FLAG_MMF_AFFINE, DP(u)->ucid, cid, knh_Vocab_IntX, (Object*)u);
 				knh_addMapper(ctx, mpr);
 			}
 		}
@@ -308,9 +308,9 @@ void knh_ClassSpec_reuse(Ctx *ctx, knh_ClassSpec_t *u, knh_class_t cid)
 		KNH_ASSERT(DP(u)->fvalue == NULL);
 		KNH_INITv(DP(u)->fvalue, new_FloatX__fast(ctx, cid, v));
 		if(DP(u)->ivalue != NULL) {
-			knh_Mapper_t *mpr = new_Mapper(ctx, FLAG_Mapper_Affine, cid, DP(u)->ucid, knh_FloatX_IntX, (Object*)u);
+			Mapper *mpr = new_Mapper(ctx, KNH_FLAG_MMF_AFFINE, cid, DP(u)->ucid, knh_FloatX_IntX, (Object*)u);
 			knh_addMapper(ctx, mpr);
-			mpr = new_Mapper(ctx, FLAG_Mapper_Affine, DP(u)->ucid, cid, knh_IntX_FloatX, (Object*)u);
+			mpr = new_Mapper(ctx, KNH_FLAG_MMF_AFFINE, DP(u)->ucid, cid, knh_IntX_FloatX, (Object*)u);
 			knh_addMapper(ctx, mpr);
 		}
 	}
@@ -323,14 +323,14 @@ void knh_ClassSpec_reuse(Ctx *ctx, knh_ClassSpec_t *u, knh_class_t cid)
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(knh_ClassSpec_t*) new_Enum(Ctx *ctx, char *tag, knh_bytes_t urn, knh_int_t min, knh_int_t max)
+KNHAPI(ClassSpec*) new_Enum(Ctx *ctx, char *tag, knh_bytes_t urn, knh_int_t min, knh_int_t max)
 {
 	knh_class_t cid = knh_ClassTable_newId(ctx);
-	knh_ClassSpec_t* u = (knh_ClassSpec_t*)new_Object_bcid(ctx, CLASS_ClassSpec, (int)cid);
+	ClassSpec* u = (ClassSpec*)new_Object_bcid(ctx, CLASS_ClassSpec, (int)cid);
 	DP(u)->ubcid = CLASS_Int;
 	KNH_SETv(ctx, DP(u)->urn, new_String(ctx, urn, NULL));
 	if(tag != NULL || tag[0] != 0) {
-		KNH_SETv(ctx, DP(u)->tag, T__(tag));
+		KNH_SETv(ctx, DP(u)->tag, T(tag));
 	}
 	knh_ClassSpec_initIntRange(ctx, u, min, max);
 	if(!DP(u)->fichk(u, 0)) {
@@ -346,25 +346,23 @@ KNHAPI(knh_ClassSpec_t*) new_Enum(Ctx *ctx, char *tag, knh_bytes_t urn, knh_int_
 /* ------------------------------------------------------------------------ */
 
 
-KNHAPI(knh_ClassSpec_t*) new_Unit(Ctx *ctx, char *tag, knh_bytes_t urn, knh_float_t min, knh_float_t max, knh_float_t step)
+KNHAPI(ClassSpec*) new_Unit(Ctx *ctx, char *tag, knh_bytes_t urn, knh_float_t min, knh_float_t max, knh_float_t step)
 {
 	knh_class_t cid = knh_ClassTable_newId(ctx);
-	knh_ClassSpec_t* u = (knh_ClassSpec_t*)new_Object_bcid(ctx, CLASS_ClassSpec, (int)cid);
+	ClassSpec* u = (ClassSpec*)new_Object_bcid(ctx, CLASS_ClassSpec, (int)cid);
 	DP(u)->ubcid = CLASS_Float;
 	KNH_SETv(ctx, DP(u)->urn, new_String(ctx, urn, NULL));
 	if(tag != NULL || tag[0] != 0) {
-		KNH_SETv(ctx, DP(u)->tag, T__(tag));
+		KNH_SETv(ctx, DP(u)->tag, T(tag));
 
 	}
 	knh_ClassSpec_initFloatRange(ctx, u, min, max, step);
-#if !defined(KNH_USING_NOFLOT)
 	if(!DP(u)->ffchk(u, 0.0)) {
 		KNH_INITv(DP(u)->fvalue, new_FloatX__fast(ctx, cid, min));
 	}
 	else {
 		KNH_INITv(DP(u)->fvalue, new_FloatX__fast(ctx, cid, 0.0));
 	}
-#endif
 	knh_addSpecializedType(ctx, cid, CLASS_Float, u);
 	return u;
 }
@@ -373,9 +371,9 @@ KNHAPI(knh_ClassSpec_t*) new_Unit(Ctx *ctx, char *tag, knh_bytes_t urn, knh_floa
 /* [String] */
 
 static
-knh_String_t *knh_fsnew__dict(Ctx *ctx, knh_class_t cid, knh_bytes_t t, knh_String_t *orign, int *foundError)
+String *knh_fsnew__dict(Ctx *ctx, knh_class_t cid, knh_bytes_t t, String *orign, int *foundError)
 {
-	knh_ClassSpec_t *u = knh_getClassSpec(ctx, cid);
+	ClassSpec *u = knh_getClassSpec(ctx, cid);
 	knh_index_t n = knh_DictIdx_index(ctx, DP(u)->vocabDictIdx, t);
 	if(n == -1) {
 		knh_bytes_t tag;
@@ -393,7 +391,7 @@ knh_String_t *knh_fsnew__dict(Ctx *ctx, knh_class_t cid, knh_bytes_t t, knh_Stri
 /* ------------------------------------------------------------------------ */
 
 static
-int knh_fscmp__dict(knh_ClassSpec_t *u, knh_bytes_t v1, knh_bytes_t v2)
+int knh_fscmp__dict(ClassSpec *u, knh_bytes_t v1, knh_bytes_t v2)
 {
 	return knh_DictIdx_index(NULL, DP(u)->vocabDictIdx, v1) - knh_DictIdx_index(NULL, DP(u)->vocabDictIdx, v2);
 }
@@ -401,7 +399,7 @@ int knh_fscmp__dict(knh_ClassSpec_t *u, knh_bytes_t v1, knh_bytes_t v2)
 /* ------------------------------------------------------------------------ */
 
 static
-knh_DictIdx_t* new_DictIdx__Array(Ctx *ctx, knh_Array_t *a)
+DictIdx* new_DictIdx__Array(Ctx *ctx, Array *a)
 {
 	knh_DictIdx_t *o = (knh_DictIdx_t*)new_hObject(ctx, FLAG_DictIdx, CLASS_DictIdx, CLASS_DictIdx);
 	KNH_INITv(o->terms, a);
@@ -410,7 +408,7 @@ knh_DictIdx_t* new_DictIdx__Array(Ctx *ctx, knh_Array_t *a)
 	{
 		knh_uintptr_t i = 0;
 		for(i = 0; i < knh_Array_size(a); i++) {
-			knh_String_t *s = (knh_String_t*)knh_Array_n(a, i);
+			String *s = (String*)knh_Array_n(a, i);
 			knh_DictSet_append(ctx, o->termsDictSet, s, i+1);
 		}
 	}
@@ -423,7 +421,7 @@ knh_DictIdx_t* new_DictIdx__Array(Ctx *ctx, knh_Array_t *a)
 static
 MAPPER knh_fmapper_vocabidx(Ctx *ctx, knh_sfp_t *sfp)
 {
-	knh_ClassSpec_t *u = knh_getClassSpec(ctx, knh_Object_cid(sfp[0].o));
+	ClassSpec *u = knh_getClassSpec(ctx, knh_Object_cid(sfp[0].o));
 	knh_int_t n = knh_ClassSpec_getVocabIdx(ctx, u, sfp[0].s);
 	DBG2_P("n = %d", (int)n);
 	KNH_MAPPED_Int(ctx, sfp, n);
@@ -434,10 +432,10 @@ MAPPER knh_fmapper_vocabidx(Ctx *ctx, knh_sfp_t *sfp)
 static
 MAPPER knh_fmapper_vocab(Ctx *ctx, knh_sfp_t *sfp)
 {
-	knh_ClassSpec_t *u = (knh_ClassSpec_t*)DP(sfp[1].mpr)->mapdata;
+	ClassSpec *u = (ClassSpec*)DP(sfp[1].mpr)->mapdata;
 	size_t n = (size_t)(sfp[0].ivalue - DP(u)->imin);
 	DBG2_P("n = %zd", n);
-	knh_Array_t *a = (DP(u)->vocabDictIdx)->terms;
+	Array *a = (DP(u)->vocabDictIdx)->terms;
 	Object *s = KNH_NULL;
 	if(n < knh_Array_size(a)) {
 		s = knh_Array_n(a, n);
@@ -447,21 +445,21 @@ MAPPER knh_fmapper_vocab(Ctx *ctx, knh_sfp_t *sfp)
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(knh_ClassSpec_t*) new_Vocab(Ctx *ctx, char *tag, knh_bytes_t urn, int base, char **terms)
+KNHAPI(ClassSpec*) new_Vocab(Ctx *ctx, char *tag, knh_bytes_t urn, int base, char **terms)
 {
 	knh_class_t cid = knh_ClassTable_newId(ctx);
-	knh_ClassSpec_t* u = (knh_ClassSpec_t*)new_Object_bcid(ctx, CLASS_ClassSpec, (int)cid);
+	ClassSpec* u = (ClassSpec*)new_Object_bcid(ctx, CLASS_ClassSpec, (int)cid);
 	DP(u)->ubcid = CLASS_String;
 	KNH_SETv(ctx, DP(u)->urn, new_String(ctx, urn, NULL));
 	if(tag != NULL || tag[0] != 0) {
-		KNH_SETv(ctx, DP(u)->tag, T__(tag));
+		KNH_SETv(ctx, DP(u)->tag, T(tag));
 	}
 	DP(u)->fsnew = knh_fsnew__dict;
 	DP(u)->fscmp = knh_fscmp__dict;
 	{
-		knh_Array_t *a = new_Array0(ctx, 0);
+		Array *a = new_Array0(ctx, 0);
 		while(*terms != NULL) {
-			knh_String_t *s = T__(*terms);
+			String *s = T(*terms);
 			knh_Array_add(ctx, a, UP(s));
 			s->h.cid = cid;
 			terms++;
@@ -473,9 +471,9 @@ KNHAPI(knh_ClassSpec_t*) new_Vocab(Ctx *ctx, char *tag, knh_bytes_t urn, int bas
 	}
 	knh_addSpecializedType(ctx, cid, CLASS_String, u);
 	{
-		knh_Mapper_t *mpr = new_Mapper(ctx, FLAG_Mapper_Total|FLAG_Mapper_Const|FLAG_Mapper_Final, cid, CLASS_Int, knh_fmapper_vocabidx, (Object*)u);
+		Mapper *mpr = new_Mapper(ctx, KNH_FLAG_MMF_TOTAL|KNH_FLAG_MMF_CONST|KNH_FLAG_MMF_FINAL, cid, CLASS_Int, knh_fmapper_vocabidx, (Object*)u);
 		knh_addMapper(ctx, mpr);
-		mpr = new_Mapper(ctx, FLAG_Mapper_Const|FLAG_Mapper_Final, CLASS_Int, cid, knh_fmapper_vocab, (Object*)u);
+		mpr = new_Mapper(ctx, KNH_FLAG_MMF_CONST|KNH_FLAG_MMF_FINAL, CLASS_Int, cid, knh_fmapper_vocab, (Object*)u);
 		knh_addMapper(ctx, mpr);
 	}
 	return u;
@@ -495,7 +493,7 @@ KNHAPI(knh_ClassSpec_t*) new_Vocab(Ctx *ctx, char *tag, knh_bytes_t urn, int bas
 //	KNH_ASSERT(IS_ClassSpec(u2));
 //	KNH_ASSERT(IS_DictIdx(DP(u2)->vocabDictIdx));
 //
-//	size_t n = (size_t)knh_DictIdx_index(ctx, DP(u)->vocabDictIdx, __tobytes(s));
+//	size_t n = (size_t)knh_DictIdx_index(ctx, DP(u)->vocabDictIdx, knh_String_tobytes(s));
 //	KNH_ASSERT(n < knh_DictIdx_size(DP(u2)->vocabDictIdx));
 //	KNH_MAPPED(ctx, sfp, knh_DictIdx_get__fast(DP(u2)->vocabDictIdx, n));
 //}
@@ -504,7 +502,7 @@ KNHAPI(knh_ClassSpec_t*) new_Vocab(Ctx *ctx, char *tag, knh_bytes_t urn, int bas
 //
 //KNHAPI(void) knh_addVocabularyMapper(Ctx *ctx, knh_class_t scid, char *text)
 //{
-//	DBG2_ASSERT_cid(scid);
+//	KNH_ASSERT_cid(scid);
 //	knh_class_t tcid = knh_findcid(ctx, B(text));
 //
 //	if(tcid != CLASS_unknown || ctx->share->ClassTable[tcid].bcid != tcid) {
@@ -522,7 +520,7 @@ KNHAPI(knh_ClassSpec_t*) new_Vocab(Ctx *ctx, char *tag, knh_bytes_t urn, int bas
 ////			TODO();
 ////			return;
 ////		}
-//		Mapper *mpr = new_Mapper(ctx, FLAG_Mapper_Affine, scid, tcid,
+//		Mapper *mpr = new_Mapper(ctx, KNH_FLAG_MMF_AFFINE, scid, tcid,
 //				knh_Mapper_fvocab, (Object*)KNH_NULL);
 //		if(ctx->share->ClassTable[scid].cmap == NULL) {
 //			knh_ClassTable_t *TC = (knh_ClassTable_t*)(&ctx->share->ClassTable[scid]);
@@ -530,7 +528,7 @@ KNHAPI(knh_ClassSpec_t*) new_Vocab(Ctx *ctx, char *tag, knh_bytes_t urn, int bas
 //		}
 //		knh_ClassMap_add(ctx, ctx->share->ClassTable[scid].cmap, mpr);
 //
-//		mpr = new_Mapper(ctx, FLAG_Mapper_Affine, tcid, scid,
+//		mpr = new_Mapper(ctx, KNH_FLAG_MMF_AFFINE, tcid, scid,
 //				knh_Mapper_fvocab, (Object*)KNH_NULL);
 //		if(ctx->share->ClassTable[tcid].cmap == NULL) {
 //			knh_ClassTable_t *TC = (knh_ClassTable_t*)(&ctx->share->ClassTable[tcid]);
@@ -545,9 +543,9 @@ KNHAPI(knh_ClassSpec_t*) new_Vocab(Ctx *ctx, char *tag, knh_bytes_t urn, int bas
 
 knh_bytes_t knh_getURNAlias(Ctx *ctx, knh_bytes_t aurn)
 {
-	knh_String_t *s = (knh_String_t*)knh_DictMap_get__b(ctx,  DP(ctx->sys)->URNAliasDictMap, aurn);
+	String *s = (String*)knh_DictMap_get__b(ctx,  DP(ctx->sys)->URNAliasDictMap, aurn);
 	if(IS_NOTNULL(s)) {
-		return __tobytes(s);
+		return knh_String_tobytes(s);
 	}
 	return aurn;
 }
@@ -557,9 +555,9 @@ knh_bytes_t knh_getURNAlias(Ctx *ctx, knh_bytes_t aurn)
 //void knh_setAliasURN(Ctx *ctx, String *alias, String *urn)
 //{
 //	if(knh_Context_isVerbose(ctx)) {
-//		String *s =(String*)knh_DictMap_get__b(ctx,  DP(ctx->sys)->URNAliasDictMap, __tobytes(urn));
+//		String *s =(String*)knh_DictMap_get__b(ctx,  DP(ctx->sys)->URNAliasDictMap, knh_String_tobytes(urn));
 //		if(IS_NOTNULL(s)) {
-//			KNH_WARNING(ctx, "Overriding %s %s", __tochar(alias), __tochar(s));
+//			KNH_WARNING(ctx, "Overriding %s %s", knh_String_tochar(alias), knh_String_tochar(s));
 //		}
 //	}
 //	knh_DictMap_set(ctx, DP(ctx->sys)->URNAliasDictMap, alias, UP(urn));
@@ -569,19 +567,19 @@ knh_bytes_t knh_getURNAlias(Ctx *ctx, knh_bytes_t aurn)
 
 KNHAPI(void) knh_loadURNAliasData(Ctx *ctx, knh_StringConstData_t *data)
 {
-	knh_DictMap_t *map = DP(ctx->sys)->URNAliasDictMap;
+	DictMap *map = DP(ctx->sys)->URNAliasDictMap;
 	knh_StringConstData_t *d = data;
 	while(d->name != NULL) {
-		knh_String_t *s =(knh_String_t*)knh_DictMap_get__b(ctx,  map, B(d->name));
+		String *s =(String*)knh_DictMap_get__b(ctx,  map, B(d->name));
 		if(IS_NOTNULL(s) && !knh_String_equals(s, B(d->value))) {
-			KNH_WARNING(ctx, _("overriding alias %s %s as %s"), d->name, __tochar(s), d->value);
+			KNH_WARNING(ctx, _("overriding alias %s %s as %s"), d->name, knh_String_tochar(s), d->value);
 		}
 		d++;
 	}
 	d = data;
 	while(d->name != NULL) {
-		knh_String_t *n = T__(d->name);
-		knh_DictMap_append(ctx, map, n, UP(T__(d->value)));
+		String *n = T(d->name);
+		knh_DictMap_append(ctx, map, n, UP(T(d->value)));
 		d++;
 	}
 }
@@ -590,11 +588,11 @@ KNHAPI(void) knh_loadURNAliasData(Ctx *ctx, knh_StringConstData_t *data)
 
 KNHAPI(void) knh_loadClassSpecFuncData(Ctx *ctx, knh_NamedPointerData_t *data)
 {
-	knh_DictSet_t *ds = DP(ctx->sys)->SpecFuncDictSet;
+	DictSet *ds = DP(ctx->sys)->SpecFuncDictSet;
 	KNH_LOCK(ctx, LOCK_SYSTBL, NULL);
 	while(data->name != NULL) {
 		DBG2_P("adding.. '%s'", data->name);
-		knh_String_t *n = T__(data->name);
+		String *n = T(data->name);
 		knh_DictSet_set(ctx, ds, n, (knh_uintptr_t)data->ptr);
 		data++;
 	}
@@ -604,9 +602,9 @@ KNHAPI(void) knh_loadClassSpecFuncData(Ctx *ctx, knh_NamedPointerData_t *data)
 /* ------------------------------------------------------------------------ */
 
 static
-knh_ClassSpec_t *new_ClassSpecNULL(Ctx *ctx, knh_bytes_t urn)
+ClassSpec *new_ClassSpecNULL(Ctx *ctx, knh_bytes_t urn)
 {
-	knh_ClassSpec_t *u = NULL;
+	ClassSpec *u = NULL;
 	KNH_LOCK(ctx, LOCK_SYSTBL, NULL);
 	knh_index_t loc = 0;
 	knh_bytes_t p = urn;
@@ -631,9 +629,9 @@ knh_ClassSpec_t *new_ClassSpecNULL(Ctx *ctx, knh_bytes_t urn)
 /* ======================================================================== */
 /* [SPEC] */
 
-knh_ClassSpec_t *knh_getClassSpec(Ctx *ctx, knh_class_t cid)
+ClassSpec *knh_getClassSpec(Ctx *ctx, knh_class_t cid)
 {
-	knh_ClassSpec_t *u = (knh_ClassSpec_t*)ClassTable(cid).cspec;
+	ClassSpec *u = (ClassSpec*)ClassTable(cid).cspec;
 	KNH_ASSERT(IS_ClassSpec(u));
 	return u;
 }
@@ -643,7 +641,7 @@ knh_ClassSpec_t *knh_getClassSpec(Ctx *ctx, knh_class_t cid)
 static
 Object *knh_ClassTable_fdefault__ISPEC(Ctx *ctx, knh_class_t cid)
 {
-	knh_ClassSpec_t *u = knh_getClassSpec(ctx, cid);
+	ClassSpec *u = knh_getClassSpec(ctx, cid);
 	return UP(DP(u)->ivalue);
 }
 
@@ -652,7 +650,7 @@ Object *knh_ClassTable_fdefault__ISPEC(Ctx *ctx, knh_class_t cid)
 static
 Object *knh_ClassTable_fdefault__FSPEC(Ctx *ctx, knh_class_t cid)
 {
-	knh_ClassSpec_t *u = knh_getClassSpec(ctx, cid);
+	ClassSpec *u = knh_getClassSpec(ctx, cid);
 	return UP(DP(u)->fvalue);
 }
 
@@ -661,28 +659,28 @@ Object *knh_ClassTable_fdefault__FSPEC(Ctx *ctx, knh_class_t cid)
 static
 Object *knh_ClassTable_fdefault__SSPEC(Ctx *ctx, knh_class_t cid)
 {
-	knh_ClassSpec_t *u = knh_getClassSpec(ctx, cid);
+	ClassSpec *u = knh_getClassSpec(ctx, cid);
 	return UP(DP(u)->svalue);
 }
 
 /* ------------------------------------------------------------------------ */
 
 static
-knh_class_t knh_addSpecializedType(Ctx *ctx, knh_class_t cid, knh_class_t bcid, knh_ClassSpec_t *u)
+knh_class_t knh_addSpecializedType(Ctx *ctx, knh_class_t cid, knh_class_t bcid, ClassSpec *u)
 {
 	char bufcn[CLASSNAME_BUFSIZ];
 	if(cid == CLASS_newid) {
 		cid = knh_ClassTable_newId(ctx);
 	}
-	knh_snprintf(bufcn, sizeof(bufcn), KNH_CLASSSPEC_FMT, CLASSN(bcid), __tochar(DP(u)->urn));
+	knh_snprintf(bufcn, sizeof(bufcn), KNH_CLASSSPEC_FMT, CLASSN(bcid), knh_String_tochar(DP(u)->urn));
 	knh_setClassName(ctx, cid, new_String(ctx, B(bufcn), NULL));
 	{
 		knh_ClassTable_t *t = pClassTable(cid);
 		if((DP(u)->tag)->size > 0) {
-			knh_snprintf(bufcn, sizeof(bufcn), "%s:%s", CLASSN(bcid), __tochar(DP(u)->tag));
+			knh_snprintf(bufcn, sizeof(bufcn), "%s:%s", CLASSN(bcid), knh_String_tochar(DP(u)->tag));
 			KNH_SETv(ctx, t->sname, new_String(ctx, B(bufcn), NULL));
 		}
-		DBG2_P("added %d, '%s'", cid, __tochar(t->lname));
+		DBG2_P("added %d, '%s'", cid, knh_String_tochar(t->lname));
 
 		t->bcid   = bcid;
 		t->supcid = bcid;
@@ -692,7 +690,7 @@ knh_class_t knh_addSpecializedType(Ctx *ctx, knh_class_t cid, knh_class_t bcid,
 		//	else if(bcid == CLASS_String) bcid = CLASS_StringX;
 
 		//DBG2_P("%s\n\tcopying from %s", bufcn, CLASSN(bcid));
-		t->cflag  = ClassTable(bcid).cflag | FLAG_Class_Final;
+		t->cflag  = ClassTable(bcid).cflag | KNH_FLAG_CF_FINAL;
 		t->oflag  = ClassTable(bcid).oflag;
 		t->offset = ClassTable(bcid).offset;
 
@@ -736,7 +734,7 @@ knh_class_t knh_addSpecializedType(Ctx *ctx, knh_class_t cid, knh_class_t bcid,
 /* ------------------------------------------------------------------------ */
 
 static
-knh_ClassSpec_t* knh_findClassSpecNULL(Ctx *ctx, knh_bytes_t lname)
+ClassSpec* knh_findClassSpecNULL(Ctx *ctx, knh_bytes_t lname)
 {
 	knh_index_t loc = knh_bytes_index(lname, '{');
 	if(loc != -1) {
@@ -765,7 +763,7 @@ knh_ClassSpec_t* knh_findClassSpecNULL(Ctx *ctx, knh_bytes_t lname)
 
 knh_class_t knh_findcidx(Ctx *ctx, knh_bytes_t lname)
 {
-	knh_ClassSpec_t *u = 	knh_findClassSpecNULL(ctx, lname);
+	ClassSpec *u = 	knh_findClassSpecNULL(ctx, lname);
 	if(u != NULL) {
 		knh_index_t loc = knh_bytes_index(lname, '{');
 		knh_class_t bcid = knh_getcid(ctx, knh_bytes_first(lname, loc));
