@@ -51,6 +51,7 @@ typedef struct {
 
 /* ------------------------------------------------------------------------ */
 
+static
 size_t knh_dict_capacity(knh_dict_t *a)
 {
 	if(a == NULL) {
@@ -100,6 +101,7 @@ knh_dict_t *knh_dict_malloc(Ctx *ctx, size_t capacity,
 	if(!(capacity > 0)) {
 		capacity = (KONOHA_SMALLPAGESIZE - sizeof(knh_hdict_t)) / sizeof(knh_dict_t);
 	}
+
 	h = (knh_hdict_t*)KNH_MALLOC(ctx, (capacity * sizeof(knh_dict_t)) + sizeof(knh_hdict_t));
 	h->capacity = capacity;
 	h->sorted = 0;
@@ -190,7 +192,7 @@ void knh_dict_sort(knh_dict_t *a, size_t size)
 	if(a != NULL) {
 		knh_hdict_t *h = ((knh_hdict_t*)a) - 1;
 		if(h->fdict_compar != NULL) {
-			knh_qsort(a, size, sizeof(knh_dict_t), h->fdict_compar);
+			knh_sort(a, size, sizeof(knh_dict_t), h->fdict_compar);
 			h->sorted = size;
 		}
 		else {
@@ -211,13 +213,13 @@ knh_index_t knh_dict_findb_between(knh_dict_t *a, knh_fbytescmp fcmp, knh_bytes_
 	if(ep - sp < UNSORTED_MAXSIZ) {
 		size_t i;
 		for(i = sp; i < ep; i++) {
-			if(fcmp(__tobytes(a[i].key), key) == 0) return i;
+			if(fcmp(knh_String_tobytes(a[i].key), key) == 0) return i;
 		}
 		return -1;
 	}
 	else {
 		size_t cp = KNH_MID(sp, ep);
-		int i = fcmp(key, __tobytes(a[cp].key));
+		int i = fcmp(key, knh_String_tobytes(a[cp].key));
 		if(i == 0) {
 			return cp;
 		}
@@ -251,10 +253,9 @@ knh_index_t knh_dict_index_b(knh_dict_t *a, size_t size, knh_fbytescmp fcmp, knh
 	}
 }
 
-
 /* ------------------------------------------------------------------------ */
 
-void knh_DictMap_sort(knh_DictMap_t *o)
+void knh_DictMap_sort(DictMap *o)
 {
 	if(knh_dict_sorted(o->_list) != o->size) {
 		knh_dict_sort(o->_list, o->size);
@@ -263,7 +264,7 @@ void knh_DictMap_sort(knh_DictMap_t *o)
 
 /* ------------------------------------------------------------------------ */
 
-void knh_DictSet_sort(knh_DictSet_t *o)
+void knh_DictSet_sort(DictSet *o)
 {
 	if(knh_dict_sorted(o->_list) != o->size) {
 		knh_dict_sort(o->_list, o->size);
@@ -316,7 +317,7 @@ int knh_dict_fcmp(const void *p, const void *p2)
 {
 	knh_dict_t *e = (knh_dict_t*)p;
 	knh_dict_t *e2 = (knh_dict_t*)p2;
-	return knh_bytes_strcmp(__tobytes(e->key), __tobytes(e2->key));
+	return knh_bytes_strcmp(knh_String_tobytes(e->key), knh_String_tobytes(e2->key));
 }
 
 /* ------------------------------------------------------------------------ */
@@ -326,7 +327,7 @@ int knh_dict_fcasecmp(const void *p, const void *p2)
 {
 	knh_dict_t *e = (knh_dict_t*)p;
 	knh_dict_t *e2 = (knh_dict_t*)p2;
-	return knh_bytes_strcasecmp(__tobytes(e->key), __tobytes(e2->key));
+	return knh_bytes_strcasecmp(knh_String_tobytes(e->key), knh_String_tobytes(e2->key));
 }
 
 /* ------------------------------------------------------------------------ */
@@ -342,7 +343,7 @@ void knh_dict_tocase(knh_dict_t *a)
 
 /* ------------------------------------------------------------------------ */
 
-void knh_DictMap_toIgnoreCase(knh_DictMap_t *o)
+void knh_DictMap_toIgnoreCase(DictMap *o)
 {
 	knh_dict_tocase(o->_list);
 	o->fcmp = knh_bytes_strcasecmp;
@@ -351,7 +352,7 @@ void knh_DictMap_toIgnoreCase(knh_DictMap_t *o)
 
 /* ------------------------------------------------------------------------ */
 
-void knh_DictSet_toIgnoreCase(knh_DictSet_t *o)
+void knh_DictSet_toIgnoreCase(DictSet *o)
 {
 	knh_dict_tocase(o->_list);
 	o->fcmp = knh_bytes_strcasecmp;
@@ -368,7 +369,7 @@ knh_dict_t *knh_dictmap_malloc(Ctx *ctx, size_t init)
 
 /* ------------------------------------------------------------------------ */
 
-knh_DictMap_t* new_DictMap0(Ctx *ctx, knh_intptr_t capacity)
+DictMap* new_DictMap0(Ctx *ctx, knh_intptr_t capacity)
 {
 	knh_DictMap_t *o = (knh_DictMap_t*)new_hObject(ctx, FLAG_DictMap, CLASS_DictMap, CLASS_DictMap);
 	o->_list = knh_dict_malloc(ctx, capacity, knh_dictmap_finit, knh_dictmap_ftraverse, knh_dict_fcmp);
@@ -379,9 +380,9 @@ knh_DictMap_t* new_DictMap0(Ctx *ctx, knh_intptr_t capacity)
 
 /* ------------------------------------------------------------------------ */
 
-knh_DictMap_t* new_DictMap(Ctx *ctx, knh_class_t p1, knh_intptr_t capacity)
+DictMap* new_DictMap(Ctx *ctx, knh_class_t p1, knh_intptr_t capacity)
 {
-	knh_class_t cid = knh_class_Generics(ctx, CLASS_DictMap, p1, CLASS_Tvoid);
+	knh_class_t cid = knh_class_Generics(ctx, CLASS_DictMap, p1, CLASS_unknown);
 	knh_DictMap_t *o = (knh_DictMap_t*)new_hObject(ctx, FLAG_DictMap, CLASS_DictMap, cid);
 	o->_list = knh_dict_malloc(ctx, capacity, knh_dictmap_finit, knh_dictmap_ftraverse, knh_dict_fcmp);
 	o->size = 0;
@@ -392,14 +393,14 @@ knh_DictMap_t* new_DictMap(Ctx *ctx, knh_class_t p1, knh_intptr_t capacity)
 /* ======================================================================== */
 /* [methods] */
 
-size_t knh_DictMap_size(knh_DictMap_t *o)
+size_t knh_DictMap_size(DictMap *o)
 {
 	return o->size;
 }
 
 /* ------------------------------------------------------------------------ */
 
-knh_String_t* knh_DictMap_keyAt(knh_DictMap_t *o, size_t n)
+String* knh_DictMap_keyAt(DictMap *o, size_t n)
 {
 	KNH_ASSERT(IS_bDictMap(o));
 	KNH_ASSERT(n < o->size);
@@ -408,7 +409,7 @@ knh_String_t* knh_DictMap_keyAt(knh_DictMap_t *o, size_t n)
 
 /* ------------------------------------------------------------------------ */
 
-Object* knh_DictMap_valueAt(knh_DictMap_t *o, size_t n)
+Object* knh_DictMap_valueAt(DictMap *o, size_t n)
 {
 	KNH_ASSERT(IS_bDictMap(o));
 	KNH_ASSERT(n < o->size);
@@ -417,7 +418,7 @@ Object* knh_DictMap_valueAt(knh_DictMap_t *o, size_t n)
 
 /* ------------------------------------------------------------------------ */
 
-void knh_DictMap_removeAt(Ctx *ctx, knh_DictMap_t *o, size_t n)
+void knh_DictMap_removeAt(Ctx *ctx, DictMap *o, size_t n)
 {
 	KNH_ASSERT(IS_bDictMap(o));
 	KNH_ASSERT(n < o->size);
@@ -426,7 +427,7 @@ void knh_DictMap_removeAt(Ctx *ctx, knh_DictMap_t *o, size_t n)
 
 /* ------------------------------------------------------------------------ */
 
-knh_index_t knh_DictMap_index(knh_DictMap_t *o, knh_bytes_t key)
+knh_index_t knh_DictMap_index__b(DictMap *o, knh_bytes_t key)
 {
 	KNH_ASSERT(IS_bDictMap(o));
 	return knh_dict_index_b(o->_list, o->size, o->fcmp, key);
@@ -434,45 +435,7 @@ knh_index_t knh_DictMap_index(knh_DictMap_t *o, knh_bytes_t key)
 
 /* ------------------------------------------------------------------------ */
 
-static
-knh_index_t knh_dict_first(knh_dict_t *a, knh_fbytescmp fcmp, knh_bytes_t key, size_t sp, size_t ep)
-{
-	L_TAIL:;
-	if(ep - sp < UNSORTED_MAXSIZ) {
-		size_t i;
-		for(i = sp; i < ep; i++) {
-			int res = fcmp(__tobytes(a[i].key), key);
-			if(res >= 0) return i;
-		}
-		return -1;
-	}
-	else {
-		size_t cp = KNH_MID(sp, ep);
-		int i = fcmp(key, __tobytes(a[cp].key));
-		if(i == 0) {
-			return cp;
-		}
-		else if(i < 0) {
-			ep = cp;
-		}else {
-			sp = cp + 1;
-		}
-		goto L_TAIL;
-	}
-}
-
-/* ------------------------------------------------------------------------ */
-
-knh_index_t knh_DictMap_firstIndex(knh_DictMap_t *o, knh_bytes_t key)
-{
-	KNH_ASSERT(IS_bDictMap(o));
-	knh_DictMap_sort(o);
-	return knh_dict_first(o->_list, o->fcmp, key, 0, o->size);
-}
-
-/* ------------------------------------------------------------------------ */
-
-knh_String_t *new_String__DictMap(Ctx *ctx, knh_DictMap_t *o, knh_bytes_t key)
+String *new_String__DictMap(Ctx *ctx, DictMap *o, knh_bytes_t key)
 {
 	KNH_ASSERT(IS_bDictMap(o));
 	knh_index_t loc = knh_dict_index_b(o->_list, o->size, o->fcmp, key);
@@ -486,7 +449,7 @@ knh_String_t *new_String__DictMap(Ctx *ctx, knh_DictMap_t *o, knh_bytes_t key)
 
 /* ------------------------------------------------------------------------ */
 
-Object *knh_DictMap_get__b(Ctx *ctx, knh_DictMap_t *o, knh_bytes_t key)
+Object *knh_DictMap_get__b(Ctx *ctx, DictMap *o, knh_bytes_t key)
 {
 	KNH_ASSERT(IS_bDictMap(o));
 	knh_intptr_t loc = knh_dict_index_b(o->_list, o->size, o->fcmp, key);
@@ -495,16 +458,16 @@ Object *knh_DictMap_get__b(Ctx *ctx, knh_DictMap_t *o, knh_bytes_t key)
 
 /* ------------------------------------------------------------------------ */
 
-Object *knh_DictMap_get(Ctx *ctx, knh_DictMap_t *o, knh_String_t *key)
+Object *knh_DictMap_get(Ctx *ctx, DictMap *o, String *key)
 {
 	KNH_ASSERT(IS_bDictMap(o));
-	knh_intptr_t loc = knh_dict_index_b(o->_list, o->size, o->fcmp, __tobytes(key));
+	knh_intptr_t loc = knh_dict_index_b(o->_list, o->size, o->fcmp, knh_String_tobytes(key));
 	return (loc == -1) ? KNH_NULL : o->list[loc].value;
 }
 
 /* ------------------------------------------------------------------------ */
 
-void knh_DictMap_append(Ctx *ctx, knh_DictMap_t *o, knh_String_t *key, Any *value)
+void knh_DictMap_append(Ctx *ctx, DictMap *o, String *key, Any *value)
 {
 	KNH_ASSERT(IS_bDictMap(o));
 	if(o->_list == NULL) {
@@ -524,10 +487,10 @@ void knh_DictMap_append(Ctx *ctx, knh_DictMap_t *o, knh_String_t *key, Any *valu
 
 /* ------------------------------------------------------------------------ */
 
-void knh_DictMap_set(Ctx *ctx, knh_DictMap_t *o, knh_String_t *key, Any *v)
+void knh_DictMap_set(Ctx *ctx, DictMap *o, String *key, Any *v)
 {
 	knh_Object_t *value = v;
-	int loc = knh_dict_index_b(o->_list, o->size, o->fcmp, __tobytes(key));
+	int loc = knh_dict_index_b(o->_list, o->size, o->fcmp, knh_String_tobytes(key));
 	if(loc != -1) {
 		KNH_SETv(ctx, o->list[loc].key, key);  /* To avoid losing key */
 		KNH_SETv(ctx, o->list[loc].value, value);
@@ -539,24 +502,18 @@ void knh_DictMap_set(Ctx *ctx, knh_DictMap_t *o, knh_String_t *key, Any *v)
 
 /* ------------------------------------------------------------------------ */
 
-void knh_DictMap_remove(Ctx *ctx, knh_DictMap_t *o, knh_String_t *key)
+void knh_DictMap_remove(Ctx *ctx, DictMap *o, String *key)
 {
-	int loc = knh_dict_index_b(o->_list, o->size, o->fcmp, __tobytes(key));
-	if(loc != -1) {
-		int i;
-		for(i = loc; i < o->size - 1; i++) {
-			KNH_SETv(ctx, o->list[i].key,   o->list[i+1].key);
-			KNH_SETv(ctx, o->list[i].value, o->list[i+1].value);
-		}
-		KNH_SETv(ctx, o->list[i].key,   KNH_NULL);
-		KNH_SETv(ctx, o->list[i].value, KNH_NULL);
-		o->size--;
+	int loc = knh_dict_index_b(o->_list, o->size, o->fcmp, knh_String_tobytes(key));
+	if(loc != 1) {
+		KNH_SETv(ctx, o->list[loc].key, key);  /* To avoid losing key */
+		KNH_SETv(ctx, o->list[loc].value, KNH_NULL);
 	}
 }
 
 /* ------------------------------------------------------------------------ */
 
-void knh_DictMap_clear(Ctx *ctx, knh_DictMap_t *o)
+void knh_DictMap_clear(Ctx *ctx, DictMap *o)
 {
 	size_t i, capacity = knh_dict_capacity(o->_list);
 	for(i = 0; i < capacity; i++) {
@@ -577,7 +534,9 @@ knh_dict_t *knh_dictset_malloc(Ctx *ctx, size_t init)
 
 /* ------------------------------------------------------------------------ */
 
-knh_DictSet_t* new_DictSet0(Ctx *ctx, knh_intptr_t capacity)
+#define _new_DictSet(ctx, c)   new_DictSet0(ctx, c)
+
+DictSet* new_DictSet0(Ctx *ctx, knh_intptr_t capacity)
 {
 	knh_DictSet_t *o = (knh_DictSet_t*)new_hObject(ctx, FLAG_DictSet, CLASS_DictSet, CLASS_DictSet);
 	o->_list = knh_dict_malloc(ctx, capacity, knh_dictset_finit, knh_dictset_ftraverse, knh_dict_fcmp);
@@ -589,14 +548,14 @@ knh_DictSet_t* new_DictSet0(Ctx *ctx, knh_intptr_t capacity)
 /* ======================================================================== */
 /* [methods] */
 
-size_t knh_DictSet_size(knh_DictSet_t *o)
+size_t knh_DictSet_size(DictSet *o)
 {
 	return o->size;
 }
 
 /* ------------------------------------------------------------------------ */
 
-knh_String_t* knh_DictSet_keyAt(knh_DictSet_t *o, size_t n)
+String* knh_DictSet_keyAt(DictSet *o, size_t n)
 {
 	KNH_ASSERT(n < o->size);
 	return o->list[n].key;
@@ -604,7 +563,7 @@ knh_String_t* knh_DictSet_keyAt(knh_DictSet_t *o, size_t n)
 
 /* ------------------------------------------------------------------------ */
 
-knh_uintptr_t knh_DictSet_valueAt(knh_DictSet_t *o, size_t n)
+knh_uintptr_t knh_DictSet_valueAt(DictSet *o, size_t n)
 {
 	KNH_ASSERT(n < o->size);
 	return o->list[n].value;
@@ -612,7 +571,7 @@ knh_uintptr_t knh_DictSet_valueAt(knh_DictSet_t *o, size_t n)
 
 /* ------------------------------------------------------------------------ */
 
-void knh_DictSet_removeAt(Ctx *ctx, knh_DictSet_t *o, size_t n)
+void knh_DictSet_removeAt(Ctx *ctx, DictSet *o, size_t n)
 {
 	KNH_ASSERT(n < o->size);
 	o->list[n].value = 0;
@@ -620,7 +579,7 @@ void knh_DictSet_removeAt(Ctx *ctx, knh_DictSet_t *o, size_t n)
 
 /* ------------------------------------------------------------------------ */
 
-knh_String_t *new_String__DictSet(Ctx *ctx, knh_DictSet_t *o, knh_bytes_t key)
+String *new_String__DictSet(Ctx *ctx, DictSet *o, knh_bytes_t key)
 {
 	knh_index_t loc = knh_dict_index_b(o->_list, o->size, o->fcmp, key);
 	if(loc == -1) {
@@ -633,7 +592,7 @@ knh_String_t *new_String__DictSet(Ctx *ctx, knh_DictSet_t *o, knh_bytes_t key)
 
 /* ------------------------------------------------------------------------ */
 
-knh_uintptr_t knh_DictSet_get__b(knh_DictSet_t *o, knh_bytes_t key)
+knh_uintptr_t knh_DictSet_get__b(DictSet *o, knh_bytes_t key)
 {
 	knh_intptr_t loc = knh_dict_index_b(o->_list, o->size, o->fcmp, key);
 	return (loc == -1) ? 0 : o->list[loc].value;
@@ -641,15 +600,15 @@ knh_uintptr_t knh_DictSet_get__b(knh_DictSet_t *o, knh_bytes_t key)
 
 /* ------------------------------------------------------------------------ */
 
-knh_uintptr_t knh_DictSet_get(Ctx *ctx, knh_DictSet_t *o, knh_String_t *key)
+knh_uintptr_t knh_DictSet_get(Ctx *ctx, DictSet *o, String *key)
 {
-	knh_intptr_t loc = knh_dict_index_b(o->_list, o->size, o->fcmp, __tobytes(key));
+	knh_intptr_t loc = knh_dict_index_b(o->_list, o->size, o->fcmp, knh_String_tobytes(key));
 	return (loc == -1) ? 0 : o->list[loc].value;
 }
 
 /* ------------------------------------------------------------------------ */
 
-void knh_DictSet_append(Ctx *ctx, knh_DictSet_t *o, knh_String_t *key, knh_uintptr_t n)
+void knh_DictSet_append(Ctx *ctx, DictSet *o, String *key, knh_uintptr_t n)
 {
 	if(o->_list == NULL) {
 		o->_list = knh_dict_malloc(ctx, 8, knh_dictmap_finit, knh_dictmap_ftraverse, knh_dict_fcmp);
@@ -668,9 +627,9 @@ void knh_DictSet_append(Ctx *ctx, knh_DictSet_t *o, knh_String_t *key, knh_uintp
 
 /* ------------------------------------------------------------------------ */
 
-void knh_DictSet_set(Ctx *ctx, knh_DictSet_t *o, knh_String_t *key, knh_uintptr_t n)
+void knh_DictSet_set(Ctx *ctx, DictSet *o, String *key, knh_uintptr_t n)
 {
-	int loc = knh_dict_index_b(o->_list, o->size, o->fcmp, __tobytes(key));
+	int loc = knh_dict_index_b(o->_list, o->size, o->fcmp, knh_String_tobytes(key));
 	if(loc != -1) {
 		KNH_SETv(ctx, o->list[loc].key, key);  /* To avoid losing key */
 		o->list[loc].value = n;
@@ -682,9 +641,9 @@ void knh_DictSet_set(Ctx *ctx, knh_DictSet_t *o, knh_String_t *key, knh_uintptr_
 
 /* ------------------------------------------------------------------------ */
 
-void knh_DictSet_add(Ctx *ctx, knh_DictSet_t *o, knh_String_t *key)
+void knh_DictSet_add(Ctx *ctx, DictSet *o, String *key)
 {
-	int loc = knh_dict_index_b(o->_list, o->size, o->fcmp, __tobytes(key));
+	int loc = knh_dict_index_b(o->_list, o->size, o->fcmp, knh_String_tobytes(key));
 	if(loc != -1) {
 		KNH_SETv(ctx, o->list[loc].key, key);  /* To avoid losing key */
 		o->list[loc].value += 1;
@@ -696,10 +655,10 @@ void knh_DictSet_add(Ctx *ctx, knh_DictSet_t *o, knh_String_t *key)
 
 /* ------------------------------------------------------------------------ */
 
-void knh_DictSet_remove(Ctx *ctx, knh_DictSet_t *o, knh_String_t *key)
+void knh_DictSet_remove(Ctx *ctx, DictSet *o, String *key)
 {
-	int loc = knh_dict_index_b(o->_list, o->size, o->fcmp, __tobytes(key));
-	if(loc != -1) {
+	int loc = knh_dict_index_b(o->_list, o->size, o->fcmp, knh_String_tobytes(key));
+	if(loc != 1) {
 		KNH_SETv(ctx, o->list[loc].key, key);  /* To avoid losing key */
 		o->list[loc].value = 0;
 	}
@@ -707,7 +666,7 @@ void knh_DictSet_remove(Ctx *ctx, knh_DictSet_t *o, knh_String_t *key)
 
 /* ------------------------------------------------------------------------ */
 
-void knh_DictSet_clear(Ctx *ctx, knh_DictSet_t *o)
+void knh_DictSet_clear(Ctx *ctx, DictSet *o)
 {
 	size_t i, capacity = knh_dict_capacity(o->_list);
 	for(i = 0; i < capacity; i++) {
@@ -717,39 +676,7 @@ void knh_DictSet_clear(Ctx *ctx, knh_DictSet_t *o)
 	o->size = 0;
 	knh_dict_setsorted(o->_list, 0);
 }
-/* ------------------------------------------------------------------------ */
-//## method String[] DictMap.keys();
 
-METHOD DictMap_keys(Ctx *ctx, knh_sfp_t *sfp)
-{
-	// TODO
-	// we must test this method.
-	knh_DictMap_t *o = (knh_DictMap_t*)sfp[0].o;
-	knh_Array_t *a = new_Array(ctx, CLASS_String, o->size);
-	int i;
-	for(i=0; i<o->size; i++) {
-		knh_Array_add(ctx, a, UP(o->list[i].key));
-		//fprintf(stderr, "%d:%s\n",i,o->list[i].key->str);
-	}
-	KNH_RETURN(ctx, sfp, UP(a));
-}
-
-/* ------------------------------------------------------------------------ */
-//## method Any[] DictMap.values();
-
-METHOD DictMap_values(Ctx *ctx, knh_sfp_t *sfp)
-{
-	// TODO
-	// we must test this method.
-	knh_DictMap_t *o = (knh_DictMap_t*)sfp[0].o;
-	knh_Array_t *a = new_Array0(ctx, o->size);
-	int i;
-	for(i=0; i<o->size; i++) {
-		knh_Array_add(ctx, a, UP(o->list[i].value));
-		//fprintf(stderr, "%d:%p\n",i,o->list[i].value);
-	}
-	KNH_RETURN(ctx, sfp, UP(a));
-}
 /* ------------------------------------------------------------------------ */
 
 #ifdef __cplusplus
