@@ -38,9 +38,9 @@ extern "C" {
 /* ======================================================================== */
 /* [Bytes] */
 
-KNHAPI(knh_InputStream_t*) new_InputStream__io(Ctx *ctx, knh_String_t *urn, knh_io_t fd, knh_iodrv_t *drv)
+KNHAPI(InputStream*) new_InputStream__io(Ctx *ctx, String *urn, knh_io_t fd, knh_iodrv_t *drv)
 {
-	knh_InputStream_t* o = (knh_InputStream_t*)new_Object_bcid(ctx, CLASS_InputStream, 0);
+	InputStream* o = (InputStream*)new_Object_bcid(ctx, CLASS_InputStream, 0);
 	KNH_SETv(ctx, DP(o)->urn, urn);
 	DP(o)->fd = fd;
 	if(fd != -1) {
@@ -62,9 +62,9 @@ KNHAPI(knh_InputStream_t*) new_InputStream__io(Ctx *ctx, knh_String_t *urn, knh_
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(knh_InputStream_t*) new_InputStream__FILE(Ctx *ctx, knh_String_t *urn, FILE *fp, knh_iodrv_t *drv)
+KNHAPI(InputStream*) new_InputStream__FILE(Ctx *ctx, String *urn, FILE *fp, knh_iodrv_t *drv)
 {
-	knh_InputStream_t* o = (knh_InputStream_t*)new_Object_bcid(ctx, CLASS_InputStream, 0);
+	InputStream* o = (InputStream*)new_Object_bcid(ctx, CLASS_InputStream, 0);
 	KNH_SETv(ctx, DP(o)->urn, urn);
 	if(fp != NULL) {
 		DP(o)->fp = fp;
@@ -77,9 +77,9 @@ KNHAPI(knh_InputStream_t*) new_InputStream__FILE(Ctx *ctx, knh_String_t *urn, FI
 /* ======================================================================== */
 /* [methods] */
 
-Object *knh_InputStream_open(Ctx *ctx, knh_InputStream_t *o, knh_String_t *urn, knh_String_t *m)
+Object *knh_InputStream_open(Ctx *ctx, InputStream *o, String *urn, String *m)
 {
-	knh_bytes_t fname = __tobytes(urn);
+	knh_bytes_t fname = knh_String_tobytes(urn);
 	knh_index_t loc = knh_bytes_index(fname, ':');
 	if(loc == -1 || (loc == 1 && isalpha(fname.buf[0]))) {  /* 'C:/' */
 		DP(o)->driver = knh_getIODriver(ctx, STEXT("file"));
@@ -89,7 +89,7 @@ Object *knh_InputStream_open(Ctx *ctx, knh_InputStream_t *o, knh_String_t *urn,
 	}
 	KNH_SETv(ctx, DP(o)->urn, new_String(ctx, fname, NULL));
 	char *mode = "r";
-	if(IS_NOTNULL(m)) mode = __tochar(m);
+	if(IS_NOTNULL(m)) mode = knh_String_tochar(m);
 	DP(o)->fd = DP(o)->driver->fopen(ctx, fname, mode, knh_Context_isStrict(ctx));
 	if(DP(o)->fd != -1) {
 		KNH_SETv(ctx, DP(o)->urn, urn);
@@ -113,7 +113,7 @@ Object *knh_InputStream_open(Ctx *ctx, knh_InputStream_t *o, knh_String_t *urn,
 
 /* ------------------------------------------------------------------------ */
 
-int knh_InputStream_getc(Ctx *ctx, knh_InputStream_t *o)
+int knh_InputStream_getc(Ctx *ctx, InputStream *o)
 {
 	int ch;
 	if(knh_InputStream_isFILE(o)) {
@@ -147,7 +147,7 @@ int knh_InputStream_getc(Ctx *ctx, knh_InputStream_t *o)
 /* ------------------------------------------------------------------------ */
 
 size_t
-knh_InputStream_read(Ctx *ctx, knh_InputStream_t *o, char *buf, size_t bufsiz)
+knh_InputStream_read(Ctx *ctx, InputStream *o, char *buf, size_t bufsiz)
 {
 	if(knh_InputStream_isFILE(o)) {
 		return knh_fread(ctx, buf, bufsiz, DP(o)->fp);
@@ -162,9 +162,7 @@ knh_InputStream_read(Ctx *ctx, knh_InputStream_t *o, char *buf, size_t bufsiz)
 			}
 			return bufsiz;
 		}
-		// XXX when both InputStream.read and InputStream.readLine method call,
-		// it seams strange. so, move DP(o)->buf's pointer to bufpos.
-		knh_memcpy(buf, DP(o)->buf + DP(o)->bufpos, inbufsiz);
+		knh_memcpy(buf, DP(o)->buf, inbufsiz);
 		DP(o)->bufpos += inbufsiz;
 		DP(o)->size += bufsiz;
 		buf += inbufsiz;
@@ -175,8 +173,9 @@ knh_InputStream_read(Ctx *ctx, knh_InputStream_t *o, char *buf, size_t bufsiz)
 }
 
 /* ------------------------------------------------------------------------ */
+/* @method String InputStream.readLine() */
 
-knh_String_t* knh_InputStream_readLine(Ctx *ctx, knh_InputStream_t *in)
+String* knh_InputStream_readLine(Ctx *ctx, InputStream *in)
 {
 	int ch;
 	knh_cwb_t cwbbuf, *cwb = knh_cwb_open(ctx, &cwbbuf);
@@ -197,12 +196,13 @@ knh_String_t* knh_InputStream_readLine(Ctx *ctx, knh_InputStream_t *in)
 	if(knh_cwb_size(cwb) != 0) {
 		return new_String__cwbconv(ctx, cwb, DP(in)->bconv);
 	}
-	return (knh_String_t*)KNH_NULL;
+	return (String*)KNH_NULL;
 }
 
 /* ------------------------------------------------------------------------ */
+/* @method void InputStream.close() */
 
-void knh_InputStream_close(Ctx *ctx, knh_InputStream_t *o)
+void knh_InputStream_close(Ctx *ctx, InputStream *o)
 {
 	f_io_close f = DP(o)->driver->fclose;
 	DP(o)->driver = knh_getDefaultIODriver();
@@ -213,86 +213,15 @@ void knh_InputStream_close(Ctx *ctx, knh_InputStream_t *o)
 
 /* ------------------------------------------------------------------------ */
 
-int knh_InputStream_isClosed(Ctx *ctx, knh_InputStream_t *o)
+int knh_InputStream_isClosed(Ctx *ctx, InputStream *o)
 {
 	return (DP(o)->fd == -1 && IS_NULL(DP(o)->ba));
 }
 
-/* ------------------------------------------------------------------------ */
-
-void knh_InputStream_setEncoding(Ctx *ctx, knh_InputStream_t *o, knh_String_t *enc)
-{
-	if(IS_NULL(enc)) {
-		KNH_SETv(ctx, DP(o)->enc, TS_ENCODING);
-		KNH_SETv(ctx, DP(o)->bconv, KNH_NULL);
-	}
-	else if(knh_bytes_strcasecmp(__tobytes(enc), STEXT(KONOHA_ENCODING)) == 0) {
-		KNH_SETv(ctx, DP(o)->enc, TS_ENCODING);
-		KNH_SETv(ctx, DP(o)->bconv, KNH_NULL);
-	}
-	else {
-		knh_BytesConv_t *bin = new_BytesConv__in(ctx, __tochar(enc));
-		if(IS_NULL(bin)) {
-			KNH_SETv(ctx, DP(o)->enc, TS_ENCODING);
-			KNH_WARNING(ctx, "unsupported character encoding: %s", __tochar(enc));
-		}
-		else {
-			KNH_SETv(ctx, DP(o)->enc, enc);
-		}
-		KNH_SETv(ctx, DP(o)->bconv, bin);
-	}
-}
-
-/* ------------------------------------------------------------------------ */
-
-#define MAXBUF 256
-knh_bool_t knh_InputStream_checkUTF8(Ctx *ctx, knh_InputStream_t *in)
-{
-	knh_short_t ret = 0;
-	knh_iodrv_t *d = DP(in)->driver;
-	size_t total_size = 0;
-
-	// TODO We have to care about FileInputStream
-	if(!knh_InputStream_isFILE(in)) {
-		TODO();
-		return 1;
-	}
-#if !defined(KNH_USING_NOFILE)
-	while (1) {
-		char buf[MAXBUF] = {0};
-		long pre_pos = ftell(DP(in)->fp);
-		size_t size = knh_InputStream_read(ctx, in, buf, MAXBUF);
-		total_size += size;
-		fseek(DP(in)->fp, total_size, SEEK_SET);
-		buf[MAXBUF-1] = '\0';
-		ret = knh_bytes_checkENCODING(B(buf));
-		if (ret == -1) {
-			fseek(DP(in)->fp, 0L, SEEK_SET);
-			return 0;
-		}
-		else if (size < MAXBUF) {
-			break;
-		}
-		else if (ret > 0) {
-			total_size -= ret;
-			fseek(DP(in)->fp, total_size, SEEK_SET);
-			if (pre_pos == ftell(DP(in)->fp)) {
-				fseek(DP(in)->fp, 0L, SEEK_SET);
-				return 0;
-			}
-		}
-	}
-	/* Reopen InputStream */
-	d->fclose(ctx, DP(in)->fd);
-	d->fopen(ctx, __tobytes(DP(in)->urn), "r", 0/*isThrowable*/);
-#endif
-	return 1;
-}
-
 /* ======================================================================== */
 /* [File] */
 
-KNHAPI(knh_InputStream_t*) new_FileInputStream(Ctx *ctx, knh_bytes_t file, int isThrowable)
+KNHAPI(InputStream*) new_FileInputStream(Ctx *ctx, knh_bytes_t file, int isThrowable)
 {
 	knh_iodrv_t *drv = knh_getIODriver(ctx, STEXT("file"));
 	knh_io_t fd = drv->fopen(ctx, file, "r", isThrowable);
@@ -302,9 +231,9 @@ KNHAPI(knh_InputStream_t*) new_FileInputStream(Ctx *ctx, knh_bytes_t file, int i
 /* ======================================================================== */
 /* [Bytes] */
 
-KNHAPI(knh_InputStream_t*) new_BytesInputStream(Ctx *ctx, knh_Bytes_t *ba, size_t s, size_t e)
+KNHAPI(InputStream*) new_BytesInputStream(Ctx *ctx, Bytes *ba, size_t s, size_t e)
 {
-	knh_InputStream_t* o = (knh_InputStream_t*)new_Object_bcid(ctx, CLASS_InputStream, 0);
+	InputStream* o = (InputStream*)new_Object_bcid(ctx, CLASS_InputStream, 0);
 	DP(o)->fd = -1;
 	KNH_ASSERT(IS_Bytes(ba));
 	KNH_SETv(ctx, DP(o)->ba, ba);
@@ -322,28 +251,95 @@ KNHAPI(knh_InputStream_t*) new_BytesInputStream(Ctx *ctx, knh_Bytes_t *ba, size_
 
 MAPPER knh_Bytes_InputStream(Ctx *ctx, knh_sfp_t *sfp)
 {
-	knh_Bytes_t *ba = (knh_Bytes_t*)sfp[0].o;
+	Bytes *ba = (Bytes*)sfp[0].o;
 	KNH_MAPPED(ctx, sfp, new_BytesInputStream(ctx, ba, 0, knh_Bytes_size(ba)));
 }
 
 /* ======================================================================== */
 /* [String] */
 
-KNHAPI(knh_InputStream_t*) new_StringInputStream(Ctx *ctx, knh_String_t *str, size_t s, size_t e)
+KNHAPI(InputStream*) new_StringInputStream(Ctx *ctx, String *str, size_t s, size_t e)
 {
-	knh_InputStream_t* o = (knh_InputStream_t*)new_Object_bcid(ctx, CLASS_InputStream, 0);
+	InputStream* o = (InputStream*)new_Object_bcid(ctx, CLASS_InputStream, 0);
 	DP(o)->fd = -1;
-	DBG2_ASSERT(IS_Bytes(str));
+	KNH_ASSERT(IS_Bytes(str));
 	KNH_SETv(ctx, DP(o)->str, str);
 	DP(o)->buf = (char*)(str)->str;
 	DP(o)->bufsiz = (str)->size;
-	DBG2_ASSERT(e <= (str)->size);
-	DBG2_ASSERT(s <= e);
+	KNH_ASSERT(e <= (str)->size);
+	KNH_ASSERT(s <= e);
 	DP(o)->bufpos   = s;
 	DP(o)->bufend   = e;
 	return o;
 }
 
+/* ======================================================================== */
+/* [Data] */
+
+int knh_bytes_checkStmtLine(knh_bytes_t line)
+{
+	char *ln = (char*)line.buf;
+	size_t i = 0, len = line.len;
+	int ch, quote = 0, nest =0;
+	L_NORMAL:
+	for(; i < len; i++) {
+		ch = ln[i];
+		if(ch == '{' || ch == '[' || ch == '(') nest++;
+		if(ch == '}' || ch == ']' || ch == ')') nest--;
+		if(ch == '\'' || ch == '"' || ch == '`') {
+			quote = ch; i++;
+			goto L_QUOTE;
+		}
+	}
+	return nest;
+
+	L_QUOTE:
+	KNH_ASSERT(i > 0);
+	for(; i < len; i++) {
+		ch = ln[i];
+		if(ln[i-1] != '\\' && ch == quote) {
+			i++;
+			goto L_NORMAL;
+		}
+	}
+	return 1;
+}
+
+///* ------------------------------------------------------------------------ */
+//
+//KNHAPI(Object*) knh_InputStream_readData(Ctx *ctx, InputStream *in, knh_class_t reqc)
+//{
+//	int ch, linenum = DP(in)->line;
+//	knh_cwb_t cwbbuf, *cwb = knh_cwb_open(ctx, &cwbbuf);
+//	L_AGAIN:;
+//	while((ch = knh_InputStream_getc(ctx, in)) != EOF) {
+//		if(ch == '\r' || ch == '`') {
+//			int prev = DP(in)->prev;
+//			if(prev == '\r' || prev == '\n') continue;
+//			if(prev == ']' || prev == '}' || knh_bytes_checkStmtLine(knh_cwb_tobytes(cwb)) == 0) {
+//				break;
+//			}
+//		}
+//		knh_Bytes_putc(ctx, cwb->ba, ch);
+//	}
+//	if(knh_cwb_size(cwb) == 0) {
+//		if(ch == EOF) return KNH_NULL;
+//		goto L_AGAIN;
+//	}
+//
+//	{
+//		InputStream *bin = new_BytesInputStream(ctx, cwb->ba, cwb->pos, knh_Bytes_size(cwb->ba));
+//		Object *value = NULL;
+//		DP(bin)->uri = URI_EVAL;
+//		DP(bin)->line = linenum;
+//		knh_InputStream_setEncoding(ctx, bin, DP(in)->enc);
+//		value = konohac_data(ctx, bin, reqc);
+//		knh_cwb_close(cwb);
+//		if(IS_NULL(value)) goto L_AGAIN;
+//		return value;
+//	}
+//}
+
 /* ------------------------------------------------------------------------ */
 
 #ifdef __cplusplus
