@@ -36,33 +36,49 @@ extern "C" {
 #endif
 
 /* ------------------------------------------------------------------------ */
+/* [macros] */
+
+
+#define _knh_Method_mn(mtd)    DP(mtd)->mn
+#define _knh_Method_mf(mtd)    DP(mtd)->mf
 
 METHOD knh_fmethod_abstract(Ctx *ctx, knh_sfp_t *sfp);
 
 /* ======================================================================== */
 /* [MethodField] */
 
-knh_MethodField_t* new_MethodField(Ctx *ctx, knh_type_t rtype, size_t psize)
+MethodField* new_MethodField(Ctx *ctx, knh_type_t rtype, size_t psize)
 {
-	knh_MethodField_t *mf = (knh_MethodField_t*)new_Object_init(ctx, FLAG_MethodField, CLASS_MethodField, psize);
+	MethodField *mf = (knh_MethodField_t*)new_Object_init(ctx, FLAG_MethodField, CLASS_MethodField, psize);
 	mf->rtype = rtype;
 	return mf;
 }
 
-/* ------------------------------------------------------------------------ */
+/* ======================================================================== */
+/* [hcode] */
 
-knh_hashcode_t knh_mparam_hcode(knh_hashcode_t h, knh_type_t type, knh_fieldn_t fn)
+knh_hcode_t knh_mparam_hcode(knh_hcode_t h, knh_type_t type, knh_fieldn_t fn)
 {
 	h = h * (KNH_TCLASS_SIZE+1) + type;
 	return h + fn;
 }
 
-/* ------------------------------------------------------------------------ */
+/* ======================================================================== */
 /* [param] */
 
-void knh_MethodField_set(knh_MethodField_t *o, size_t n, knh_type_t type, knh_fieldn_t fn)
+#define _knh_MethodField_rztype(mf)   ((mf)->rtype)
+#define _knh_MethodField_psize(mf)    ((mf)->psize)
+
+#define _knh_Method_rztype(mtd)      ((DP(mtd)->mf)->rtype)
+#define _knh_Method_psize(mtd)       ((DP(mtd)->mf)->psize)
+#define _knh_Method_pztype(mtd,n)    knh_MethodField_pztype(DP(mtd)->mf,n)
+
+//#define _KONOHA_LASTPARAM     256
+
+/* ------------------------------------------------------------------------ */
+
+void knh_MethodField_set(MethodField *o, size_t n, knh_type_t type, knh_fieldn_t fn)
 {
-	//DBG2_P("o->psize=%d, n=%d", o->psize, n);
 	if(o->psize > KNH_METHODFIELD_2) {
 		DBG2_ASSERT(n < o->psize);
 		o->mparams[n].type = type;
@@ -81,7 +97,7 @@ void knh_MethodField_set(knh_MethodField_t *o, size_t n, knh_type_t type, knh_fi
 
 /* ------------------------------------------------------------------------ */
 
-knh_type_t knh_MethodField_pztype(knh_MethodField_t *o, size_t n)
+knh_type_t knh_MethodField_pztype(MethodField *o, size_t n)
 {
 	if(o->psize > KNH_METHODFIELD_2) {
 		if(n < o->psize) {
@@ -99,7 +115,7 @@ knh_type_t knh_MethodField_pztype(knh_MethodField_t *o, size_t n)
 
 /* ------------------------------------------------------------------------ */
 
-knh_mparam_t knh_MethodField_param(knh_MethodField_t *o, size_t n)
+knh_mparam_t knh_MethodField_param(MethodField *o, size_t n)
 {
 	if(o->psize > KNH_METHODFIELD_2) {
 		if(n < o->psize) {
@@ -117,21 +133,21 @@ knh_mparam_t knh_MethodField_param(knh_MethodField_t *o, size_t n)
 
 /* ------------------------------------------------------------------------ */
 
-knh_type_t knh_Method_rtype(Ctx *ctx, knh_Method_t *mtd, knh_class_t cid)
+knh_type_t knh_Method_rtype(Ctx *ctx, Method *mtd, knh_class_t cid)
 {
 	return knh_pmztype_totype(ctx, DP(mtd)->mf->rtype, cid);
 }
 
 /* ------------------------------------------------------------------------ */
 
-knh_type_t knh_Method_ptype(Ctx *ctx, knh_Method_t *o, knh_class_t cid, size_t n)
+knh_type_t knh_Method_ptype(Ctx *ctx, Method *o, knh_class_t cid, size_t n)
 {
 	return knh_pmztype_totype(ctx, knh_MethodField_pztype(DP(o)->mf, n), cid);
 }
 
 /* ------------------------------------------------------------------------ */
 
-knh_mparam_t knh_Method_param(knh_Method_t *o, size_t n)
+knh_mparam_t knh_Method_param(Method *o, size_t n)
 {
 	return knh_MethodField_param(DP(o)->mf, n);
 }
@@ -139,7 +155,7 @@ knh_mparam_t knh_Method_param(knh_Method_t *o, size_t n)
 /* ------------------------------------------------------------------------ */
 /* [Type] */
 
-int knh_MethodField_equalsType(knh_MethodField_t *o, knh_MethodField_t *o2)
+int knh_MethodField_equalsType(MethodField *o, MethodField *o2)
 {
 	if(o->psize != o2->psize || o->rtype != o2->rtype) return 0;
 	if(o->psize > KNH_METHODFIELD_2) {
@@ -159,22 +175,24 @@ int knh_MethodField_equalsType(knh_MethodField_t *o, knh_MethodField_t *o2)
 
 METHOD knh_fmethod_abstract(Ctx *ctx, knh_sfp_t *sfp)
 {
-	knh_Method_t *mtd = sfp[-1].mtd;
+	Method *mtd = sfp[-1].mtd;
 	knh_cwb_t cwbbuf, *cwb = knh_cwb_open(ctx, &cwbbuf);
 	knh_printf(ctx, cwb->w, "AbstractMethod!!: %C.%M", knh_Object_cid(sfp[0].o), DP(mtd)->mn);
-	knh_stack_throw(ctx, ctx->esp, knh_cwb_newException(ctx, cwb), NULL, 0);
+	String *s = new_String__cwb(ctx, cwb);
+	KNH_THROW(ctx, s);
 }
 
 /* ------------------------------------------------------------------------ */
+/* @method Boolean! Method.isAbstract() */
 
-knh_bool_t knh_Method_isAbstract(knh_Method_t *o)
+knh_bool_t knh_Method_isAbstract(Method *o)
 {
 	return (DP(o)->fproceed == knh_fmethod_abstract);
 }
 
 /* ------------------------------------------------------------------------ */
 
-void knh_Method_toAbstract(Ctx *ctx, knh_Method_t *o)
+void knh_Method_toAbstract(Ctx *ctx, Method *o)
 {
 	if(knh_Method_isObjectCode(o)) {
 		KNH_FINALv(ctx, DP(o)->code);
@@ -187,7 +205,7 @@ void knh_Method_toAbstract(Ctx *ctx, knh_Method_t *o)
 /* ======================================================================== */
 /* [Method] */
 
-void knh_Method_syncFunc(knh_Method_t *o, knh_fmethod f)
+void knh_Method_syncFunc(Method *o, knh_fmethod f)
 {
 	if(f == NULL) {
 		f = knh_fmethod_abstract;
@@ -198,9 +216,9 @@ void knh_Method_syncFunc(knh_Method_t *o, knh_fmethod f)
 
 /* ------------------------------------------------------------------------ */
 
-knh_Method_t* new_Method(Ctx *ctx, knh_flag_t flag, knh_class_t cid, knh_methodn_t mn, knh_fmethod func)
+Method* new_Method(Ctx *ctx, knh_flag_t flag, knh_class_t cid, knh_methodn_t mn, knh_fmethod func)
 {
-	knh_Method_t* o = (knh_Method_t*)new_Object_bcid(ctx, CLASS_Method, 0);
+	Method* o = (Method*)new_Object_bcid(ctx, CLASS_Method, 0);
 	DP(o)->flag  = flag;
 	DP(o)->cid    = cid;
 	DP(o)->mn     = mn;
@@ -224,22 +242,23 @@ METHOD knh_fmethod_NoSuchMethod(Ctx *ctx, knh_sfp_t *sfp)
 	knh_Bytes_write(ctx, cwb->ba, STEXT("NoSuchMethod!!: "));
 	knh_write_cid(ctx, cwb->w, knh_Object_cid(sfp[0].o));
 	knh_putc(ctx, cwb->w, '.');
-	knh_write_mn(ctx, cwb->w, knh_Method_rztype(sfp[-1].mtd), DP(sfp[-1].mtd)->mn);
-	knh_stack_throw(ctx, ctx->esp, knh_cwb_newException(ctx, cwb), NULL, 0);
+	knh_write_mn(ctx, cwb->w, DP(sfp[-1].mtd)->mn);
+	String *s = new_String__cwb(ctx, cwb);
+	KNH_THROW(ctx, s);
 }
 
 /* ------------------------------------------------------------------------ */
 
-knh_bool_t knh_Method_isNoSuchMethod(knh_Method_t *o)
+knh_bool_t knh_Method_isNoSuchMethod(Method *o)
 {
 	return (DP(o)->fproceed == knh_fmethod_NoSuchMethod);
 }
 
 /* ------------------------------------------------------------------------ */
 
-knh_Method_t* new_Method__NoSuchMethod(Ctx *ctx, knh_class_t cid, knh_methodn_t mn)
+Method* new_Method__NoSuchMethod(Ctx *ctx, knh_class_t cid, knh_methodn_t mn)
 {
-	knh_Method_t *mtd = new_Method(ctx, 0, cid, mn, knh_fmethod_NoSuchMethod);
+	Method *mtd = new_Method(ctx, 0, cid, mn, knh_fmethod_NoSuchMethod);
 	return mtd;
 }
 
@@ -249,141 +268,124 @@ knh_Method_t* new_Method__NoSuchMethod(Ctx *ctx, knh_class_t cid, knh_methodn_t
 int knh_methodn_isNew(Ctx *ctx, knh_methodn_t mn)
 {
 	if(mn == METHODN_new) return 1;
-	if(!METHODN_IS_MOVTEXT(mn) && !METHODN_IS_GETTER(mn) && !METHODN_IS_SETTER(mn)) {
-		char *n = FIELDN(METHODN_TOFIELDN(mn));
-		if(n[0] == 'n' && n[1] == 'e' && n[2] == 'w' && n[3] == ':') {
-			return 1;
-		}
+	if(METHODN_IS_MOVTEXT(mn) || METHODN_IS_GETTER(mn) || METHODN_IS_SETTER(mn)) {
+		return 0;
+	}
+	char *n = FIELDN(METHODN_TOFIELDN(mn));
+	if(n[0] == 'n' && n[1] == 'e' && n[2] == 'w' && n[3] == ':') {
+		return 1;
 	}
 	return 0;
 }
 
 /* ------------------------------------------------------------------------ */
 
-int knh_Method_isConstructor(Ctx *ctx, knh_Method_t *o)
+int knh_Method_isConstructor(Ctx *ctx, Method *o)
 {
-	if(IS_Method(o)) {
-		knh_class_t rtype = CLASS_type(knh_Method_rztype(o));
-		if(rtype != DP(o)->cid) {
-			return 0;
-		}
-		return knh_methodn_isNew(ctx, DP(o)->mn);
+	knh_class_t rtype = CLASS_type(knh_Method_rztype(o));
+	if(rtype != DP(o)->cid) {
+		return 0;
 	}
-	return 0;
+	return knh_methodn_isNew(ctx, DP(o)->mn);
 }
 
 /* ------------------------------------------------------------------------ */
 
 int knh_methodn_isOp(Ctx *ctx, knh_methodn_t mn)
 {
-	if(!METHODN_IS_MOVTEXT(mn) && !METHODN_IS_GETTER(mn) && !METHODN_IS_SETTER(mn)) {
-		char *n = FIELDN(METHODN_TOFIELDN(mn));
-		if(n[0] == 'o' && n[1] == 'p' && isupper(n[2])) {
-			return 1;
-		}
+	if(METHODN_IS_MOVTEXT(mn) || METHODN_IS_GETTER(mn) || METHODN_IS_SETTER(mn)) {
+		return 0;
+	}
+	char *n = FIELDN(METHODN_TOFIELDN(mn));
+	if(n[0] == 'o' && n[1] == 'p' && isupper(n[2])) {
+		return 1;
 	}
 	return 0;
 }
 
-
 /* ======================================================================== */
 /* [methods] */
 
 /* ------------------------------------------------------------------------ */
+/* @method String! Method.getName() */
 
-knh_String_t* knh_Method_getName(Ctx *ctx, knh_Method_t *o)
+
+String* knh_Method_getName(Ctx *ctx, Method *o)
 {
-	knh_cwb_t cwbbuf, *cwb = knh_cwb_open(ctx, &cwbbuf);
-	knh_write_mn(ctx, cwb->w, knh_Method_rztype(o), DP(o)->mn);
-	return knh_cwb_newString(ctx, cwb);
+	char buf[CLASSNAME_BUFSIZ];
+	knh_format_methodn(ctx, buf, sizeof(buf), DP(o)->mn);
+	return new_String(ctx, B(buf), NULL);
+}
+
+/* ======================================================================== */
+/* [Weaving] */
+
+int knh_Method_isWoven(Method *mtd)
+{
+	return (mtd->fcall_1 != DP(mtd)->fproceed);
 }
 
 /* ------------------------------------------------------------------------ */
-///* @method Int! Method.getProfCount() */
-//
-//
-//Int* knh_Method_getProfCount(Ctx *ctx, Method *o)
-//{
-//	return new_Int(ctx, DP(o)->prof_count);
-//}
-//
-///* ------------------------------------------------------------------------ */
-///* @method Int! Method.getProfTime() */
-//
-//
-//Int* knh_Method_getProfTime(Ctx *ctx, Method *o)
-//{
-//	return new_Int(ctx, DP(o)->prof_time);
-//}
-//
 
-///* ======================================================================== */
-///* [Weaving] */
-//
-//int knh_Method_isWoven(knh_Method_t *mtd)
-//{
-//	return (mtd->fcall_1 != DP(mtd)->fproceed);
-//}
-//
-///* ------------------------------------------------------------------------ */
-//
-//int knh_Method_canWeave(Ctx *ctx, knh_Method_t *mtd, knh_Method_t *aspect)
-//{
-//	if(knh_Method_isGenerator(mtd) || knh_Method_isAspect(mtd)) {
-//		TODO();
-//		return 0;
-//	}
-//	if(!IS_Method(aspect)) return 1; /* remove aspect */
-//	if(knh_Method_isWoven(aspect)) {
-//		KNH_WARNING(ctx, "nested aspect is not supported");
-//		return 0;
-//	}
-//	if(knh_Method_isAspect(aspect)) return 1;
-//	if(knh_Method_isVarArgs(mtd)) {
-//		KNH_WARNING(ctx, "variable length method can be woven");
-//		return 0;
-//	}
-//	if(knh_MethodField_equalsType(DP(mtd)->mf, DP(aspect)->mf)) {
-//		return 1;
-//	}
-//	return 0;
-//}
-//
-///* ------------------------------------------------------------------------ */
-//
-//void knh_Method_weave(Ctx *ctx, knh_Method_t *aspect, knh_Method_t *mtd)
-//{
-//	if(IS_Method(aspect)) {
-//		mtd->fcall_1 = aspect->fcall_1;
-//		mtd->pc_start = aspect->pc_start;
-//	}
-//	else { /* remove aspect */
-//		mtd->fcall_1 = DP(mtd)->fproceed;
-//		mtd->pc_start = knh_Method_pcstartNULL(mtd);
-//	}
-//}
-//
-///* ------------------------------------------------------------------------ */
+int knh_Method_canWeave(Ctx *ctx, Method *mtd, Method *aspect)
+{
+	if(knh_Method_isGenerator(mtd) || knh_Method_isAspect(mtd)) {
+		TODO();
+		return 0;
+	}
+	if(!IS_Method(aspect)) return 1; /* remove aspect */
+	if(knh_Method_isWoven(aspect)) {
+		KNH_WARNING(ctx, "nested aspect is not supported");
+		return 0;
+	}
+	if(knh_Method_isAspect(aspect)) return 1;
+	if(knh_Method_isVarArgs(mtd)) {
+		KNH_WARNING(ctx, "variable length method can be woven");
+		return 0;
+	}
+	if(knh_MethodField_equalsType(DP(mtd)->mf, DP(aspect)->mf)) {
+		return 1;
+	}
+	return 0;
+}
+
+/* ------------------------------------------------------------------------ */
+
+void knh_Method_weave(Ctx *ctx, Method *aspect, Method *mtd)
+{
+	if(IS_Method(aspect)) {
+		mtd->fcall_1 = aspect->fcall_1;
+		mtd->pc_start = aspect->pc_start;
+	}
+	else { /* remove aspect */
+		mtd->fcall_1 = DP(mtd)->fproceed;
+		mtd->pc_start = knh_Method_pcstartNULL(mtd);
+	}
+}
+
+/* ------------------------------------------------------------------------ */
 
 /* ======================================================================== */
 /* [Method] */
 
-void knh_Class_addMethod(Ctx *ctx, knh_class_t cid, knh_Method_t *mtd)
+void knh_Class_addMethod(Ctx *ctx, knh_class_t cid, Method *mtd)
 {
 	KNH_ASSERT(IS_Method(mtd));
 	KNH_ASSERT(cid == DP(mtd)->cid);
-	DBG2_ASSERT_cid(cid);
-	knh_ClassField_t *cs = ClassTable(cid).cstruct;;
+	KNH_ASSERT_cid(cid);
+	ClassStruct *cs = ClassTable(cid).cstruct;;
 	size_t i;
 	for(i = 0; i < knh_Array_size(cs->methods); i++) {
-		knh_Method_t *mtd2 = (knh_Method_t*)knh_Array_n(cs->methods, i);
+		Method *mtd2 = (Method*)knh_Array_n(cs->methods, i);
 		if(DP(mtd2)->mn == DP(mtd)->mn) {
-			KNH_WARNING(ctx, "Duplicated method: %s", FIELDN(METHODN_TOFIELDN(DP(mtd)->mn)));
+			char buf[CLASSNAME_BUFSIZ];
+			knh_format_cmethodn(ctx, buf, sizeof(buf), cid, DP(mtd)->mn);
+			KNH_WARNING(ctx, "Duplicated method: %s", buf);
 			return ;
 		}
 	}
 	if(knh_class_isSingleton(cid)) {
-		DP(mtd)->flag = DP(mtd)->flag | FLAG_Method_Static;
+		DP(mtd)->flag = DP(mtd)->flag | KNH_FLAG_MF_STATIC;
 	}
 	knh_Array_add(ctx, cs->methods, UP(mtd));
 }
@@ -391,20 +393,20 @@ void knh_Class_addMethod(Ctx *ctx, knh_class_t cid, knh_Method_t *mtd)
 /* ------------------------------------------------------------------------ */
 /* [MethodField] */
 
-void knh_addMethodFieldTable(Ctx *ctx, knh_MethodField_t *mf)
+void knh_addMethodFieldTable(Ctx *ctx, MethodField *mf)
 {
-	knh_hashcode_t h = knh_MethodField_hashCode(ctx, mf);
-	knh_HashMap_t *hmap = DP(ctx->sys)->MethodFieldHashMap;
+	knh_hcode_t h = knh_MethodField_hachCode(ctx, mf);
+	HashMap *hmap = DP(ctx->sys)->MethodFieldHashMap;
 	if(IS_NOTNULL(hmap)) {
 		KNH_LOCK(ctx, LOCK_SYSTBL, NULL);
-		knh_MethodField_t *mf2 = (knh_MethodField_t*)knh_HashMap_get__hcode(ctx, hmap, h);
+		MethodField *mf2 = (MethodField*)knh_HashMap_get__hcode(ctx, hmap, h);
 		if(IS_MethodField(mf2)) {
 			//DBG2_P("MISS!! %lu  rtype=%s%s psize=%d", h, TYPEQN(mf2->rtype), mf2->psize);
 		}
 		if(mf2->psize != 0) {
 			knh_HashMap_set__hcode(ctx, hmap, h, UP(mf));
 			DBG2_({
-				mf2 = (knh_MethodField_t*)knh_HashMap_get__hcode(ctx, hmap, h);
+				mf2 = (MethodField*)knh_HashMap_get__hcode(ctx, hmap, h);
 				KNH_ASSERT(mf == mf2);
 			});
 		}
@@ -414,13 +416,13 @@ void knh_addMethodFieldTable(Ctx *ctx, knh_MethodField_t *mf)
 
 /* ------------------------------------------------------------------------ */
 
-knh_MethodField_t *knh_findMethodField0(Ctx *ctx, knh_type_t rtype)
+MethodField *knh_findMethodField0(Ctx *ctx, knh_type_t rtype)
 {
-	knh_HashMap_t *hmap = DP(ctx->sys)->MethodFieldHashMap;
+	HashMap *hmap = DP(ctx->sys)->MethodFieldHashMap;
 	if(IS_NOTNULL(hmap)) {
-		knh_hashcode_t h = rtype;
+		knh_hcode_t h = rtype;
 		KNH_LOCK(ctx, LOCK_SYSTBL, NULL);
-		knh_MethodField_t *mf = (knh_MethodField_t*)knh_HashMap_get__hcode(ctx, hmap, h);
+		MethodField *mf = (MethodField*)knh_HashMap_get__hcode(ctx, hmap, h);
 		KNH_UNLOCK(ctx, LOCK_SYSTBL, NULL);
 		if(IS_MethodField(mf)) {
 			if(mf->rtype == rtype && mf->psize == 0) {
@@ -428,21 +430,21 @@ knh_MethodField_t *knh_findMethodField0(Ctx *ctx, knh_type_t rtype)
 			}
 		}
 	}
-	knh_MethodField_t *mf = new_MethodField(ctx, rtype, 0);
+	MethodField *mf = new_MethodField(ctx, rtype, 0);
 	knh_addMethodFieldTable(ctx, mf);
 	return mf;
 }
 
 /* ------------------------------------------------------------------------ */
 
-knh_MethodField_t *knh_findMethodField1(Ctx *ctx, knh_type_t rtype, knh_type_t p1, knh_fieldn_t fn1)
+MethodField *knh_findMethodField1(Ctx *ctx, knh_type_t rtype, knh_type_t p1, knh_fieldn_t fn1)
 {
-	knh_HashMap_t *hmap = DP(ctx->sys)->MethodFieldHashMap;
+	HashMap *hmap = DP(ctx->sys)->MethodFieldHashMap;
 	if(IS_NOTNULL(hmap)) {
-		knh_hashcode_t h = rtype;
+		knh_hcode_t h = rtype;
 		h = knh_mparam_hcode(h, p1, fn1);
 		KNH_LOCK(ctx, LOCK_SYSTBL, NULL);
-		knh_MethodField_t *mf = (knh_MethodField_t*)knh_HashMap_get__hcode(ctx, hmap, h);
+		MethodField *mf = (MethodField*)knh_HashMap_get__hcode(ctx, hmap, h);
 		KNH_UNLOCK(ctx, LOCK_SYSTBL, NULL);
 		if(IS_MethodField(mf)) {
 			if(mf->rtype == rtype && mf->psize == 1 && mf->p0.type == p1 && mf->p0.fn == fn1) {
@@ -450,7 +452,7 @@ knh_MethodField_t *knh_findMethodField1(Ctx *ctx, knh_type_t rtype, knh_type_t p
 			}
 		}
 	}
-	knh_MethodField_t *mf = new_MethodField(ctx, rtype, 1);
+	MethodField *mf = new_MethodField(ctx, rtype, 1);
 	mf->p0.type = p1;
 	mf->p0.fn = fn1;
 	knh_addMethodFieldTable(ctx, mf);
@@ -477,7 +479,7 @@ METHOD knh_fmethod_igetter(Ctx *ctx, knh_sfp_t *sfp)
 	knh_int_t *data = (knh_int_t*)(&(KNH_FIELDn(sfp[0].o, DP(sfp[-1].mtd)->delta)));
 	KNH_RETURN_Int(ctx, sfp, data[0]);
 #else/*KNH_USING_UNBOXFIELD*/
-	knh_Int_t *o = (knh_Int_t*)KNH_FIELDn(sfp[0].o, DP(sfp[-1].mtd)->delta);
+	Int *o = (Int*)KNH_FIELDn(sfp[0].o, DP(sfp[-1].mtd)->delta);
 	KNH_RETURN_Int(ctx, sfp, o->n.ivalue);
 #endif/*KNH_USING_UNBOXFIELD*/
 }
@@ -492,7 +494,7 @@ METHOD knh_fmethod_fgetter(Ctx *ctx, knh_sfp_t *sfp)
 	knh_float_t *data = (knh_float_t*)(&(KNH_FIELDn(sfp[0].o, DP(sfp[-1].mtd)->delta)));
 	KNH_RETURN_Float(ctx, sfp, data[0]);
 #else/*KNH_USING_UNBOXFIELD*/
-	knh_Float_t *o = (knh_Float_t*)KNH_FIELDn(sfp[0].o, DP(sfp[-1].mtd)->delta);
+	Float *o = (Float*)KNH_FIELDn(sfp[0].o, DP(sfp[-1].mtd)->delta);
 	KNH_RETURN_Float(ctx, sfp, o->n.fvalue);
 #endif/*KNH_USING_UNBOXFIELD*/
 }
@@ -507,7 +509,7 @@ METHOD knh_fmethod_bgetter(Ctx *ctx, knh_sfp_t *sfp)
 	knh_bool_t *data = (knh_bool_t*)(&(KNH_FIELDn(sfp[0].o, DP(sfp[-1].mtd)->delta)));
 	KNH_RETURN_Boolean(ctx, sfp, data[0]);
 #else/*KNH_USING_UNBOXFIELD*/
-	knh_Boolean_t *o = (knh_Boolean_t*)KNH_FIELDn(sfp[0].o, DP(sfp[-1].mtd)->delta);
+	Boolean *o = (Boolean*)KNH_FIELDn(sfp[0].o, DP(sfp[-1].mtd)->delta);
 	KNH_RETURN_Boolean(ctx, sfp, o->n.bvalue);
 #endif/*KNH_USING_UNBOXFIELD*/
 }
@@ -547,7 +549,7 @@ METHOD knh_fmethod_isetter(Ctx *ctx, knh_sfp_t *sfp)
 	knh_int_t *data = (knh_int_t*)(&(KNH_FIELDn(sfp[0].o, DP(sfp[-1].mtd)->delta)));
 	data[0] = sfp[1].ivalue;
 #else/*KNH_USING_UNBOXFIELD*/
-	knh_Int_t *n = new_Int(ctx, sfp[1].ivalue);
+	Int *n = new_Int(ctx, sfp[1].ivalue);
 	KNH_MOV(ctx, KNH_FIELDn(sfp[0].o, DP(sfp[-1].mtd)->delta), n);
 #endif/*KNH_USING_UNBOXFIELD*/
 	KNH_RETURN_void(ctx, sfp);
@@ -559,7 +561,7 @@ static
 METHOD knh_fmethod_insetter(Ctx *ctx, knh_sfp_t *sfp)
 {
 	DBG2_ASSERT(IS_Method(sfp[-1].mtd));
-	knh_Int_t *n = IS_NULL(sfp[1].o) ? sfp[1].i : new_Int(ctx, sfp[1].ivalue);
+	Int *n = IS_NULL(sfp[1].o) ? sfp[1].i : new_Int(ctx, sfp[1].ivalue);
 	KNH_MOV(ctx, KNH_FIELDn(sfp[0].o, DP(sfp[-1].mtd)->delta), n);
 	KNH_RETURN_void(ctx, sfp);
 }
@@ -574,7 +576,7 @@ METHOD knh_fmethod_fsetter(Ctx *ctx, knh_sfp_t *sfp)
 	knh_float_t *data = (knh_float_t*)(&(KNH_FIELDn(sfp[0].o, DP(sfp[-1].mtd)->delta)));
 	data[0] = sfp[1].fvalue;
 #else/*KNH_USING_UNBOXFIELD*/
-	knh_Float_t *n = new_Float(ctx, sfp[1].fvalue);
+	Float *n = new_Float(ctx, sfp[1].fvalue);
 	KNH_MOV(ctx, KNH_FIELDn(sfp[0].o, DP(sfp[-1].mtd)->delta), n);
 #endif/*KNH_USING_UNBOXFIELD*/
 	KNH_RETURN_void(ctx, sfp);
@@ -586,7 +588,7 @@ static
 METHOD knh_fmethod_fnsetter(Ctx *ctx, knh_sfp_t *sfp)
 {
 	DBG2_ASSERT(IS_Method(sfp[-1].mtd));
-	knh_Float_t *n = IS_NULL(sfp[1].o) ? sfp[1].f : new_Float(ctx, sfp[1].fvalue);
+	Float *n = IS_NULL(sfp[1].o) ? sfp[1].f : new_Float(ctx, sfp[1].fvalue);
 	KNH_MOV(ctx, KNH_FIELDn(sfp[0].o, DP(sfp[-1].mtd)->delta), n);
 	KNH_RETURN_void(ctx, sfp);
 }
@@ -595,7 +597,7 @@ METHOD knh_fmethod_fnsetter(Ctx *ctx, knh_sfp_t *sfp)
 /* ------------------------------------------------------------------------ */
 
 static
-knh_Method_t *new_Method_getter(Ctx *ctx, knh_class_t cid, knh_methodn_t mn, knh_type_t type, int idx)
+Method *new_Method_getter(Ctx *ctx, knh_class_t cid, knh_methodn_t mn, knh_type_t type, int idx)
 {
 	knh_fmethod f = knh_fmethod_getter;
 	if(IS_ubxint(type)) {
@@ -607,7 +609,7 @@ knh_Method_t *new_Method_getter(Ctx *ctx, knh_class_t cid, knh_methodn_t mn, knh
 	else if(IS_ubxboolean(type)) {
 		f = knh_fmethod_bgetter;
 	}
-	knh_Method_t *mtd = new_Method(ctx, 0, cid, mn, f);
+	Method *mtd = new_Method(ctx, /*KNH_FLAG_MF_GENERATED*/0, cid, mn, f);
 	DP(mtd)->delta = idx;
 	KNH_SETv(ctx, DP(mtd)->mf, knh_findMethodField0(ctx, type));
 	return mtd;
@@ -616,7 +618,7 @@ knh_Method_t *new_Method_getter(Ctx *ctx, knh_class_t cid, knh_methodn_t mn, knh
 /* ------------------------------------------------------------------------ */
 
 static
-knh_Method_t *new_Method_setter(Ctx *ctx, knh_class_t cid, knh_methodn_t mn, knh_type_t type, int idx)
+Method *new_Method_setter(Ctx *ctx, knh_class_t cid, knh_methodn_t mn, knh_type_t type, int idx)
 {
 	knh_fmethod f = knh_fmethod_setter;
 	if(IS_ubxint(type)) {
@@ -634,7 +636,7 @@ knh_Method_t *new_Method_setter(Ctx *ctx, knh_class_t cid, knh_methodn_t mn, knh
 	else if(IS_ubxboolean(type)) {
 		f = knh_fmethod_bsetter;
 	}
-	knh_Method_t *mtd = new_Method(ctx, 0, cid, mn, f);
+	Method *mtd = new_Method(ctx, /*KNH_FLAG_MF_GENERATED*/0, cid, mn, f);
 	DP(mtd)->delta = idx;
 	KNH_SETv(ctx, DP(mtd)->mf, knh_findMethodField1(ctx, TYPE_void, type, FIELDN_v));
 	return mtd;
@@ -642,33 +644,17 @@ knh_Method_t *new_Method_setter(Ctx *ctx, knh_class_t cid, knh_methodn_t mn, knh
 
 /* ------------------------------------------------------------------------ */
 
-static knh_methodn_t knh_methodn_base(Ctx *ctx, knh_methodn_t mn)
-{
-	size_t i;
-	knh_bytes_t n = __tobytes(knh_getFieldName(ctx, METHODN_TOFIELDN(mn)));
-	for(i = 0; i < n.len; i++) {
-		if(n.buf[i] == ':') {
-			n.len = i;
-			knh_fieldn_t fn = knh_getfnq(ctx, n, FIELDN_NEWID);
-			return (fn | ((~KNH_FLAG_MN_MOVTEXT) & mn));
-		}
-	}
-	return mn;
-}
-
-/* ------------------------------------------------------------------------ */
-
-knh_Method_t*
+Method*
 knh_Class_getMethod__(Ctx *ctx, knh_class_t this_cid, knh_methodn_t mn, knh_bool_t gen)
 {
 	knh_class_t cid = this_cid;
 	TAIL_RECURSION:;
-	DBG2_ASSERT_cid(cid);
+	KNH_ASSERT_cid(cid);
 	{
-		knh_ClassField_t *cs = ClassTable(cid).cstruct;;
+		ClassStruct *cs = ClassTable(cid).cstruct;;
 		size_t i;
 		for(i = 0; i < knh_Array_size(cs->methods); i++) {
-			knh_Method_t *mtd = (knh_Method_t*)knh_Array_n(cs->methods, i);
+			Method *mtd = (Method*)knh_Array_n(cs->methods, i);
 			if(DP(mtd)->mn == mn) return mtd;
 		}
 		if(cid == CLASS_Object) {
@@ -680,7 +666,6 @@ knh_Class_getMethod__(Ctx *ctx, knh_class_t this_cid, knh_methodn_t mn, knh_bool
 	goto TAIL_RECURSION;
 
 	L_GenerateField:;
-
 	if(METHODN_IS_GETTER(mn)) {
 		knh_index_t idx = knh_Class_indexOfField(ctx, this_cid, METHODN_TOFIELDN(mn));
 		if(idx == -1) {
@@ -688,70 +673,71 @@ knh_Class_getMethod__(Ctx *ctx, knh_class_t this_cid, knh_methodn_t mn, knh_bool
 		}
 		else {
 			knh_cfield_t *cf = knh_Class_fieldAt(ctx, this_cid, idx);
-			if(!KNH_FLAG_IS(cf->flag, FLAG_ClassField_Getter)) {
+			if(!KNH_FLAG_IS(cf->flag, KNH_FLAG_CFF_GETTER)) {
 				goto L_NoSuchMethod;
 			}
 			else {
-				knh_Method_t *mtd = new_Method_getter(ctx, this_cid, mn, cf->type, idx);
-				knh_ClassField_t *cs = ctx->share->ClassTable[this_cid].cstruct;;
+				Method *mtd = new_Method_getter(ctx, this_cid, mn, cf->type, idx);
+				ClassStruct *cs = ctx->share->ClassTable[this_cid].cstruct;;
 				knh_Array_add(ctx, cs->methods, UP(mtd));
 				return mtd;
 			}
 		}
 	}
-	else if(METHODN_IS_SETTER(mn)) {
+
+	if(METHODN_IS_SETTER(mn)) {
 		knh_index_t idx = knh_Class_indexOfField(ctx, this_cid, METHODN_TOFIELDN(mn));
 		if(idx == -1) {
 			goto L_NoSuchMethod;
 		}
 		else {
 			knh_cfield_t *cf = knh_Class_fieldAt(ctx, this_cid, idx);
-			if(!KNH_FLAG_IS(cf->flag, FLAG_ClassField_Setter)) {
+			if(!KNH_FLAG_IS(cf->flag, KNH_FLAG_CFF_SETTER)) {
 				goto L_NoSuchMethod;
 			}
 			else {
-				knh_Method_t *mtd = new_Method_setter(ctx, this_cid, mn, cf->type, idx);
-				knh_ClassField_t *cs = ctx->share->ClassTable[this_cid].cstruct;;
+				Method *mtd = new_Method_setter(ctx, this_cid, mn, cf->type, idx);
+				ClassStruct *cs = ctx->share->ClassTable[this_cid].cstruct;;
 				knh_Array_add(ctx, cs->methods, UP(mtd));
 				return mtd;
 			}
 		}
 	}
-	else if(!METHODN_IS_MOVTEXT(mn)) {
-		knh_methodn_t mnbase = knh_methodn_base(ctx, mn);
-		if(mnbase != mn) {
-			DBG2_P("searching again %s, %s", FIELDN(METHODN_TOFIELDN(mn)), FIELDN(METHODN_TOFIELDN(mnbase)));
-			mn = mnbase;
-			goto TAIL_RECURSION;
-		}
-	}
 
 	L_NoSuchMethod:;
+	//DEBUG("Not Found: cid=%s, mn=%d,%s", CLASSN(this_cid), mn, METHODN(mn));
 	if(gen) {
 		if(METHODN_IS_MOVTEXT(mn)) {
-			DBG2_P("Generating %%empty: cid=%s mn=%%%s", CLASSN(cid), METHODN(mn));
 			return knh_Class_getMethod(ctx, cid, METHODN__empty);
 		}
 		else {
-			knh_Method_t *mtd = new_Method__NoSuchMethod(ctx, cid, mn);
-			knh_ClassField_t *cs = ctx->share->ClassTable[this_cid].cstruct;;
+			Method *mtd = new_Method__NoSuchMethod(ctx, cid, mn);
+			ClassStruct *cs = ctx->share->ClassTable[this_cid].cstruct;;
+			DBG2_({
+				char bufcm[CLASSNAME_BUFSIZ];
+				knh_format_cmethodn(ctx, bufcm, sizeof(bufcm), cid, mn);
+				DBG2_P("GENERATE NoSuchMethod: %s", bufcm);
+			})
 			knh_Array_add(ctx, cs->methods, UP(mtd));
 			return mtd;
 		}
 	}
 	else {
-		return (knh_Method_t*)KNH_NULL;
+		return (Method*)KNH_NULL;
 	}
 }
 
+#define _knh_Class_getMethod(ctx, c, mn)    knh_Class_getMethod__(ctx, c, mn, 0)
+#define _knh_Class_findMethod(ctx, c, mn)   knh_Class_getMethod__(ctx, c, mn, 1)
+
 /* ======================================================================== */
 /* [utils] */
 
-knh_Method_t *knh_lookupMethod(Ctx *ctx, knh_class_t cid, knh_methodn_t mn)
+Method *knh_lookupMethod(Ctx *ctx, knh_class_t cid, knh_methodn_t mn)
 {
 	KNH_ASSERT(!METHODN_IS_MOVTEXT(mn));
-	knh_hashcode_t h = ((((knh_hashcode_t)cid) << (sizeof(knh_class_t) * 8)) + mn) % ctx->cachesize;
-	knh_Method_t *mtd = ctx->mtdCache[h];
+	knh_hcode_t h = ((((knh_hcode_t)cid) << (sizeof(knh_class_t) * 8)) + mn) % ctx->cachesize;
+	Method *mtd = ctx->mtdCache[h];
 	if(mtd != NULL) {
 		if(DP(mtd)->mn == mn) {
 			if(DP(mtd)->cid == cid || knh_class_instanceof(ctx, cid, DP(mtd)->cid)) {
@@ -760,7 +746,11 @@ knh_Method_t *knh_lookupMethod(Ctx *ctx, knh_class_t cid, knh_methodn_t mn)
 			}
 		}
 		knh_stat_mtdCacheMiss(ctx);
-		DBG2_P("Cache missed[%ld]. looking up %s.%s", h, CLASSN(cid), METHODN(mn));
+		DBG2_({
+			char bufcm[CLASSNAME_BUFSIZ];
+			knh_format_cmethodn(ctx, bufcm, sizeof(bufcm), cid, mn);
+			DBG2_P("Cache missed. looking up %s", bufcm);
+		})
 	}
 	mtd = knh_Class_findMethod(ctx, cid, mn);
 	ctx->mtdCache[h] = mtd;
@@ -769,11 +759,11 @@ knh_Method_t *knh_lookupMethod(Ctx *ctx, knh_class_t cid, knh_methodn_t mn)
 
 /* ------------------------------------------------------------------------ */
 
-knh_Method_t *knh_lookupFormatter(Ctx *ctx, knh_class_t cid, knh_methodn_t mn)
+Method *knh_lookupFormatter(Ctx *ctx, knh_class_t cid, knh_methodn_t mn)
 {
 	DBG2_ASSERT(METHODN_IS_MOVTEXT(mn));
-	knh_hashcode_t h = ((((knh_hashcode_t)cid) << (sizeof(knh_class_t) * 8)) + mn) % ctx->cachesize;
-	knh_Method_t *mtd = ctx->fmtCache[h];
+	knh_hcode_t h = ((((knh_hcode_t)cid) << (sizeof(knh_class_t) * 8)) + mn) % ctx->cachesize;
+	Method *mtd = ctx->fmtCache[h];
 	if(mtd != NULL) {
 		if(DP(mtd)->mn == mn) {
 			if(DP(mtd)->cid == cid || knh_class_instanceof(ctx, cid, DP(mtd)->cid)) {
@@ -782,7 +772,11 @@ knh_Method_t *knh_lookupFormatter(Ctx *ctx, knh_class_t cid, knh_methodn_t mn)
 			}
 		}
 		knh_stat_fmtCacheMiss(ctx);
-		DBG2_P("Cache[%ld] missed. looking up %s.%%%s <%s>", h, CLASSN(cid), METHODN(mn), CLASSN(DP(mtd)->cid));
+		DBG2_({
+			char bufcm[CLASSNAME_BUFSIZ];
+			knh_format_cmethodn(ctx, bufcm, sizeof(bufcm), cid, mn);
+			DBG2_P("Cache missed. looking up %s <%s>", bufcm, CLASSN(DP(mtd)->cid));
+		})
 	}
 	mtd = knh_Class_findMethod(ctx, cid, mn);
 	ctx->fmtCache[h] = mtd;
