@@ -47,9 +47,9 @@ knh_int_t knh_Number_tointeger(Any *o)
 		case CLASS_Boolean :
 			return (IS_TRUE(n)) ? 1 : 0;
 		case CLASS_Int :
-			return (knh_int_t)((knh_Int_t*)n)->n.ivalue;
+			return (knh_int_t)((Int*)n)->n.ivalue;
 		case CLASS_Float :
-			return (knh_int_t)((knh_Float_t*)n)->n.fvalue;
+			return (knh_int_t)((Float*)n)->n.fvalue;
 	}
 	KNH_ASSERT(n->h.bcid == CLASS_Int); /* STOP */
 	return 0;
@@ -61,21 +61,29 @@ knh_float_t knh_Number_tofloat(Any *o)
 {
 	Object *n = (Object*)o;
 	switch(n->h.bcid) {
-	case CLASS_Boolean :
-		return (IS_TRUE(n)) ? KNH_FLOAT_ONE : KNH_FLOAT_ZERO;
+		case CLASS_Boolean :
+#ifndef KNH_USING_NOFLOAT
+			return (IS_TRUE(n)) ? 1 : 0;
+#else
+			return (IS_TRUE(n)) ? 1.0 : 0.0;
+#endif
 		case CLASS_Int :
-			return (knh_float_t)((knh_Int_t*)n)->n.ivalue;
+			return (knh_float_t)((Int*)n)->n.ivalue;
 		case CLASS_Float :
-			return (knh_float_t)((knh_Float_t*)n)->n.fvalue;
+			return (knh_float_t)((Float*)n)->n.fvalue;
 	}
 	KNH_ASSERT(n->h.bcid == CLASS_Int); /* STOP */
-	return KNH_FLOAT_ZERO;
+#ifndef KNH_USING_NOFLOAT
+	return (IS_TRUE(n)) ? 1 : 0;
+#else
+	return (IS_TRUE(n)) ? 1.0 : 0.0;
+#endif
 }
 
 /* ======================================================================== */
 /* [Int] */
 
-KNHAPI(knh_Int_t*) new_Int(Ctx *ctx, knh_int_t value)
+KNHAPI(Int*) new_Int(Ctx *ctx, knh_int_t value)
 {
 	knh_Int_t *b = (knh_Int_t*)new_hObject(ctx, FLAG_Int, CLASS_Int, CLASS_Int);
 	b->n.ivalue = value;
@@ -84,7 +92,7 @@ KNHAPI(knh_Int_t*) new_Int(Ctx *ctx, knh_int_t value)
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(knh_Int_t*) new_IntX__fast(Ctx *ctx, knh_class_t cid, knh_int_t value)
+KNHAPI(Int*) new_IntX__fast(Ctx *ctx, knh_class_t cid, knh_int_t value)
 {
 	knh_Int_t *b = (knh_Int_t*)new_hObject(ctx, FLAG_Int, CLASS_Int, cid);
 	b->n.ivalue = value;
@@ -93,11 +101,11 @@ KNHAPI(knh_Int_t*) new_IntX__fast(Ctx *ctx, knh_class_t cid, knh_int_t value)
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(knh_Int_t*) new_IntX(Ctx *ctx, knh_class_t cid, knh_int_t value)
+KNHAPI(Int*) new_IntX(Ctx *ctx, knh_class_t cid, knh_int_t value)
 {
-	knh_ClassSpec_t *u = knh_getClassSpec(ctx, cid);
+	ClassSpec *u = knh_getClassSpec(ctx, cid);
 	if(DP(u)->fichk(u, value)) {
-		knh_Int_t *n = (knh_Int_t*)new_hObject(ctx, FLAG_Int, CLASS_Int, cid);
+		Int *n = (knh_Int_t*)new_hObject(ctx, FLAG_Int, CLASS_Int, cid);
 		n->n.ivalue = value;
 		return n;
 	}
@@ -115,30 +123,30 @@ KNHAPI(knh_Int_t*) new_IntX(Ctx *ctx, knh_class_t cid, knh_int_t value)
 /* ======================================================================== */
 /* [Float] */
 
-KNHAPI(knh_Float_t*) new_Float(Ctx *ctx, knh_float_t value)
+KNHAPI(Float*) new_Float(Ctx *ctx, knh_float_t value)
 {
-	knh_Float_t *b = (knh_Float_t*)new_hObject(ctx, FLAG_Float, CLASS_Float, CLASS_Float);
+	Float *b = (Float*)new_hObject(ctx, FLAG_Float, CLASS_Float, CLASS_Float);
 	b->n.fvalue = value;
 	return b;
 }
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(knh_Float_t*) new_FloatX__fast(Ctx *ctx, knh_class_t cid, knh_float_t value)
+KNHAPI(Float*) new_FloatX__fast(Ctx *ctx, knh_class_t cid, knh_float_t value)
 {
-	knh_Float_t *b = (knh_Float_t*)new_hObject(ctx, FLAG_Float, CLASS_Float, cid);
+	Float *b = (Float*)new_hObject(ctx, FLAG_Float, CLASS_Float, cid);
 	b->n.fvalue = value;
 	return b;
 }
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(knh_Float_t*) new_FloatX(Ctx *ctx, knh_class_t cid, knh_float_t value)
+KNHAPI(Float*) new_FloatX(Ctx *ctx, knh_class_t cid, knh_float_t value)
 {
-	DBG2_ASSERT_cid(cid);
-	knh_ClassSpec_t *u = knh_getClassSpec(ctx, cid);
+	KNH_ASSERT_cid(cid);
+	ClassSpec *u = knh_getClassSpec(ctx, cid);
 	if(DP(u)->ffchk(u, value)) {
-		knh_Float_t *f = (knh_Float_t*)new_hObject(ctx, FLAG_Float, CLASS_Float, cid);
+		Float *f = (Float*)new_hObject(ctx, FLAG_Float, CLASS_Float, cid);
 		f->n.fvalue = value;
 		return f;
 	}
@@ -152,9 +160,9 @@ KNHAPI(knh_Float_t*) new_FloatX(Ctx *ctx, knh_class_t cid, knh_float_t value)
 /* [Affine] */
 
 static
-knh_AffineConv_t *new_AffineConv(Ctx *ctx, knh_float_t fa, knh_float_t fb)
+AffineConv *new_AffineConv(Ctx *ctx, knh_float_t fa, knh_float_t fb)
 {
-	knh_AffineConv_t *af = (knh_AffineConv_t*)new_Object_bcid(ctx, CLASS_AffineConv, 0);
+	knh_AffineConv_t *af = (AffineConv*)new_Object_bcid(ctx, CLASS_AffineConv, 0);
 	af->scale = (knh_affinefloat_t)fa;
 	af->shift = (knh_affinefloat_t)fb;
 	return af;
@@ -166,7 +174,7 @@ knh_AffineConv_t *new_AffineConv(Ctx *ctx, knh_float_t fa, knh_float_t fb)
 static
 MAPPER knh_AffineConv_fmap__i2i(Ctx *ctx, knh_sfp_t *sfp)
 {
-	knh_AffineConv_t *af = (knh_AffineConv_t*)DP(sfp[1].mpr)->mapdata;
+	AffineConv *af = (AffineConv*)DP(sfp[1].mpr)->mapdata;
 	KNH_ASSERT(IS_AffineConv(af));
 	knh_float_t y = (sfp[0].ivalue * af->scale) + af->shift;
 	KNH_MAPPED_Int(ctx, sfp, (knh_int_t)y);
@@ -177,7 +185,7 @@ MAPPER knh_AffineConv_fmap__i2i(Ctx *ctx, knh_sfp_t *sfp)
 static
 MAPPER knh_AffineConv_fmap__i2f(Ctx *ctx, knh_sfp_t *sfp)
 {
-	knh_AffineConv_t *af = (knh_AffineConv_t*)DP(sfp[1].mpr)->mapdata;
+	AffineConv *af = (AffineConv*)DP(sfp[1].mpr)->mapdata;
 	KNH_ASSERT(IS_AffineConv(af));
 	knh_float_t y = (sfp[0].ivalue * af->scale) + af->shift;
 	KNH_MAPPED_Float(ctx, sfp, y);
@@ -188,7 +196,7 @@ MAPPER knh_AffineConv_fmap__i2f(Ctx *ctx, knh_sfp_t *sfp)
 static
 MAPPER knh_AffineConv_fmap__f2i(Ctx *ctx, knh_sfp_t *sfp)
 {
-	knh_AffineConv_t *af = (knh_AffineConv_t*)DP(sfp[1].mpr)->mapdata;
+	AffineConv *af = (AffineConv*)DP(sfp[1].mpr)->mapdata;
 	KNH_ASSERT(IS_AffineConv(af));
 	knh_float_t y = (sfp[0].fvalue * af->scale) + af->shift;
 	KNH_MAPPED_Int(ctx, sfp, (knh_int_t)y);
@@ -199,7 +207,7 @@ MAPPER knh_AffineConv_fmap__f2i(Ctx *ctx, knh_sfp_t *sfp)
 static
 MAPPER knh_AffineConv_fmap__f2f(Ctx *ctx, knh_sfp_t *sfp)
 {
-	knh_AffineConv_t *af = (knh_AffineConv_t*)DP(sfp[1].mpr)->mapdata;
+	AffineConv *af = (AffineConv*)DP(sfp[1].mpr)->mapdata;
 	KNH_ASSERT(IS_AffineConv(af));
 	knh_float_t y = (sfp[0].fvalue * af->scale) + af->shift;
 	KNH_MAPPED_Float(ctx, sfp, y);
@@ -226,10 +234,12 @@ knh_fmapper knh_fmapper_affine(knh_class_t scid, knh_class_t tcid)
 /* ------------------------------------------------------------------------ */
 /* [TAFFINE] */
 
+#define _KNH_FLAG_MMF_AFFINE (KNH_FLAG_MMF_ICAST|KNH_FLAG_MMF_SYNONYM|KNH_FLAG_MMF_TOTAL|KNH_FLAG_MMF_CONST)
+
 static
 void KNH_TAFFINE(Ctx *ctx, knh_class_t scid, knh_class_t tcid, knh_float_t scale, knh_float_t shift)
 {
-	knh_Mapper_t *mpr = new_Mapper(ctx, FLAG_Mapper_Affine, scid, tcid,
+	Mapper *mpr = new_Mapper(ctx, KNH_FLAG_MMF_AFFINE, scid, tcid,
 			knh_fmapper_affine(ctx->share->ClassTable[scid].bcid, ctx->share->ClassTable[tcid].bcid),
 			(Object*)new_AffineConv(ctx, scale, shift));
 	knh_ClassMap_add(ctx, ctx->share->ClassTable[scid].cmap, mpr);
@@ -239,7 +249,7 @@ void KNH_TAFFINE(Ctx *ctx, knh_class_t scid, knh_class_t tcid, knh_float_t scale
 
 KNHAPI(void) knh_addAffineMapper(Ctx *ctx, knh_class_t scid, char *text, knh_float_t scale, knh_float_t shift)
 {
-	//DBG2_ASSERT_cid(scid);
+	//KNH_ASSERT_cid(scid);
 	knh_class_t tcid = knh_findcid(ctx, B(text));
 	if(tcid != CLASS_unknown && ctx->share->ClassTable[tcid].bcid != tcid) {
 		KNH_TAFFINE(ctx, scid, tcid, scale, shift);
