@@ -61,28 +61,42 @@ KNHAPI(Object*) new_Glue(Ctx *ctx, char *lname, void *ptr, knh_fgfree gfree)
 {
 	knh_class_t cid = knh_getcid(ctx, B(lname));
 	if(cid == CLASS_unknown) {
-		fprintf(stderr, "Unknown glue class: %s", lname);
-		KNH_BUGSTOP();
-	}
-	{
-		knh_Glue_t *g = (knh_Glue_t*)new_Object_bcid(ctx, CLASS_Any, 0);
-		g->h.cid = cid;
-		knh_Glue_init(ctx, g, ptr, gfree);
-		return (Object*)g;
+		KNH_THROWf(ctx, "Unknown!!: %s", lname);
 	}
+	knh_Glue_t *g = (knh_Glue_t*)new_Object_bcid(ctx, CLASS_Any, 0);
+	g->h.cid = cid;
+	knh_Glue_init(ctx, g, ptr, gfree);
+	return (Object*)g;
+}
+
+/* ======================================================================== */
+/* [new] */
+/* @method[VIRTUAL] This! Object.new() */
+
+METHOD knh__Object_new(Ctx *ctx, knh_sfp_t *sfp)
+{
+	KNH_RETURN(ctx, sfp, sfp[0].o);
 }
 
 /* ======================================================================== */
 /* [getkey] */
 
-knh_String_t *knh_Object_getkey(Ctx *ctx, Object *o)
+String *knh_Object_getkey(Ctx *ctx, Object *o)
 {
 	knh_sfp_t lsfp;
 	lsfp.o = o;
-	lsfp.data = ((knh_Int_t*)o)->n.data;
+	lsfp.data = ((Int*)o)->n.data;
 	return ctx->share->StructTable[knh_Object_bcid(o)].fgetkey(ctx, &lsfp);
 }
 
+/* ------------------------------------------------------------------------ */
+/* @method String! Object.getKey() */
+
+METHOD knh__Object_getKey(Ctx *ctx, knh_sfp_t *sfp)
+{
+	KNH_RETURN(ctx, sfp, ctx->share->StructTable[knh_Object_bcid(sfp[0].o)].fgetkey(ctx, sfp));
+}
+
 /* ======================================================================== */
 /* [copy] */
 
@@ -98,73 +112,42 @@ Object *knh_Object_copy(Ctx *ctx, Object *b)
 /* ======================================================================== */
 /* [commons] */
 
-knh_hashcode_t knh_Object_hashCode(Ctx *ctx, Object *o)
+knh_hcode_t knh_Object_hashCode(Ctx *ctx, Object *o)
 {
-	return StructTable(o->h.bcid).fhashCode(ctx, o);
+	return ctx->share->StructTable[o->h.bcid].fhashCode(ctx, o);
 }
 
 /* ------------------------------------------------------------------------ */
 
-knh_String_t *knh_Object_key(Ctx *ctx, Object *o)
-{
-	knh_String_t *s;
-	knh_sfp_t *lsfp = ctx->esp + 1;
-	KNH_SETv(ctx, lsfp[0].o, o);
-	lsfp[0].data = knh_Object_data(o);
-	s = StructTable(o->h.bcid).fgetkey(ctx, lsfp);
-	KNH_SETv(ctx, lsfp[-1].o, s);
-	return s;
-}
-/* ------------------------------------------------------------------------ */
 
 int knh_Object_compareTo(Ctx *ctx, Object *o1, Object *o2)
 {
-	knh_class_t bcid1 = o1->h.bcid;
+	knh_class_t bcid = o1->h.bcid;
 	knh_class_t bcid2 = o2->h.bcid;
-	int res;
-	if(bcid1 == bcid2) {
-		res = StructTable(bcid1).fcompareTo(ctx, o1, o2);
+	if(bcid == bcid2) {
+		return ctx->share->StructTable[bcid].fcompareTo(o1, o2);
 	}
 	else {
 		if((o1->h.cid == CLASS_Int || o1->h.cid == CLASS_Float)
 		&& (o2->h.cid == CLASS_Int || o2->h.cid == CLASS_Float)) {
-			res = (int)knh_Number_tofloat(o1) - (int)knh_Number_tofloat(o2);
-		}
-		else {
-			res = (int)(o1 - o2);
+			return (int)(knh_Number_tofloat(o1) - knh_Number_tofloat(o2));
 		}
+		DBG2_P("Compared Incompatible Type %s - %s", CLASSN(o1->h.cid), CLASSN(o2->h.cid));
+		return (int)(o1 - o2);
 	}
-	DBG2_P("compared %s %s res=%d", CLASSNo(o1), CLASSNo(o2), res);
-	return res;
 }
 
 /* ------------------------------------------------------------------------ */
 
-int knh_Object_compareTo2(Ctx *ctx, Object **o1, Object **o2)
+int knh_Object_cmp(Object *o1, Object *o2)
 {
-	Object* _o1 = *((Object**) o1);
-	Object* _o2 = *((Object**) o2);
-	knh_class_t bcid1 = _o1->h.bcid;
-	knh_class_t bcid2 = _o2->h.bcid;
-	int res;
-	if(bcid1 == bcid2) {
-		res = StructTable(bcid1).fcompareTo(ctx, _o1, _o2);
-	}
-	else {
-		if((_o1->h.cid == CLASS_Int || _o1->h.cid == CLASS_Float)
-		&& (_o2->h.cid == CLASS_Int || _o2->h.cid == CLASS_Float)) {
-			res = (int)knh_Number_tofloat(_o1) - (int)knh_Number_tofloat(_o2);
-		}
-		else {
-			res = (int)(_o1 - _o2);
-		}
-	}
-	DBG2_P("compared %s %s res=%d", CLASSNo(_o1), CLASSNo(_o2), res);
-	return res;
+	/* this funtion is prepared for the compatibility with qsort. */
+	return knh_Object_compareTo(knh_getCurrentContext(), o1, o2);
 }
 
 /* ------------------------------------------------------------------------ */
 
+
 knh_bool_t knh_Object_equals(Ctx *ctx, Object *o1, Object *o2)
 {
 	return (knh_Object_compareTo(ctx, o1, o2) == 0);
@@ -176,8 +159,8 @@ knh_bool_t knh_Object_equals(Ctx *ctx, Object *o1, Object *o2)
 knh_bytes_t knh_Object_tobytes(Ctx *ctx, Object *o)
 {
 	switch(o->h.bcid) {
-		case STRUCT_String : return __tobytes((knh_String_t*)o);
-		case STRUCT_Bytes : return knh_Bytes_tobytes((knh_Bytes_t*)o);
+		case STRUCT_String : return knh_String_tobytes((String*)o);
+		case STRUCT_Bytes : return knh_Bytes_tobytes((Bytes*)o);
 	}
 	return STEXT("");
 }
