@@ -38,9 +38,9 @@ extern "C" {
 /* ======================================================================== */
 /* [constructor] */
 
-KNHAPI(knh_OutputStream_t*) new_OutputStream__io(Ctx *ctx, knh_String_t *urn, knh_io_t fd, knh_iodrv_t *drv)
+KNHAPI(OutputStream*) new_OutputStream__io(Ctx *ctx, String *urn, knh_io_t fd, knh_iodrv_t *drv)
 {
-	knh_OutputStream_t* o = (knh_OutputStream_t*)new_Object_bcid(ctx, CLASS_OutputStream, 0);
+	OutputStream* o = (OutputStream*)new_Object_bcid(ctx, CLASS_OutputStream, 0);
 	KNH_SETv(ctx, DP(o)->urn, urn);
 	DP(o)->fd = fd;
 	if(fd != -1) {
@@ -53,9 +53,9 @@ KNHAPI(knh_OutputStream_t*) new_OutputStream__io(Ctx *ctx, knh_String_t *urn, kn
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(knh_OutputStream_t*) new_OutputStream__FILE(Ctx *ctx, knh_String_t *urn, FILE *fp, knh_iodrv_t *drv)
+KNHAPI(OutputStream*) new_OutputStream__FILE(Ctx *ctx, String *urn, FILE *fp, knh_iodrv_t *drv)
 {
-	knh_OutputStream_t* o = (knh_OutputStream_t*)new_Object_bcid(ctx, CLASS_OutputStream, 0);
+	OutputStream* o = (OutputStream*)new_Object_bcid(ctx, CLASS_OutputStream, 0);
 	KNH_SETv(ctx, DP(o)->urn, urn);
 	if(fp != NULL) {
 		DP(o)->fd = (knh_io_t)fp;
@@ -69,9 +69,9 @@ KNHAPI(knh_OutputStream_t*) new_OutputStream__FILE(Ctx *ctx, knh_String_t *urn,
 /* ======================================================================== */
 /* [methods] */
 
-Object *knh_OutputStream_open(Ctx *ctx, knh_OutputStream_t *o, knh_String_t *urn, knh_String_t *m)
+Object *knh_OutputStream_open(Ctx *ctx, OutputStream *o, String *urn, String *m)
 {
-	knh_bytes_t fname = __tobytes(urn);
+	knh_bytes_t fname = knh_String_tobytes(urn);
 	knh_index_t loc = knh_bytes_index(fname, ':');
 
 	if(loc == -1 || (loc == 1 && isalpha(fname.buf[0]))) {  /* 'C:/' */
@@ -80,8 +80,8 @@ Object *knh_OutputStream_open(Ctx *ctx, knh_OutputStream_t *o, knh_String_t *urn
 	else {
 		DP(o)->driver = knh_getIODriver(ctx, knh_bytes_first(fname, loc));
 	}
-	char *mode = "w";
-	if(IS_NOTNULL(m)) mode = __tochar(m);
+	char *mode = "r";
+	if(IS_NOTNULL(m)) mode = knh_String_tochar(m);
 	DP(o)->fd = DP(o)->driver->fopen(ctx, fname, mode, knh_Context_isStrict(ctx));
 	if(DP(o)->fd != -1) {
 		KNH_SETv(ctx, DP(o)->urn, urn);
@@ -99,9 +99,9 @@ Object *knh_OutputStream_open(Ctx *ctx, knh_OutputStream_t *o, knh_String_t *urn
 /* ======================================================================== */
 /* [methods] */
 
-void knh_OutputStream_putc(Ctx *ctx, knh_OutputStream_t *o, knh_int_t ch)
+void knh_OutputStream_putc(Ctx *ctx, OutputStream *o, knh_int_t ch)
 {
-	knh_Bytes_t *ba = DP(o)->ba;
+	Bytes *ba = DP(o)->ba;
 	KNH_ASSERT(IS_Bytes(ba));
 	knh_Bytes_putc(ctx, ba, ch);
 	if(!knh_OutputStream_isStoringBuffer(o) && ba->size > DP(o)->driver->bufsiz) {
@@ -113,9 +113,9 @@ void knh_OutputStream_putc(Ctx *ctx, knh_OutputStream_t *o, knh_int_t ch)
 
 /* ------------------------------------------------------------------------ */
 
-void knh_OutputStream_write(Ctx *ctx, knh_OutputStream_t *o, knh_bytes_t buf)
+void knh_OutputStream_write(Ctx *ctx, OutputStream *o, knh_bytes_t buf)
 {
-	knh_Bytes_t *ba = DP(o)->ba;
+	Bytes *ba = DP(o)->ba;
 	KNH_ASSERT(IS_Bytes(ba));
 	knh_Bytes_write(ctx, ba, buf);
 	if(!knh_OutputStream_isStoringBuffer(o) && ba->size > DP(o)->driver->bufsiz) {
@@ -126,19 +126,21 @@ void knh_OutputStream_write(Ctx *ctx, knh_OutputStream_t *o, knh_bytes_t buf)
 }
 
 /* ------------------------------------------------------------------------ */
+/* @method void OutputStream.flush() */
 
-void knh_OutputStream_flush(Ctx *ctx, knh_OutputStream_t *o)
+void knh_OutputStream_flush(Ctx *ctx, OutputStream *o)
 {
 	if(!knh_OutputStream_isStoringBuffer(o)) {
-		knh_Bytes_t *ba = DP(o)->ba;
+		Bytes *ba = DP(o)->ba;
 		DP(o)->driver->fwrite(ctx, DP(o)->fd, (char*)(ba)->buf, (ba)->size);
 		knh_Bytes_clear(ba, 0);
 	}
 }
 
 /* ------------------------------------------------------------------------ */
+/* @method void OutputStream.clear() */
 
-void knh_OutputStream_clear(Ctx *ctx, knh_OutputStream_t *o)
+void knh_OutputStream_clear(Ctx *ctx, OutputStream *o)
 {
 	if(knh_OutputStream_isStoringBuffer(o)) {
 		knh_Bytes_clear(DP(o)->ba, 0);
@@ -146,8 +148,9 @@ void knh_OutputStream_clear(Ctx *ctx, knh_OutputStream_t *o)
 }
 
 /* ------------------------------------------------------------------------ */
+/* @method void OutputStream.close() */
 
-void knh_OutputStream_close(Ctx *ctx, knh_OutputStream_t *o)
+void knh_OutputStream_close(Ctx *ctx, OutputStream *o)
 {
 	knh_OutputStream_flush(ctx, o);
 	f_io_close f = DP(o)->driver->fclose;
@@ -158,40 +161,38 @@ void knh_OutputStream_close(Ctx *ctx, knh_OutputStream_t *o)
 
 /* ------------------------------------------------------------------------ */
 
-int knh_OutputStream_isClosed(knh_OutputStream_t *o)
+int knh_OutputStream_isClosed(OutputStream *o)
 {
 	return (DP(o)->fd == -1);
 }
 
 /* ------------------------------------------------------------------------ */
 
-void knh_OutputStream_setEncoding(Ctx *ctx, knh_OutputStream_t *o, knh_String_t *enc)
+void knh_OutputStream_indent_inc(Ctx *ctx, OutputStream *o)
 {
-	if(IS_NULL(enc)) {
-		KNH_SETv(ctx, DP(o)->enc, TS_ENCODING);
-		KNH_SETv(ctx, DP(o)->bconv, KNH_NULL);
-	}
-	else if(knh_bytes_strcasecmp(__tobytes(enc), STEXT(KONOHA_ENCODING)) == 0) {
-		KNH_SETv(ctx, DP(o)->enc, TS_ENCODING);
-		KNH_SETv(ctx, DP(o)->bconv, KNH_NULL);
-	}
-	else {
-		knh_BytesConv_t *bout = new_BytesConv__out(ctx, __tochar(enc));
-		if(IS_NULL(bout)) {
-			KNH_SETv(ctx, DP(o)->enc, TS_ENCODING);
-			KNH_WARNING(ctx, "unsupported character encoding: %s", __tochar(enc));
-		}
-		else {
-			KNH_SETv(ctx, DP(o)->enc, enc);
-		}
-		KNH_SETv(ctx, DP(o)->bconv, bout);
-	}
+	DP(o)->indent++;
 }
 
-
 /* ------------------------------------------------------------------------ */
 
-void knh_OutputStream_print_(Ctx *ctx, knh_OutputStream_t *o, knh_bytes_t str, knh_bool_t isnl)
+void knh_OutputStream_indent_dec(Ctx *ctx, OutputStream *o)
+{
+	DP(o)->indent--;
+}
+
+/* ------------------------------------------------------------------------ */
+
+void knh_OutputStream_write_indent(Ctx *ctx, OutputStream *o)
+{
+	int i;
+	for(i = 0; i < DP(o)->indent; i++) {
+		knh_OutputStream_write(ctx, o, knh_String_tobytes(DP(o)->TAB));
+	}
+}
+
+/* ------------------------------------------------------------------------ */
+
+void knh_OutputStream_print_(Ctx *ctx, OutputStream *o, knh_bytes_t str, knh_bool_t isnl)
 {
 	if(str.len > 0) {
 		if(knh_OutputStream_isBOL(o)) {
@@ -212,7 +213,7 @@ void knh_OutputStream_print_(Ctx *ctx, knh_OutputStream_t *o, knh_bytes_t str, k
 
 /* ======================================================================== */
 
-KNHAPI(knh_OutputStream_t*) new_FileOutputStream(Ctx *ctx, knh_bytes_t file, char *mode, int isThrowable)
+KNHAPI(OutputStream*) new_FileOutputStream(Ctx *ctx, knh_bytes_t file, char *mode, int isThrowable)
 {
 	knh_iodrv_t *drv = knh_getIODriver(ctx, STEXT("file"));
 	knh_io_t fd = drv->fopen(ctx, file, mode, isThrowable);
@@ -221,9 +222,9 @@ KNHAPI(knh_OutputStream_t*) new_FileOutputStream(Ctx *ctx, knh_bytes_t file, cha
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(knh_OutputStream_t*) new_BytesOutputStream(Ctx *ctx, knh_Bytes_t *ba)
+KNHAPI(OutputStream*) new_BytesOutputStream(Ctx *ctx, Bytes *ba)
 {
-	knh_OutputStream_t* o = (knh_OutputStream_t*)new_Object_bcid(ctx, CLASS_OutputStream, -1);
+	OutputStream* o = (OutputStream*)new_Object_bcid(ctx, CLASS_OutputStream, -1);
 	KNH_ASSERT(IS_Bytes(ba));
 	knh_OutputStream_setBOL(o, 1);
 	KNH_SETv(ctx, DP(o)->ba, ba);
