@@ -28,7 +28,7 @@
 /* ************************************************************************ */
 
 #include"commons.h"
-#if defined(KONOHA_ON_LKM) && defined(__linux__)
+#ifdef KONOHA_ON_LKM
 #include<linux/string.h>
 #else
 #include<string.h>
@@ -44,6 +44,11 @@
 extern "C" {
 #endif
 
+/* ------------------------------------------------------------------------ */
+/* [macros] */
+
+#define _KNH_SIZE(v)         knh_size(v)
+
 /* ------------------------------------------------------------------------ */
 /* [lib] */
 
@@ -60,14 +65,14 @@ size_t knh_size(size_t s)
 KNHAPI(knh_bytes_t) new_bytes(char *c)
 {
 	knh_bytes_t v;
-	// TODO
-	// care if c is NULL
-	//DBG2_ASSERT(c != NULL);
 	v.buf = (knh_uchar_t*)c;
 	v.len = knh_strlen(c);
 	return v;
 }
 
+#undef  _B
+#define _B(c)     new_bytes(c)
+
 /* ------------------------------------------------------------------------ */
 
 KNHAPI(knh_bytes_t) new_bytes__2(char *buf, size_t len)
@@ -78,6 +83,11 @@ KNHAPI(knh_bytes_t) new_bytes__2(char *buf, size_t len)
 	return v;
 }
 
+#define _B2(c,n)   new_bytes__2(c,n)
+#define _STEXT(c)  new_bytes__2(c,sizeof(c)-1)
+#define _ISB(t,c) (t.len == (sizeof(c)-1) && knh_strncmp((char*)t.buf,c,t.len) == 0)
+
+
 /* ======================================================================== */
 /* [lib] */
 
@@ -163,23 +173,6 @@ KNHAPI(knh_bool_t) knh_bytes_endsWith(knh_bytes_t v1, knh_bytes_t v2)
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(knh_bool_t) knh_bytes_matchWildCard(knh_bytes_t t, knh_bytes_t p)
-{
-	if(p.len == 0 || (p.len == 1 && p.buf[0] == '*')) {
-		return 1;
-	}
-	else if(p.buf[p.len-1] == '*') {
-		p.len -= 1;
-		return knh_bytes_startsWith(t, p);
-	}
-	else {
-		TODO();
-		return 0;
-	}
-}
-
-/* ------------------------------------------------------------------------ */
-
 KNHAPI(knh_index_t) knh_bytes_index(knh_bytes_t v, knh_intptr_t ch)
 {
 	size_t i;
@@ -208,21 +201,6 @@ KNHAPI(knh_index_t) knh_bytes_rindex(knh_bytes_t v, knh_intptr_t ch)
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(knh_bytes_t) knh_bytes_mod(knh_bytes_t t, int ch)
-{
-	int i;
-	for(i = 0; i < t.len; i++) {
-		if(t.buf[i] == ch) {
-			t.buf = t.buf + i + 1;
-			t.len = t.len -(i+1);
-			return t;
-		}
-	}
-	return t;
-}
-
-/* ------------------------------------------------------------------------ */
-
 KNHAPI(knh_bytes_t) knh_bytes_rmod(knh_bytes_t t, int ch)
 {
 	int i;
@@ -403,9 +381,9 @@ int knh_bytes_parsefloat(knh_bytes_t t, knh_float_t *value)
 	}
 
 	size_t i = 0;
-	knh_float_t v = KNH_FLOAT_ZERO, prev = KNH_FLOAT_ZERO, c = KNH_FLOAT_ONE;
+	knh_float_t v = 0.0, prev = 0.0, c = 1.0;
 
-	if(t.buf[0] == '-' || t.buf[0] == '+') i = 1;
+	if(t.buf[0] == '-') i = 1;
 
 	for(;i < t.len; i++) {
 		if('0' <= t.buf[i] && t.buf[i] <= '9') {
@@ -480,7 +458,7 @@ KNHAPI(knh_intptr_t) knh_bytes_toint(knh_bytes_t t)
 			else if(t.buf[1] == 'b') {
 				base = 2;  i = 2;
 			}
-		}else if(t.buf[0] == '-' || t.buf[0] == '+') {
+		}else if(t.buf[0] == '-') {
 			base = 10; i = 1;
 		}
 	}
@@ -516,7 +494,8 @@ KNHAPI(knh_intptr_t) knh_bytes_toint(knh_bytes_t t)
 KNHAPI(knh_float_t) knh_bytes_tofloat(knh_bytes_t t)
 {
 	size_t i = 0, c = 1;
-	knh_float_t v = KNH_FLOAT_ZERO;
+	knh_float_t v = 0.0;
+
 	if(t.buf[0] == '-') i = 1;
 
 	for(;i < t.len; i++) {
@@ -582,24 +561,6 @@ KNHAPI(knh_int64_t) knh_bytes_toint64(knh_bytes_t t)
 	return n;
 }
 
-/* ======================================================================== */
-/* [parse] */
-
-KNHAPI(knh_bytes_t) knh_bytes_parsecid(Ctx *ctx, knh_bytes_t t, int delim, knh_class_t *cid)
-{
-	knh_index_t loc = knh_bytes_index(t, delim);
-	if(loc > 0) {
-		*cid = knh_NameSpace_getcid(ctx, ctx->share->mainns, knh_bytes_first(t, loc));
-		return knh_bytes_last(t, delim + 1);
-	}
-	else {
-		*cid = CLASS_unknown;
-		return t;
-	}
-}
-
-/* ------------------------------------------------------------------------ */
-
 /* ======================================================================== */
 /* [URL] */
 
@@ -651,7 +612,7 @@ KNHAPI(int) knh_bytes_parseURL(knh_bytes_t url, char *buf, size_t bufsiz)
 		knh_bytes_t t = knh_bytes_last(url, loc+3);
 		loc = knh_bytes_index(t, '@');
 		if(loc > 0) {
-			t = knh_bytes_last(t, loc+1);
+			knh_bytes_t t = knh_bytes_last(t, loc+1);
 		}
 		knh_format_join2(buf, bufsiz, scheme, t);
 		return 1;
