@@ -1,7 +1,7 @@
 /****************************************************************************
  * KONOHA COPYRIGHT, LICENSE NOTICE, AND DISCRIMER
  *
- * Copyright (c) 2005-2008, Kimio Kuramitsu <kimio at ynu.ac.jp>
+ * Copyright (c) 2005-2009, Kimio Kuramitsu <kimio at ynu.ac.jp>
  *           (c) 2008-      Konoha Software Foundation
  * All rights reserved.
  *
@@ -28,7 +28,6 @@
 /* ************************************************************************ */
 
 #include"commons.h"
-#include<konoha/gen/konohac_klr_.h>
 
 /* ************************************************************************ */
 
@@ -36,6 +35,7 @@
 extern "C" {
 #endif
 
+static int knh_Asm_inTry(Asm *abr);
 void knh_code_traverse(Ctx *ctx, knh_code_t *pc, knh_ftraverse ftr);
 void knh_Asm_writeAddress(Ctx *ctx, Asm *o, knh_code_t *pc_start);
 
@@ -48,7 +48,7 @@ static knh_labelid_t knh_Asm_newLabelId(Ctx *ctx, Asm *abr, Token *tk);
 
 #define KNH_ASM_PANIC(ctx, abr, fmt, ...) {\
 		knh_Asm_setCancelled(abr, 1);\
-		fprintf(stderr, "PANIC[%s:%d/%s]: ", KNH_SAFEFILE(__FILE__), __LINE__, __FUNCTION__); \
+		fprintf(stderr, "PANIC[%s:%d/%s]: ", knh_safefile(__FILE__), __LINE__, __FUNCTION__); \
 		fprintf(stderr, fmt, ## __VA_ARGS__); \
 		fprintf(stderr, "\n"); \
 	}\
@@ -56,10 +56,6 @@ static knh_labelid_t knh_Asm_newLabelId(Ctx *ctx, Asm *abr, Token *tk);
 static
 void TERMs_asm(Ctx *ctx, Stmt *stmt, size_t n, Asm *abr, knh_type_t reqt, int sfpidx);
 
-/* ======================================================================== */
-/* [structs] */
-
-
 /* ======================================================================== */
 /* [constructor] */
 
@@ -71,44 +67,43 @@ static Asm *new_Asm(Ctx *ctx)
 /* ======================================================================== */
 /* [Asm] */
 
-Asm* knh_Context_getAsm(Ctx *ctx)
+void knh_Context_initAsm(Ctx *ctx)
 {
-	if(IS_NULL(ctx->cmpr)) {
-		KNH_SETv(ctx, ((Context*)ctx)->cmpr, new_Asm(ctx));
+	if(IS_NULL(ctx->abr)) {
+		KNH_SETv(ctx, ((Context*)ctx)->abr, new_Asm(ctx));
 	}
-	return ctx->cmpr;
 }
 
 /* ======================================================================== */
 /* [namespace] */
 
-NameSpace *knh_Context_getNameSpace(Ctx *ctx)
+NameSpace *knh_getCurrentNameSpace(Ctx *ctx)
 {
+	knh_Context_initAsm(ctx);
 	return DP(knh_Context_getAsm(ctx))->ns;
 }
 
 /* ------------------------------------------------------------------------ */
 
-NameSpace *knh_Context_setNameSpace(Ctx *ctx, String *nsname)
+NameSpace *knh_setCurrentNameSpace(Ctx *ctx, String *nsname)
 {
+	knh_Context_initAsm(ctx);
 	Asm *abr = knh_Context_getAsm(ctx);
-	if(IS_NULL(nsname)) {
-		if(IS_NameSpace(DP(abr)->ns)) {
-			KNH_SETv(ctx, DP(abr)->ns, knh_System_getNameSpace(ctx, knh_String_tobytes(TS_main)));
-		}
+	if(IS_NULL(nsname) || knh_String_equals(nsname, STEXT("main"))) {
+		KNH_SETv(ctx, DP(abr)->ns, ctx->share->mainns);
 	}
 	else {
-		KNH_SETv(ctx, DP(abr)->ns, knh_System_getNameSpace(ctx, knh_String_tobytes(nsname)));
+		KNH_SETv(ctx, DP(abr)->ns, knh_getNameSpace(ctx, knh_String_tobytes(nsname)));
 	}
 	return DP(abr)->ns;
 }
 
 /* ------------------------------------------------------------------------ */
 
-NameSpace *knh_Context_switchNameSpace(Ctx *ctx, NameSpace *newns)
+NameSpace *knh_switchCurrentNameSpace(Ctx *ctx, NameSpace *newns)
 {
+	knh_Context_initAsm(ctx);
 	Asm *abr = knh_Context_getAsm(ctx);
-	KNH_ASSERT(IS_NameSpace(DP(abr)->ns));
 	NameSpace *oldns = DP(abr)->ns;
 	KNH_SETv(ctx, DP(abr)->ns, newns);
 	return oldns;
@@ -116,9 +111,10 @@ NameSpace *knh_Context_switchNameSpace(Ctx *ctx, NameSpace *newns)
 
 /* ------------------------------------------------------------------------ */
 
-Script *knh_Asm_getScript(Ctx *ctx, Asm *o)
+Script *knh_getCurrentScript(Ctx *ctx)
 {
-	return knh_NameSpace_getScript(ctx, DP(o)->ns);
+	Asm *abr = knh_Context_getAsm(ctx);
+	return knh_NameSpace_getScript(ctx, DP(abr)->ns);
 }
 
 /* ======================================================================== */
@@ -139,7 +135,6 @@ void knh_Asm_prepare(Ctx *ctx, Asm *abr, Method *mtd, Stmt *stmt)
 		DP(abr)->vars[i].flag  = 0;
 		DP(abr)->vars[i].type  = TYPE_Any;
 		DP(abr)->vars[i].fn    = FIELDN_NONAME;
-//		DP(abr)->vars[i].count = 0;
 		KNH_SETv(ctx, DP(abr)->vars[i].value, KNH_NULL);
 	}
 	knh_Asm_initReg(ctx, abr);
@@ -156,7 +151,7 @@ void knh_Asm_prepare(Ctx *ctx, Asm *abr, Method *mtd, Stmt *stmt)
 	DP(abr)->labelmax = 0;
 	knh_Array_clear(ctx, DP(abr)->lstacks);
 
-	DP(abr)->fileid  = SP(stmt)->fileid;
+	DP(abr)->uri  = SP(stmt)->uri;
 	DP(abr)->line   = 0;
 	knh_Bytes_clear(DP(abr)->elf, 0);
 	knh_Bytes_clear(DP(abr)->dwarf, 0);
@@ -181,7 +176,7 @@ void knh_Asm_initThis(Ctx *ctx, Asm *abr, knh_class_t cid)
 
 void knh_Asm_initThisScript(Ctx *ctx, Asm *abr)
 {
-	knh_Asm_initThis(ctx, abr, knh_Object_cid(knh_Asm_getScript(ctx, abr)));
+	knh_Asm_initThis(ctx, abr, knh_Object_cid(knh_getCurrentScript(ctx)));
 }
 
 /* ------------------------------------------------------------------------ */
@@ -222,7 +217,7 @@ void knh_Asm_finish(Ctx *ctx, Asm *abr)
 		return;
 	}
 
-	KLRCode *vmc = new_KLRCode(ctx, DP(abr)->fileid,
+	KLRCode *vmc = new_KLRCode(ctx, DP(abr)->uri,
 			knh_Bytes_tobytes(DP(abr)->elf), knh_Bytes_tobytes(DP(abr)->dwarf));
 	knh_Method_setKLRCode(ctx, mtd, vmc);
 	knh_Asm_writeAddress(ctx, abr, DP(vmc)->code);
@@ -402,7 +397,7 @@ void KNH_ASM_BOX(Ctx *ctx, Asm *abr, knh_type_t atype, int a)
 	if(knh_rtti_issync(abr, a)) return;
 	knh_class_t cid = CLASS_type(atype);
 	KNH_ASSERT_cid(cid);
-	knh_class_t bcid = ctx->share->ClassTable[cid].bcid;
+	knh_class_t bcid = ClassTable(cid).bcid;
 	if(bcid == CLASS_Boolean || bcid == CLASS_Int || bcid == CLASS_Float) {
 		if(IS_NNTYPE(atype)) {
 			KNH_ASM_BOX_(ctx, abr, a, cid);
@@ -422,7 +417,7 @@ void KNH_ASM_NNBOX(Ctx *ctx, Asm *abr, knh_type_t atype, int a)
 	if(knh_rtti_issync(abr, a)) return;
 	knh_class_t cid = CLASS_type(atype);
 	KNH_ASSERT_cid(cid);
-	knh_class_t bcid = ctx->share->ClassTable[cid].bcid;
+	knh_class_t bcid = ClassTable(cid).bcid;
 	if(bcid == CLASS_Boolean || bcid == CLASS_Int || bcid == CLASS_Float) {
 		if(IS_NNTYPE(atype)) {
 			KNH_ASM_NNBOX_(ctx, abr, a, cid);
@@ -495,7 +490,7 @@ void KNH_ASM_SMOV(Ctx *ctx, Asm *abr, knh_type_t atype, int a, Token *tkb)
 			}
 			if(IS_ubxtype(btype)) {
 				KNH_ASM_MOVn_(ctx, abr, sfi_(a), sfi_(b));
-				if(IS_ANY(atype)) {
+				if(IS_ANY(atype) || !IS_NNTYPE(atype)) {
 					KNH_ASM_NNBOX(ctx, abr, btype, a);
 				}
 			}
@@ -1129,12 +1124,13 @@ static
 void TERMs_ASM_THROW(Ctx *ctx, Stmt *stmt, size_t n, Asm *abr)
 {
 	Token *tk = DP(stmt)->tokens[n];
+
 	if(IS_Token(tk) && SP(tk)->tt == TT_STACK /*TT_LOCAL ??*/) {
-		KNH_ASM_THROW_(ctx, abr, sfi_(DP(tk)->index));
+		KNH_ASM_THROW_(ctx, abr, knh_Asm_inTry(abr), sfi_(DP(tk)->index));
 	}
 	else {
 		TERMs_asm(ctx, stmt, n, abr, NNTYPE_Exception, DP(abr)->stack);
-		KNH_ASM_THROW_(ctx, abr, sfi_(DP(abr)->stack));
+		KNH_ASM_THROW_(ctx, abr, knh_Asm_inTry(abr), sfi_(DP(abr)->stack));
 	}
 }
 
@@ -1463,10 +1459,17 @@ int knh_StmtOP_asm(Ctx *ctx, Stmt *stmt, Asm *abr, knh_type_t reqt, int sfpidx)
 					KNH_ASM_fSUBn_(ctx, abr, sfi_(sfpidx), sfi_(a), b);
 					return 1;
 				case METHODN_opDiv:
+#ifndef KONOHA_ON_LKM
 					if(b == 0.0) {
 						b = 1.0;
 						knh_Asm_perror(ctx, abr, KERR_ERRATA, _("divided by zero: 0.0 ==> 1.0"));
 					}
+#else
+					if(b == 0) {
+						b = 1;
+						knh_Asm_perror(ctx, abr, KERR_ERRATA, _("divided by zero: 0.0 ==> 1.0"));
+					}
+#endif
 					KNH_ASM_fDIVn_(ctx, abr, sfi_(sfpidx), sfi_(a), b);
 					return 1;
 				}
@@ -1513,8 +1516,8 @@ void knh_StmtCALL_asm(Ctx *ctx, Stmt *stmt, Asm *abr, knh_type_t reqt, int sfpid
 		for(i = 1; i < DP(stmt)->size; i++) {
 			TERMs_asm(ctx, stmt, i, abr, TYPE_Any, local + i);
 		}
-		KNH_ASM_AINVOKE_(ctx, abr, sfi_(local), (knh_ushort_t)DP(stmt)->size, UP(mtd));
-		return;
+		KNH_ASM_AINVOKE_(ctx, abr, sfi_(local), (knh_ushort_t)DP(stmt)->size/*, UP(mtd)*/);
+		goto L_RTYPE;
 	}
 	/* INSTRUCTION */
 	if(cid == CLASS_Array || cid == CLASS_IArray || cid == CLASS_FArray) {
@@ -1659,7 +1662,7 @@ static
 void knh_StmtMT_asm(Ctx *ctx, Stmt *stmt, Asm *abr, knh_type_t reqt, int sfpidx)
 {
 	int local = ASML(abr, sfpidx);
-	TERMs_asm(ctx, stmt, 1, abr, TYPE_any, local);
+	TERMs_asm(ctx, stmt, 1, abr, TYPE_Any, local);
 	Token *tk = DP(stmt)->tokens[0];
 	if(DP(stmt)->size == 2) {
 		KNH_ASM_TOSTR_(ctx, abr, local, DP(tk)->mn);
@@ -1844,6 +1847,9 @@ knh_StmtEXPR_asm(Ctx *ctx, Stmt *stmt, Asm *abr, knh_type_t reqt, int sfpidx)
 	if(IS_ANY(reqt)) {
 		KNH_ASM_NNBOX(ctx, abr, DP(stmt)->type, sfpidx);
 	}
+	else if(IS_NNTYPE(reqt) && !IS_NNTYPE(DP(stmt)->type)) {
+		KNH_ASM_ISNULL_(ctx, abr, sfpidx);
+	}
 }
 
 /* ------------------------------------------------------------------------ */
@@ -1973,9 +1979,14 @@ knh_labelid_t knh_Asm_pushLabelStack(Ctx *ctx, Asm *abr, Stmt *stmt)
 	{
 		knh_labelid_t labelid = knh_Asm_newLabelId(ctx, abr, tkL);
 		knh_labelid_t labelid2 = knh_Asm_newLabelId(ctx, abr, NULL);
-		DBG2_P("label %s (begin, end) = %d, %d", sToken(tkL), labelid, labelid2);
-		KNH_ASSERT(labelid + 1 == labelid2);
-		return labelid;
+		if(labelid + 1 == labelid2) {
+			return labelid;
+		}
+		else {
+			DBG2_P("label %s (begin, end) = %d, %d", sToken(tkL), labelid, labelid2);
+			KNH_ASSERT(labelid + 1 == labelid2);
+			return labelid;
+		}
 	}
 }
 
@@ -2135,18 +2146,18 @@ void knh_StmtFOREACH_asm(Ctx *ctx, Stmt *stmt, Asm *abr)
 	knh_class_t itrcid = CLASS_type(DP(tkitr)->type);
 	KNH_ASSERT_cid(itrcid); itrcid = ctx->share->ClassTable[itrcid].p1;
 	if(ncid == itrcid || ncid == CLASS_Any || knh_class_instanceof(ctx, ncid, itrcid)) {
-		KNH_ASM_NEXT_(ctx, abr, lbend, sfi_(DP(tkn)->index), sfi_(DP(tkitr)->index), sfi_(DP(abr)->stack));
+		KNH_ASM_NEXT_(ctx, abr, lbend, sfi_(DP(tkn)->index), sfi_(DP(tkitr)->index));
 	}
 	else if(!knh_Stmt_isMAPNEXT(stmt)) {
-		KNH_ASM_INEXT_(ctx, abr, lbend, ncid, sfi_(DP(tkn)->index), sfi_(DP(tkitr)->index), sfi_(DP(abr)->stack));
+		KNH_ASM_INEXT_(ctx, abr, lbend, ncid, sfi_(DP(tkn)->index), sfi_(DP(tkitr)->index));
 	}
 	else {
 		Mapper *mpr = knh_Class_getMapper(ctx, itrcid, ncid);
 		if(IS_NOTNULL(mpr) && knh_Mapper_isFinal(mpr)) {
-			KNH_ASM_SMAPNEXT_(ctx, abr, lbend, sfi_(DP(tkn)->index), sfi_(DP(tkitr)->index), sfi_(DP(abr)->stack), UP(mpr));
+			KNH_ASM_SMAPNEXT_(ctx, abr, lbend, sfi_(DP(tkn)->index), sfi_(DP(tkitr)->index), UP(mpr));
 		}
 		else {
-			KNH_ASM_MAPNEXT_(ctx, abr, lbend, ncid, sfi_(DP(tkn)->index), sfi_(DP(tkitr)->index), sfi_(DP(abr)->stack));
+			KNH_ASM_MAPNEXT_(ctx, abr, lbend, ncid, sfi_(DP(tkn)->index), sfi_(DP(tkitr)->index));
 		}
 	}
 
@@ -2163,6 +2174,12 @@ void knh_StmtFOREACH_asm(Ctx *ctx, Stmt *stmt, Asm *abr)
 /* ======================================================================== */
 /* [TRY] */
 
+
+static int knh_Asm_inTry(Asm *abr)
+{
+	return IS_Stmt(DP(abr)->finallyStmt);
+}
+
 /* ------------------------------------------------------------------------ */
 
 static
@@ -2199,40 +2216,39 @@ void knh_StmtTRY_asm(Ctx *ctx, Stmt *stmt, Asm *abr)
 	knh_labelid_t lbcatch   = knh_Asm_newLabelId(ctx, abr, NULL);
 	knh_labelid_t lbfinally = knh_Asm_newLabelId(ctx, abr, NULL);
 
-	Token *tkhdr = DP(stmt)->tokens[TRY_finally + 1];
-	KNH_ASSERT(SP(tkhdr)->tt == TT_STACK);
+	Token *tkHDR = DP(stmt)->tokens[TRY_finally + 1];
+	KNH_ASSERT(SP(tkHDR)->tt == TT_STACK);
 
 	knh_Asm_setFinallyStmt(ctx, abr, StmtTRY_finally(stmt));
 
 	/* try { */
-	KNH_ASM_TRY_(ctx, abr, lbcatch, sfi_(DP(tkhdr)->index));
+	KNH_ASM_TRY_(ctx, abr, lbcatch, sfi_(DP(tkHDR)->index));
 	TERMs_asmBLOCK(ctx, stmt, TRY_try, abr);
-	KNH_ASM_TRYEND_(ctx, abr, sfi_(DP(tkhdr)->index));
+	KNH_ASM_TRYEND_(ctx, abr, sfi_(DP(tkHDR)->index));
 	KNH_ASM_JMP(ctx, abr, lbfinally);
+	knh_Asm_setFinallyStmt(ctx, abr, (Stmt*)KNH_NULL); // InTry
 
 	/* catch */
 	KNH_ASM_GLABEL(ctx, abr, lbcatch);
-	Stmt *stmtcatch = DP(stmt)->stmts[TRY_catch];
-	while(IS_Stmt(stmtcatch)) {
-		if(SP(stmtcatch)->stt == STT_CATCH) {
-			Object *emsg = DP(DP(stmtcatch)->tokens[0])->data;
+	Stmt *stmtCATCH = DP(stmt)->stmts[TRY_catch];
+	KNH_ASSERT(IS_Stmt(stmtCATCH));
+	while(IS_Stmt(stmtCATCH)) {
+		if(SP(stmtCATCH)->stt == STT_CATCH) {
+			Object *emsg = DP(DP(stmtCATCH)->tokens[0])->data;
 			KNH_ASSERT(IS_String(emsg));
-			Token *tkn = DP(stmtcatch)->tokens[1];
+			Token *tkn = DP(stmtCATCH)->tokens[1];
 			KNH_ASSERT(SP(tkn)->tt == TT_STACK);
 			lbcatch = knh_Asm_newLabelId(ctx, abr, NULL);
-			KNH_ASM_CATCH_(ctx, abr, lbcatch, sfi_(DP(tkhdr)->index), sfi_(DP(tkn)->index), emsg);
-			TERMs_asmBLOCK(ctx, stmtcatch, 2, abr);
+			KNH_ASM_CATCH_(ctx, abr, lbcatch, sfi_(DP(tkHDR)->index), sfi_(DP(tkn)->index), emsg);
+			TERMs_asmBLOCK(ctx, stmtCATCH, 2, abr);
 			KNH_ASM_JMP(ctx, abr, lbfinally);  /* GOTO FINALLY */
 			KNH_ASM_GLABEL(ctx, abr, lbcatch); /* _CATCH_NEXT_ */
 		}
-		stmtcatch = DP(stmtcatch)->next;
+		stmtCATCH = DP(stmtCATCH)->next;
 	}
-
-	knh_Asm_setFinallyStmt(ctx, abr, (Stmt*)KNH_NULL);
-
 	KNH_ASM_GLABEL(ctx, abr, lbfinally); /* FINALLY */
 	TERMs_asmBLOCK(ctx, stmt, TRY_finally, abr);
-	KNH_ASM_THROW_AGAIN_(ctx, abr, sfi_(DP(tkhdr)->index));
+	KNH_ASM_THROW_AGAIN_(ctx, abr, sfi_(DP(tkHDR)->index));
 }
 
 /* ------------------------------------------------------------------------ */
@@ -2272,11 +2288,11 @@ void knh_StmtERR_asm(Ctx *ctx, Stmt *stmt, Asm *abr)
 {
 	if(!IS_bString(DP(stmt)->errMsg)) {
 		char buf[512];
-		knh_snprintf(buf, sizeof(buf), "Script!!: running errors at %s:%d", FILEIDN(SP(stmt)->fileid), SP(stmt)->line);
+		knh_snprintf(buf, sizeof(buf), "Script!!: you'll fix bugs at %s:%d", URIDN(SP(stmt)->uri), SP(stmt)->line);
 		KNH_SETv(ctx, DP(stmt)->errMsg, new_String(ctx, B(buf), NULL));
 		KNH_SETv(ctx, DP(stmt)->next, KNH_NULL);
 	}
-	KNH_ASM_THROWs_(ctx, abr, (Object*)DP(stmt)->errMsg);
+	KNH_ASM_THROWs_(ctx, abr, knh_Asm_inTry(abr), (Object*)DP(stmt)->errMsg);
 }
 
 /* ======================================================================== */
@@ -2315,7 +2331,7 @@ void knh_StmtPRINT_asm(Ctx *ctx, Stmt *stmt, Asm *abr)
 	KNH_ASM_SKIP_(ctx, abr, lbskip);
 	if(konoha_debugLevel() > 1) {
 		char buf[128];
-		knh_snprintf(buf, sizeof(buf), "[%s:%d]", FILEIDN(DP(abr)->fileid), DP(abr)->line);
+		knh_snprintf(buf, sizeof(buf), "[%s:%d]", URIDN(DP(abr)->uri), DP(abr)->line);
 		KNH_ASM_PMSG_(ctx, abr, flag | KNH_FLAG_PF_BOL, UP(new_String(ctx, B(buf), NULL)));
 	}
 	else if(flag != 0 ) {
@@ -2369,7 +2385,7 @@ void knh_StmtASSERT_asm(Ctx *ctx, Stmt *stmt, Asm *abr)
 	TERMs_ASM_JIFT(ctx, stmt, 0, abr, lbskip);
 	/*then*/
 	TERMs_asmBLOCK(ctx, stmt, 1, abr);
-	KNH_ASM_THROWs_(ctx, abr, UP(TS_AssertionException));
+	KNH_ASM_THROWs_(ctx, abr, knh_Asm_inTry(abr), UP(TS_AssertionException));
 	KNH_ASM_LLABEL(ctx, abr, lbskip);
 }
 
@@ -2502,6 +2518,7 @@ void KNH_ASM_METHOD(Ctx *ctx, Asm *abr, Method *mtd, Stmt *params, Stmt *body, i
 	NameSpace *ns = DP(abr)->ns;
 	KNH_ASSERT(IS_Method(mtd));
 	knh_Method_toAbstract(ctx, mtd);
+	DP(mtd)->uri = SP(body)->uri;
 
 	knh_Asm_prepare(ctx, abr, mtd, body);
 	knh_Asm_initThis(ctx, abr, DP(abr)->this_cid);
@@ -2526,6 +2543,7 @@ void KNH_ASM_METHOD(Ctx *ctx, Asm *abr, Method *mtd, Stmt *params, Stmt *body, i
 	}
 
 	knh_Asm_finish(ctx, abr);
+	DP(mtd)->sline = SP(body)->line;
 	DP(abr)->this_cid = prev_cid;
 }
 
