@@ -1,7 +1,7 @@
 /****************************************************************************
  * KONOHA COPYRIGHT, LICENSE NOTICE, AND DISCRIMER
  *
- * Copyright (c) 2005-2008, Kimio Kuramitsu <kimio at ynu.ac.jp>
+ * Copyright (c) 2005-2009, Kimio Kuramitsu <kimio at ynu.ac.jp>
  *           (c) 2008-      Konoha Software Foundation
  * All rights reserved.
  *
@@ -28,7 +28,11 @@
 /* ************************************************************************ */
 
 #include"commons.h"
+#ifdef KONOHA_ON_LKM
+#include"../../include/konoha/gen/konohac_klr_.h"
+#else
 #include<konoha/gen/konohac_klr_.h>
+#endif
 
 /* ************************************************************************ */
 
@@ -43,11 +47,11 @@ extern "C" {
 /* ======================================================================== */
 /* [constructors] */
 
-KLRCode* new_KLRCode(Ctx *ctx, knh_fileid_t fileid, knh_bytes_t elf, knh_bytes_t dwarf)
+KLRCode* new_KLRCode(Ctx *ctx, knh_uri_t uri, knh_bytes_t elf, knh_bytes_t dwarf)
 {
 	knh_KLRCode_t *o = (KLRCode*)new_Object_bcid(ctx, CLASS_KLRCode, 0);
 
-	DP(o)->fileid = fileid;
+	DP(o)->uri = uri;
 	DP(o)->code = (knh_code_t*)KNH_MALLOC(ctx, KNH_SIZE(elf.len));
 	DP(o)->size = elf.len;
 	knh_memcpy(DP(o)->code, elf.buf, elf.len);
@@ -75,7 +79,7 @@ void knh_Method_setKLRCode(Ctx *ctx, Method *mtd, KLRCode *code)
 		KNH_INITv(DP(mtd)->code, code);
 		knh_Method_setObjectCode(mtd, 1);
 	}
-	knh_Method_syncfunc(mtd, knh_KLRCode_exec);
+	knh_Method_syncFunc(mtd, knh_KLRCode_exec);
 	mtd->pc_start = DP(code)->code;
 }
 
@@ -109,24 +113,19 @@ char *knh_Method_file(Ctx *ctx, Method *mtd)
 		return "(naitive)";
 	}
 	KLRCode *o = (KLRCode*)DP(mtd)->code;
-	return FILEIDN(DP(o)->fileid);
+	return URIDN(DP(o)->uri);
 }
 
 /* ------------------------------------------------------------------------ */
 
 int knh_Method_pctoline(Method *mtd, knh_code_t *pc)
 {
-	if(!knh_Method_isObjectCode(mtd) || !IS_KLRCode((Object*)DP(mtd)->code)) {
-		return 0;
-	}
-	else {
-		KLRCode *o = (KLRCode*)DP(mtd)->code;
-		if(DP(o)->dwarf == NULL && DP(o)->dsize > 0) {
-			return 0;
-		}
-		else {
-			size_t i = 0, offset = pc - DP(o)->code;
-			for(i = 0; i < DP(o)->dsize - 1; i++) {
+	if(knh_Method_isObjectCode(mtd) && IS_KLRCode((Object*)DP(mtd)->code)) {
+		knh_KLRCode_t *o = (KLRCode*)DP(mtd)->code;
+		int offset = pc - DP(o)->code;
+		if(0 <= offset && offset <= DP(o)->size) {
+			int i;
+			for(i = 0; i < (int)DP(o)->dsize - 1; i++) {
 				if(DP(o)->dwarf[i].offset <= offset && offset < DP(o)->dwarf[i+1].offset) {
 					return DP(o)->dwarf[i].line;
 				}
@@ -134,6 +133,7 @@ int knh_Method_pctoline(Method *mtd, knh_code_t *pc)
 			return DP(o)->dwarf[DP(o)->dsize-1].line;
 		}
 	}
+	return 0;
 }
 
 /* ------------------------------------------------------------------------ */
@@ -175,7 +175,7 @@ int knh_Asm_classId(Ctx *ctx, Asm *abr, knh_class_t cid)
 void knh_Int__const(Ctx *ctx, Int *o, OutputStream *w, String *m)
 {
 	knh_write(ctx, w, STEXT("new_Int(ctx, "));
-	knh_write__i(ctx, w, o->n.ivalue);
+	knh_write_ifmt(ctx, w, KNH_INT_FMT, o->n.ivalue);
 	knh_write(ctx, w, STEXT(")"));
 }
 
@@ -196,7 +196,7 @@ void knh_Method__const(Ctx *ctx, Method *o, OutputStream *w, String *m)
 static
 void knh_Asm_genConstPool(Ctx *ctx, Asm *abr, OutputStream *w)
 {
-	knh_println(ctx, w, STEXT("size_t konoha_loadConstPools(Ctx *ctx)"));
+	knh_println(ctx, w, STEXT("size_t knh_loadConstPools(Ctx *ctx)"));
 	knh_println(ctx, w, STEXT("{"));
 	knh_println(ctx, w, STEXT("\tsize_t delta = (DP(ctx->sys)->constPools)->size;"));
 	if(IS_NOTNULL(DP(abr)->constPools)) {
@@ -204,7 +204,7 @@ void knh_Asm_genConstPool(Ctx *ctx, Asm *abr, OutputStream *w)
 		size_t i;
 		for(i = 0; i < (a)->size; i++) {
 			Object *ao = knh_Array_n(a, i);
-			knh_write(ctx, w, STEXT("\tkonoha_addConstPools(ctx, "));
+			knh_write(ctx, w, STEXT("\tknh_addConstPools(ctx, "));
 			knh_format(ctx, w, METHODN__const, ao, KNH_NULL);
 			knh_println(ctx, w, STEXT(");"));
 		}
@@ -246,7 +246,7 @@ void knh_Asm_genMethod(Ctx *ctx, Asm *abr, OutputStream *w)
 		}
 	}
 
-	knh_println(ctx, w, STEXT("knh_fmethod konoha_findCompiledFunc(Ctx *ctx, knh_bytes_t cname, knh_bytes_t mname)"));
+	knh_println(ctx, w, STEXT("knh_fmethod knh_findCompiledFunc(Ctx *ctx, knh_bytes_t cname, knh_bytes_t mname)"));
 	knh_println(ctx, w, STEXT("{"));
 	knh_println(ctx, w, STEXT("\tsize_t delta = (DP(ctx->sys)->constPools)->size;"));
 	if(IS_NOTNULL(DP(abr)->exportsMethods)) {
