@@ -1,7 +1,7 @@
 /****************************************************************************
  * KONOHA COPYRIGHT, LICENSE NOTICE, AND DISCRIMER
  *
- * Copyright (c) 2005-2008, Kimio Kuramitsu <kimio at ynu.ac.jp>
+ * Copyright (c) 2005-2009, Kimio Kuramitsu <kimio at ynu.ac.jp>
  *           (c) 2008-      Konoha Software Foundation
  * All rights reserved.
  *
@@ -36,10 +36,324 @@ extern "C" {
 #endif
 
 /* ======================================================================== */
-/* [CLASS] */
+/* [main] */
+
+static
+int knh_NameSpace_compile(Ctx *ctx, NameSpace *ns, Stmt *stmt, int isEval);
 
 /* ------------------------------------------------------------------------ */
-/* [META] */
+/* [import script] */
+
+static
+knh_bool_t knh_bytes_isSystemScript(knh_bytes_t path)
+{
+	size_t i;
+	for(i = 0; i < path.len; i++) {
+		int ch = path.buf[i];
+		if(ch == '/' || ch == '\\' || ch == '.') return 0;
+	}
+	return 1;
+}
+
+/* ------------------------------------------------------------------------ */
+
+static
+void knh_NameSpace_loaded(Ctx *ctx, NameSpace *ns, knh_uri_t uri)
+{
+	KNH_ASSERT(IS_NameSpace(ns));
+	String *s = knh_getResourceName(ctx, uri);
+	knh_uintptr_t trust = KNH_PATH_UNTRUSTED;
+	if(URI_ISTRUSTED(uri)) {
+		trust = KNH_PATH_TRUSTED;
+	}
+	if(IS_NULL(DP(ns)->pathTrustDictSet)) {
+		KNH_SETv(ctx, DP(ns)->pathTrustDictSet, new_DictSet0(ctx, 4));
+	}
+	knh_DictSet_set(ctx, DP(ns)->pathTrustDictSet, s, trust);
+}
+
+/* ------------------------------------------------------------------------ */
+
+static
+int knh_NameSpace_isLoaded(Ctx *ctx, NameSpace *ns, knh_uri_t uri)
+{
+	if(IS_NOTNULL(DP(ns)->pathTrustDictSet)) {
+		String *s = knh_getResourceName(ctx, uri);
+		knh_uintptr_t n = knh_DictSet_get__b(DP(ns)->pathTrustDictSet, knh_String_tobytes(s));
+		return (n != 0);
+	}
+	return 0;
+}
+
+/* ------------------------------------------------------------------------ */
+
+InputStream* new_ScriptInputStream(Ctx *ctx, knh_bytes_t path, knh_cwb_t *cwb, NameSpace *ns, int isThrowable)
+{
+	knh_uri_t uri = 0;
+	knh_iodrv_t *drv = NULL;
+	knh_cwb_t cwbbuf;
+	if(cwb == NULL) {
+		cwb = knh_cwb_openinit(ctx, &cwbbuf, path);
+	}
+	if(knh_bytes_startsWith(path, STEXT("http://"))) {
+		drv = knh_getIODriver(ctx, STEXT("http"));
+	}
+	else {
+		drv = knh_getIODriver(ctx, STEXT("file"));
+		knh_cwb_ospath(ctx, cwb);
+		if(!knh_cwb_isfile(ctx, cwb) && knh_bytes_isSystemScript(path)) {
+			String *spath = (String*)knh_Context_getProperty(ctx, (Context*)ctx, STEXT("konoha.script.path"));
+			if(IS_bString(spath)) {
+				knh_cwb_subclear(cwb, 0);
+				knh_cwb_write(ctx, cwb, knh_String_tobytes(spath));
+				knh_cwb_putc(ctx, cwb, '/');
+				knh_cwb_write(ctx, cwb, path);
+				knh_cwb_ospath(ctx, cwb);
+				if(knh_cwb_isfile(ctx, cwb)) {
+					uri = knh_cwb_getResourceId(ctx, cwb);
+					DBG2_ASSERT(uri != 0);
+					uri = URI_TRUSTED(uri);
+				}
+			}
+			spath = (String*)knh_Context_getProperty(ctx, (Context*)ctx, STEXT("user.script.path"));
+			if(IS_bString(spath) && uri == 0) {
+				knh_cwb_subclear(cwb, 0);
+				knh_cwb_write(ctx, cwb, knh_String_tobytes(spath));
+				knh_cwb_putc(ctx, cwb, '/');
+				knh_cwb_write(ctx, cwb, path);
+				knh_cwb_ospath(ctx, cwb);
+				if(knh_cwb_isfile(ctx, cwb)) {
+					uri = knh_cwb_getResourceId(ctx, cwb);
+					DBG2_ASSERT(uri != 0);
+					uri = URI_TRUSTED(uri);
+				}
+			}
+			if(uri == 0) {
+				knh_cwb_subclear(cwb, 0);
+				knh_cwb_write(ctx, cwb, path);
+			}
+		}
+	}
+	if(uri == 0) {
+		uri = knh_cwb_getResourceId(ctx, cwb);
+	}
+	if(knh_NameSpace_isLoaded(ctx, ns, uri)) {
+		KNH_WARNING(ctx, "Already imported: %s", (char*)path.buf);
+		if(!knh_ask(ctx, "Do you want to reload it [y/N] ?", 0)) {
+			return (InputStream*)KNH_DEF(ctx, CLASS_InputStream);
+		}
+	}
+	{
+		String *spath = knh_getResourceName(ctx, uri);
+		path = knh_String_tobytes(spath);
+		KNH_NOTICE(ctx, "importing script: %s", path.buf);
+		knh_io_t fd = drv->fopen(ctx, path, "r", isThrowable);
+		InputStream *in = new_InputStream__io(ctx, spath, fd, drv);
+		if(!knh_InputStream_isClosed(ctx, in)) {
+			DP(in)->uri = uri;
+			if(!knh_bytes_startsWith(path, STEXT("http://"))) {
+				knh_InputStream_setEncoding(ctx, in, KNH_ENC);
+			}
+		}
+		return in;
+	}
+}
+
+/* ------------------------------------------------------------------------ */
+
+Stmt *knh_InputStream_parseStmt(Ctx *ctx, InputStream *in, int isData)
+{
+	Token *tk = new_Token(ctx, 0, DP(in)->uri, 0, knh_char_totoken('{'));
+	KNH_LPUSH(ctx, tk);
+	KNH_LPUSH(ctx, in);
+	knh_Token_parse(ctx, tk, in);
+	//DBG2_DUMP(ctx, tk, KNH_NULL, "tokens");
+	return new_StmtINSTMT(ctx, tk, isData);
+}
+
+/* ------------------------------------------------------------------------ */
+
+static
+void knh_Asm_openDynamicLinkLibrary(Ctx *ctx, Asm *abr, knh_uri_t uri)
+{
+	knh_bytes_t path = knh_String_tobytes(knh_getResourceName(ctx, uri));
+	if(path.buf[0] != '(' && !knh_bytes_startsWith(path, STEXT("http://"))) {  // check "(eval)", "(shell)", or o
+		knh_cwb_t cwbbuf, *cwb = knh_cwb_open(ctx, &cwbbuf);
+		knh_index_t idx = knh_bytes_rindex(path, '.');
+		if(idx > 0) path = knh_bytes_first(path, idx);
+		knh_cwb_write(ctx, cwb, path);
+		knh_cwb_putc(ctx, cwb, '_');
+		knh_cwb_write(ctx, cwb, STEXT(KONOHA_PLATFORM));
+		knh_cwb_ospath(ctx, cwb);
+		DP(abr)->dlhdr =knh_cwb_dlopen(ctx, cwb);
+		if(DP(abr)->dlhdr != NULL) {
+			KNH_NOTICE(ctx, "opened dynamic library: %s", knh_cwb_tochar(ctx, cwb));
+		}
+		else {
+			if(knh_cwb_isfile(ctx, cwb)) {
+				KNH_WARNING(ctx, "cannot open dynamic library: %s", knh_cwb_tochar(ctx, cwb));
+			}
+		}
+		knh_cwb_close(cwb);
+	}
+	else {
+		DP(abr)->dlhdr = NULL;
+	}
+}
+
+/* ------------------------------------------------------------------------ */
+
+int knh_NameSpace_load(Ctx *ctx, NameSpace *ns, InputStream *in, int isEval, int isThrowable)
+{
+	int res = 0;
+	knh_sfp_t *lsfp = KNH_LOCAL(ctx);
+	knh_uri_t uri = DP(in)->uri;
+	KNH_LPUSH(ctx, in);
+	KNH_ASSERT(!knh_InputStream_isClosed(ctx, in));
+	KNH_LPUSH(ctx, new_ExceptionHandler(ctx));
+	KNH_TRY(ctx, L_CATCH, lsfp, 1);
+	knh_Context_initAsm(ctx);   // initialization
+	{
+		Stmt *stmt = knh_InputStream_parseStmt(ctx, in, 0/*isData*/);
+		KNH_LPUSH(ctx, stmt);
+		if(URI_UNMASK(uri) != 0) {
+			DBG_DUMP(ctx, stmt, KNH_NULL, "stmt");
+		}
+		if(isEval) {
+			knh_Asm_openDynamicLinkLibrary(ctx, ctx->abr, uri);
+		}
+		res = knh_NameSpace_compile(ctx, ns, stmt, isEval);
+		if(URI_UNMASK(uri) != 0) {
+			knh_NameSpace_loaded(ctx, ns, uri);
+		}
+	}
+	KNH_LOCALBACK(ctx, lsfp);
+	return res;
+
+	/* catch */
+	L_CATCH:;
+	if(isThrowable) {
+		KNH_THROW_AGAIN(ctx, lsfp, 1);
+	}
+	else {
+		KNH_PRINT_STACKTRACE(ctx, lsfp, 1);
+	}
+	return 0;
+}
+
+/* ======================================================================== */
+/* [namespace/package] */
+
+static
+char *knh_cwb_packageScript(Ctx *ctx, knh_cwb_t *cwb, knh_bytes_t rootdir, knh_bytes_t package)
+{
+	knh_cwb_subclear(cwb, 0);
+	knh_cwb_write(ctx, cwb, rootdir);
+	knh_cwb_putc(ctx, cwb, '/');
+	knh_cwb_write(ctx, cwb, package);
+	knh_cwb_putc(ctx, cwb, '/');
+	knh_index_t loc = knh_bytes_index(package, '.');
+	if(loc != -1) {
+		package = knh_bytes_first(package, loc);
+	}
+	knh_cwb_write(ctx, cwb, package);
+	knh_cwb_write(ctx, cwb, STEXT(".k"));
+	knh_cwb_ospath(ctx, cwb);
+	if(knh_cwb_isfile(ctx, cwb)) {
+		return knh_cwb_tochar(ctx, cwb);
+	}
+	return NULL;
+}
+
+/* ------------------------------------------------------------------------ */
+
+static
+char *knh_lookupPackageScript(Ctx *ctx, knh_cwb_t *cwb, knh_bytes_t pkgname)
+{
+	String *spath;
+	char *path = knh_getenv("KONOHA_PACKAGE");
+	if(path != NULL) {
+		path = knh_cwb_packageScript(ctx, cwb, B(path), pkgname);
+		if(path != NULL) return path;
+	}
+	spath = (String*)knh_Context_getProperty(ctx, (Context*)ctx, STEXT("konoha.package.path"));
+	if(IS_bString(spath)) {
+		path = knh_cwb_packageScript(ctx, cwb, knh_String_tobytes(spath), pkgname);
+		if(path != NULL) return path;
+	}
+	spath = (String*)knh_Context_getProperty(ctx, (Context*)ctx, STEXT("user.package.path"));
+	if(IS_bString(spath)){
+		path = knh_cwb_packageScript(ctx, cwb, knh_String_tobytes(spath), pkgname);
+		if(path != NULL) return path;
+	}
+	if(knh_hasScriptFunc(ctx, "getKonohaPackage")) {
+		TODO();
+	}
+	return NULL;
+}
+
+/* ------------------------------------------------------------------------ */
+
+static
+NameSpace *knh_NameSpace_newPackageNULL(Ctx *ctx, knh_bytes_t pkgname)
+{
+	knh_uri_t uri = knh_getResourceId(ctx, pkgname);
+	String *nsname = knh_getResourceName(ctx, uri);
+	knh_cwb_t cwbbuf, *cwb = knh_cwb_open(ctx, &cwbbuf);
+	char *fpath = knh_lookupPackageScript(ctx, cwb, knh_String_tobytes(nsname));
+	NameSpace *ns = NULL;
+	if(fpath != NULL) {
+		pkgname = knh_String_tobytes(nsname);
+		KNH_LOCK(ctx, LOCK_SYSTBL, NULL);
+		ns = (NameSpace*)knh_DictMap_get__b(ctx,  DP(ctx->sys)->NameSpaceTableDictMap, pkgname);
+		if(IS_NULL(ns)) {
+			ns = new_NameSpace(ctx, nsname);
+			knh_DictMap_set(ctx, DP(ctx->sys)->NameSpaceTableDictMap, nsname, UP(ns));
+			KNH_UNLOCK(ctx, LOCK_SYSTBL, NULL);
+			knh_sfp_t *lsfp = KNH_LOCAL(ctx);
+			NameSpace *curns = knh_switchCurrentNameSpace(ctx, ns);
+			InputStream *in = new_ScriptInputStream(ctx, knh_cwb_tobytes(cwb), cwb, ctx->share->mainns, 0);
+			KNH_LPUSH(ctx, in);
+			if(!knh_InputStream_isClosed(ctx, in)) {
+				knh_NameSpace_load(ctx, ns, in, 1/*isEval*/,0/*isThrowable*/);
+			}
+			else {
+				KNH_WARNING(ctx, "package script does not exist");
+				ns = NULL;
+			}
+			knh_switchCurrentNameSpace(ctx, curns);
+			KNH_LOCALBACK(ctx, lsfp);
+		}
+		else {
+			KNH_UNLOCK(ctx, LOCK_SYSTBL, NULL);
+		}
+	}
+	knh_cwb_close(cwb);
+	return ns;
+}
+
+///* ------------------------------------------------------------------------ */
+//
+//Object *konohac_data(Ctx *ctx, InputStream *in, knh_class_t reqc)
+//{
+//	knh_sfp_t *lsfp = KNH_LOCAL(ctx);
+//	KNH_LPUSH(ctx, in);
+//	KNH_LPUSH(ctx, new_ExceptionHandler(ctx));
+//	KNH_TRY(ctx, L_CATCH, lsfp, 1);
+//	{
+//		Stmt *stmt = knh_InputStream_parseStmt(ctx, in, 1/*isData*/);
+//		KNH_LPUSH(ctx, stmt);
+//		return knh_Stmt_toData(ctx, stmt, reqc);
+//	}
+//	/* catch */
+//	L_CATCH:;
+//	KNH_LOCALBACK(ctx, lsfp);
+//	//KNH_PRINT_STACKTRACE(ctx, lsfp, 1);
+//	return KNH_NULL;
+//}
+
+/* ======================================================================== */
+/* [CLASS] */
 
 static
 knh_flag_t knh_StmtCLASS_flag(Ctx *ctx, Stmt *o)
@@ -136,7 +450,7 @@ void knh_class_addInterface(Ctx *ctx, knh_class_t cid, knh_class_t icid)
 				isupcid = ctx->share->ClassTable[isupcid].supcid;
 			}
 			DBG2_P("add interface %s to %s", CLASSN(icid), CLASSN(cid));
-			knh_ClassMap_add(ctx, ctx->share->ClassTable[cid].cmap, new_Mapper__interface(ctx, cid, icid));
+			knh_ClassMap_add(ctx, ClassTable(cid).cmap, new_Mapper__interface(ctx, cid, icid));
 		}
 	}
 }
@@ -148,11 +462,10 @@ int knh_StmtCLASS_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 {
 	char bufn[CLASSNAME_BUFSIZ];
 	knh_snprintf(bufn, sizeof(bufn), "%s.%s", knh_String_tochar(DP(ns)->nsname), sToken(StmtCLASS_class(stmt)));
-
 	knh_class_t cid  = knh_NameSpace_getcid(ctx, ns, B(bufn));
 	if(cid == CLASS_unknown) {
 		cid = knh_ClassTable_newId(ctx);
-		KNH_ASSERT(ctx->share->ClassTable[cid].class_cid == NULL);
+		KNH_ASSERT(ClassTable(cid).class_cid == NULL);
 	}
 	else {
 		knh_Asm_perror(ctx, abr, KERR_ERROR, _("cannot redefine %C"), cid);
@@ -176,7 +489,7 @@ int knh_StmtCLASS_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 
 	DP(StmtCLASS_class(stmt))->cid = cid;
 
-	knh_ClassTable_t *TC = (knh_ClassTable_t*)&(ctx->share->ClassTable[cid]);
+	knh_ClassTable_t *TC = (knh_ClassTable_t*)&(ClassTable(cid));
 	TC->cflag  = knh_StmtCLASS_flag(ctx, stmt);
 	TC->oflag  = KNH_FLAG_CF2OF(TC->cflag);
 	if(SP(StmtCLASS_instmt(stmt))->stt == STT_DONE) {
@@ -188,10 +501,10 @@ int knh_StmtCLASS_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 	TC->supcid = supcid;
 	TC->offset = 0; /* will be extended in CLASS_typing */
 	KNH_ASSERT(TC->class_cid == NULL);
-	konoha_setClassName(ctx, cid, new_String(ctx, B(bufn), NULL));
+	knh_setClassName(ctx, cid, new_String(ctx, B(bufn), NULL));
 	KNH_INITv(TC->cstruct, new_ClassStruct0(ctx, 0, 8));
 	KNH_INITv(TC->cmap, ctx->share->ClassTable[CLASS_Any].cmap);
-	konoha_setClassDefaultValue(ctx, cid, KNH_NULL, NULL);
+	knh_setClassDefaultValue(ctx, cid, KNH_NULL, NULL);
 	knh_NameSpace_setLocalName(ctx, ns, cid);
 
 	if(SP(StmtCLASS_interface(stmt))->stt != STT_DONE) {
@@ -205,7 +518,7 @@ int knh_StmtCLASS_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 				continue;
 			}
 			if(!knh_class_isInterface(icid)) {
-				knh_Token_perror(ctx, tk, KERR_EWARN, _("cannot implements %C: this class is not @Interfac"), icid);
+				knh_Token_perror(ctx, tk, KERR_EWARN, _("cannot implements %C: this class is not @Interface"), icid);
 				continue;
 			}
 			knh_class_addInterface(ctx, cid, icid);
@@ -218,53 +531,35 @@ int knh_StmtCLASS_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 	return 1;
 }
 
-/* ------------------------------------------------------------------------ */
-/* [declc] */
-
-static
-char *knh_lookup_includeScript(Ctx *ctx, knh_bytes_t path, char *buf, size_t bufsiz)
-{
-	knh_index_t loc = knh_bytes_rindex(path, '/');
-	if(loc == -1) loc = knh_bytes_rindex(path, '\\');
-	char *filename = (char*)path.buf + loc + 1;
-
-#ifdef KNH_PREFIX
-	knh_snprintf(buf, bufsiz, "%s/lib/konoha/include/%s", KNH_PREFIX, filename);
-#else
-	knh_snprintf(buf, bufsiz, "%s/include/%s", knh_String_tochar(DP(ctx->sys)->homeDir), filename);
-#endif
-	if(knh_isfile(ctx, B(buf))) return buf;
-	char *p = knh_getenv("HOME");
-	if(p != NULL) {
-		knh_snprintf(buf, bufsiz, "%s/.konoha/include/%s", p, filename);
-		if(knh_isfile(ctx, B(buf))) return buf;
-	}
-	return NULL;
-}
 
 /* ------------------------------------------------------------------------ */
 
 static
 int knh_StmtIMPORT_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 {
-	String *fname = (String*)DP(StmtIMPORT_file(stmt))->data;
-	char bufp[FILENAME_BUFSIZ], buff[FILENAME_BUFSIZ];
-
-	String *cfname = konoha_getFileName(ctx, SP(stmt)->fileid);
-	KNH_ASSERT(IS_String(fname));
-	if(knh_String_endsWith(cfname, STEXT(".k"))) {
-		knh_format_parentpath(bufp, sizeof(bufp), knh_String_tobytes(cfname), 1);
+	int res = 1;
+	knh_bytes_t path = knh_String_tobytes(DP(StmtIMPORT_file(stmt))->text);
+	knh_cwb_t cwbbuf, *cwb = knh_cwb_open(ctx, &cwbbuf);
+	if(path.buf[0] != '/' && path.buf[0] != '\\' && !knh_bytes_startsWith(path, STEXT("http://"))) {
+		knh_cwb_write(ctx, cwb, knh_String_tobytes(knh_getResourceName(ctx, SP(stmt)->uri)));
+		knh_cwb_parentpath(ctx, cwb, NULL);
+		knh_cwb_putc(ctx, cwb, '/');
 	}
-	else {
-		bufp[0]=0;
+	knh_cwb_write(ctx, cwb, path);
+	{
+		knh_sfp_t *lsfp = KNH_LOCAL(ctx);
+		InputStream *in = new_ScriptInputStream(ctx, path, cwb, ctx->share->mainns, 0);
+		KNH_LPUSH(ctx, in);
+		if(!knh_InputStream_isClosed(ctx, in)) {
+			res = knh_NameSpace_load(ctx, ctx->share->mainns, in, 1/*isEval*/,0/*isThrowable*/);
+		}
+		else {
+			res = 0;
+		}
+		KNH_LOCALBACK(ctx, lsfp);
 	}
-	knh_format_catpath(buff, sizeof(buff), B(bufp), knh_String_tobytes(fname));
-	if(!knh_isfile(ctx, B(buff))) {
-		knh_lookup_includeScript(ctx, knh_String_tobytes(fname), buff, sizeof(buff));
-	}
-	KNH_NOTICE(ctx, "including script: %s", buff);
-	knh_NameSpace_loadScript(ctx, ns, B(buff), 1 /* isEval */);
-	return 1;
+	knh_cwb_close(cwb);
+	return res;
 }
 
 /* ------------------------------------------------------------------------ */
@@ -276,8 +571,8 @@ int knh_StmtUIMPORT_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 	knh_bytes_t name = knh_Token_tobytes(ctx, tk);
 	knh_index_t loc = knh_bytes_rindex(name, '.');
 	if(loc != -1 && isupper(name.buf[loc+1])) { /* using math.Math */
-		if(knh_System_loadPackage(ctx, knh_bytes_first(name, loc))) {
-			knh_class_t newcid = konoha_getcid(ctx, name);
+		if(knh_NameSpace_newPackageNULL(ctx, knh_bytes_first(name, loc))) {
+			knh_class_t newcid = knh_getcid(ctx, name);
 			if(newcid == CLASS_unknown) {
 				knh_Token_perror(ctx, tk, KERR_ERROR, _("unknown class: %s"), sToken(tk));
 				return 0;
@@ -290,7 +585,7 @@ int knh_StmtUIMPORT_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 			return 1;
 		}
 	}else {
-		if(knh_System_loadPackage(ctx, name)) {
+		if(knh_NameSpace_newPackageNULL(ctx, name)) {
 			if(knh_Token_isTailWildCard(tk)) {
 				knh_NameSpace_importClass(ctx, ns, name);
 			}
@@ -347,9 +642,9 @@ int knh_StmtXCLASS_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns, knh_class
 	Token *tkurn = DP(stmt)->tokens[1];
 
 	char bufcu[CLASSNAME_BUFSIZ];
-	knh_format_classurn(ctx, bufcu, sizeof(bufcu), bcid, konoha_getURNAlias(ctx, knh_Token_tobytes(ctx, tkurn)));
+	knh_format_classurn(ctx, bufcu, sizeof(bufcu), bcid, knh_getURNAlias(ctx, knh_Token_tobytes(ctx, tkurn)));
 
-	knh_class_t cid = konoha_findcid(ctx, B(bufcu));
+	knh_class_t cid = knh_findcid(ctx, B(bufcu));
 	if(cid == bcid || cid == CLASS_unknown) {
 		knh_Token_perror(ctx, tkurn, KERR_EWARN, _("resource not found: %s"), sToken(tkurn));
 		cid = bcid;
@@ -403,7 +698,7 @@ int knh_StmtUFUNC_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 			knh_Token_perror(ctx, tk, KERR_ERROR, _("unknown class: %s"), sToken(tk));
 			return 0;
 		}
-		knh_methodn_t mn = konoha_getMethodName(ctx, knh_bytes_last(name, loc+1), METHODN_NONAME);
+		knh_methodn_t mn = knh_getmn(ctx, knh_bytes_last(name, loc+1), METHODN_NONAME);
 		if(mn == METHODN_NONAME) {
 			knh_Token_perror(ctx, tk, KERR_ERROR, _("unknown method: %C.%s"), cid, sToken(tk));
 			return 0;
@@ -418,7 +713,7 @@ int knh_StmtUFUNC_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 		}
 		else {
 			KNH_ASSERT_cid(cid);
-			Array *a = (ctx->share->ClassTable[cid].cstruct)->methods;
+			Array *a = (ClassTable(cid).cstruct)->methods;
 			size_t i;
 			for(i = 0; i < knh_Array_size(a); i++) {
 				Method *mtd = (Method*)knh_Array_n(a, i);
@@ -449,7 +744,7 @@ int knh_StmtUMAPMAP_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 static
 int knh_Stmt_eval(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns, int isEval)
 {
-	Script *scr = knh_Asm_getScript(ctx, abr);
+	Script *scr = knh_getCurrentScript(ctx);
 	Method *mtd = knh_Class_getMethod(ctx, knh_Object_cid(scr), METHODN_lambda);
 	knh_sfp_t *lsfp = KNH_LOCAL(ctx);
 
@@ -458,7 +753,7 @@ int knh_Stmt_eval(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns, int isEval)
 	knh_methodn_t mt = METHODN__k;
 	if(SP(stmt)->stt == STT_MT) {
 		Token *tk0 = DP(stmt)->tokens[0];
-		knh_methodn_t mn = konoha_getMethodName(ctx, knh_Token_tobytes(ctx, tk0), METHODN_NEWID);
+		knh_methodn_t mn = knh_getmn(ctx, knh_Token_tobytes(ctx, tk0), METHODN_NEWID);
 		char *name = FIELDN(METHODN_TOFIELDN(mn));
 		if(name[1] != 0) mt = METHODN__s;
 	}
@@ -524,15 +819,15 @@ int knh_Stmt_eval(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns, int isEval)
 
 /* ------------------------------------------------------------------------ */
 
-int knh_Stmt_compile(Ctx *ctx, Stmt *stmt, String *nsname, int isEval)
+static
+int knh_NameSpace_compile(Ctx *ctx, NameSpace *ns, Stmt *stmt, int isEval)
 {
 	Asm *abr = knh_Context_getAsm(ctx);
-	NameSpace *ns = knh_Context_setNameSpace(ctx, nsname);
 	knh_sfp_t *lsfp = KNH_LOCAL(ctx);
 	KNH_LPUSH(ctx, stmt);
 
 	Stmt *cur = stmt;
-	DP(abr)->fileid = SP(stmt)->fileid;
+	DP(abr)->uri = SP(stmt)->uri;
 	while(IS_Stmt(cur)) {
 		knh_stmt_t stt = SP(cur)->stt;
 		DP(abr)->line = SP(cur)->line;
@@ -655,160 +950,27 @@ int knh_Stmt_compile(Ctx *ctx, Stmt *stmt, String *nsname, int isEval)
 	return 0;
 }
 
-/* ======================================================================== */
-/* [main] */
-
-static
-Stmt *knh_InputStream_parseStmt(Ctx *ctx, InputStream *in)
-{
-	Token *tk = new_Token(ctx, 0, DP(in)->fileid, 0, knh_char_totoken('{'));
-	KNH_LPUSH(ctx, tk);
-	KNH_LPUSH(ctx, in);
-	knh_Token_parse(ctx, tk, in);
-	DBG2_DUMP(ctx, tk, KNH_NULL, "tokens");
-	Stmt *stmt = new_StmtINSTMT(ctx, tk); // new_Stmt(ctx, 0, STT_DONE)(ctx);
-	return stmt;
-}
-
 /* ------------------------------------------------------------------------ */
 
-Stmt *knh_bytes_parseStmt(Ctx *ctx, knh_bytes_t kscript, int fileid, int line)
+Object *knh_Stmt_toData(Ctx *ctx, Stmt *stmt, knh_class_t reqc)
 {
-	knh_cwb_t cwb = new_cwb(ctx);
-	knh_Bytes_write(ctx, cwb.ba, kscript);
-	knh_Bytes_putc(ctx, cwb.ba, ';');
-	InputStream *in = new_BytesInputStream(ctx, cwb.ba, cwb.pos, knh_Bytes_size(cwb.ba));
-	DP(in)->fileid = (knh_fileid_t)fileid;
-	DP(in)->line = line;
-	Stmt *stmt = knh_InputStream_parseStmt(ctx, in);
-	knh_cwb_clear(cwb);
-	return stmt;
+//	Asm *abr = knh_Context_getAsm(ctx);
+//	DP(abr)->level = 0;
+//	NameSpace *ns = ctx->ns;
+//	if(IS_Stmt(stmt)) {
+//		KNH_ASM_SETLINE(ctx, abr, SP(stmt)->line);
+//		if(knh_stmt_isExpr(SP(stmt)->stt)) {
+//			Token *tk = knh_StmtEXPR_typing(ctx, stmt, abr, ns, reqc);
+//			if(tk == NULL || IS_Stmt(tk)) return KNH_NULL;
+//		}
+//		else {
+//			knh_Stmt_done(ctx, stmt);
+//		}
+//	}
+	TODO();
+	return KNH_NULL;
 }
 
-/* ------------------------------------------------------------------------ */
-
-void konohac_eval(Ctx *ctx, String *nsname, InputStream *in)
-{
-	knh_sfp_t *lsfp = KNH_LOCAL(ctx);
-	KNH_LPUSH(ctx, new_ExceptionHandler(ctx));
-	KNH_TRY(ctx, L_CATCH, lsfp, 0);
-	{
-		Stmt *stmt = knh_InputStream_parseStmt(ctx, in);
-		KNH_LPUSH(ctx, stmt);
-		DBG_DUMP(ctx, stmt, KNH_NULL, "stmt");
-		knh_Stmt_compile(ctx, stmt, nsname, !knh_Context_isCompiling(ctx) /* isrun 1 */);
-	}
-	KNH_LOCALBACK(ctx, lsfp);
-	return;
-
-	/* catch */
-	L_CATCH:;
-	KNH_PRINT_STACKTRACE(ctx, lsfp, 0);
-	return;
-}
-
-/* ------------------------------------------------------------------------ */
-
-void knh_Asm_openlib(Ctx *ctx, Asm *abr, knh_bytes_t fpath)
-{
-	knh_cwb_t cwb = new_cwb(ctx);
-	knh_index_t idx = knh_bytes_rindex(fpath, '.');
-	if(idx > 0) fpath = knh_bytes_first(fpath, idx);
-	knh_Bytes_write(ctx, cwb.ba, fpath);
-	knh_Bytes_putc(ctx, cwb.ba, '_');
-	knh_Bytes_write(ctx, cwb.ba, STEXT(KONOHA_PLATFORM));
-	knh_Bytes_write(ctx, cwb.ba, STEXT(KONOHA_OS_DLLEXT));
-	knh_Bytes_putc(ctx, cwb.ba, 0);
-
-	if(DP(abr)->dlhdr != NULL) {
-		DP(abr)->dlhdr = NULL;
-	}
-
-	knh_bytes_t f = knh_cwb_tobytes(cwb);
-	DP(abr)->dlhdr =knh_dlopen(ctx, (char*)f.buf, KNH_RTLD_LAZY);
-	if(DP(abr)->dlhdr != NULL) {
-		KNH_NOTICE(ctx, "opened dynamic library: %s", f.buf);
-	}
-	else {
-		if(knh_isfile(ctx, f)) {
-			KNH_WARNING(ctx, "cannot open dynamic library: %s", f.buf);
-		}
-	}
-	knh_cwb_clear(cwb);
-}
-
-/* ======================================================================== */
-/* [loaded] */
-
-static
-void knh_NameSpace_loaded(Ctx *ctx, NameSpace *o, knh_fileid_t fileid)
-{
-	KNH_ASSERT(IS_NameSpace(o));
-	if(IS_NULL(DP(o)->lconstDictMap)) {
-		KNH_SETv(ctx, DP(o)->lconstDictMap, new_DictMap0(ctx, 16));
-	}
-	char buf[40];
-	knh_snprintf(buf, sizeof(buf), "_LOADED_%d", (int)fileid);
-	knh_DictMap_set(ctx, DP(o)->lconstDictMap, new_String(ctx, B(buf), NULL), UP(konoha_getFileName(ctx, fileid)));
-}
-
-/* ------------------------------------------------------------------------ */
-
-static
-int knh_NameSpace_isLoaded(Ctx *ctx, NameSpace *o, knh_fileid_t fileid)
-{
-	if(IS_NOTNULL(DP(o)->lconstDictMap)) {
-		char buf[40];
-		knh_snprintf(buf, sizeof(buf), "_LOADED_%d", (int)fileid);
-		return (knh_DictMap_index__b(DP(o)->lconstDictMap, B(buf)) != -1);
-	}
-	return 0;
-}
-
-/* ------------------------------------------------------------------------ */
-
-int knh_NameSpace_loadScript(Ctx *ctx, NameSpace *ns, knh_bytes_t fpath, int isEval)
-{
-	knh_fileid_t fileid = konoha_getFileId(ctx, fpath);
-	if(knh_NameSpace_isLoaded(ctx, ns, fileid)) {
-		DBG2_P("Already imported: %s", fpath.buf);
-		return 1;
-	}
-	knh_NameSpace_loaded(ctx, ns, fileid);
-	knh_sfp_t *lsfp = KNH_LOCAL(ctx);
-	InputStream *in = new_FileInputStream(ctx, fpath);
-	KNH_LPUSH(ctx, in);
-	DP(in)->fileid = fileid;
-	knh_InputStream_setEncoding(ctx, in, knh_systemEncoding);
-	Stmt *stmt = knh_InputStream_parseStmt(ctx, in);
-	KNH_LPUSH(ctx, stmt);
-	if(isEval) {
-		knh_Asm_openlib(ctx, knh_Context_getAsm(ctx), fpath);
-	}
-	knh_Stmt_compile(ctx, stmt, DP(ns)->nsname, isEval);
-	KNH_LOCALBACK(ctx, lsfp);
-	return 1;
-}
-
-/* ------------------------------------------------------------------------ */
-
-void konoha_compile(Ctx *ctx, String *nsname, knh_bytes_t fpath)
-{
-	knh_sfp_t *lsfp = KNH_LOCAL(ctx);
-	KNH_LPUSH(ctx, new_ExceptionHandler(ctx));
-	KNH_TRY(ctx, L_CATCH, lsfp, 0);
-	{
-		NameSpace *ns = knh_Context_setNameSpace(ctx, nsname);
-		knh_NameSpace_loadScript(ctx, ns, fpath, 0);
-	}
-	KNH_LOCALBACK(ctx, lsfp);
-	return;
-
-	/* catch */
-	L_CATCH:;
-	KNH_PRINT_STACKTRACE(ctx, lsfp, 0);
-	return;
-}
 
 /* ------------------------------------------------------------------------ */
 
