@@ -1,7 +1,7 @@
 /****************************************************************************
  * KONOHA COPYRIGHT, LICENSE NOTICE, AND DISCRIMER
  *
- * Copyright (c) 2005-2008, Kimio Kuramitsu <kimio at ynu.ac.jp>
+ * Copyright (c) 2005-2009, Kimio Kuramitsu <kimio at ynu.ac.jp>
  *           (c) 2008-      Konoha Software Foundation
  * All rights reserved.
  *
@@ -41,7 +41,7 @@ extern "C" {
 Stmt* new_Stmt(Ctx *ctx, knh_flag_t flag, knh_stmt_t stt)
 {
 	knh_Stmt_t *o = (Stmt*)new_Object_bcid(ctx, CLASS_Stmt, knh_stmt_size(stt));
-	SP(o)->fileid = 0;
+	SP(o)->uri = 0;
 	SP(o)->line = 0;
 	SP(o)->flag = flag;
 	SP(o)->stt  = stt;
@@ -55,19 +55,19 @@ void knh_Stmt_toERR(Ctx *ctx, Stmt *stmt, Term *tm)
 {
 	if(SP(stmt)->stt == STT_ERR) return;
 	SP(stmt)->stt = STT_ERR;
-	knh_fileid_t fileid = 0;
+	knh_uri_t uri = 0;
 	int line = 0;
 	if(IS_Token(tm)) {
-		fileid =  SP((Token*)tm)->fileid;
+		uri =  SP((Token*)tm)->uri;
 		line =  SP((Token*)tm)->line;
 	}
 	else if(IS_Stmt(tm)) {
-		fileid =  SP((Stmt*)tm)->fileid;
+		uri =  SP((Stmt*)tm)->uri;
 		line =  SP((Stmt*)tm)->line;
 	}
 	{
 		char buf[256];
-		knh_snprintf(buf, sizeof(buf), "Script!!: running errors at %s:%d", FILEIDN(SP(stmt)->fileid), SP(stmt)->line);
+		knh_snprintf(buf, sizeof(buf), "Script!!: running errors at %s:%d", URIDN(SP(stmt)->uri), SP(stmt)->line);
 		KNH_SETv(ctx, DP(stmt)->errMsg, new_String(ctx, B(buf), NULL));
 		KNH_SETv(ctx, DP(stmt)->next, KNH_NULL);
 	}
@@ -107,7 +107,7 @@ void knh_Stmt_add(Ctx *ctx, Stmt *o, Term *tm)
 		DP(o)->line_end = SP(stmt)->line;
 		if(SP(o)->line == 0) {
 			SP(o)->line = DP(stmt)->line_end;
-			SP(o)->fileid = SP(stmt)->fileid;
+			SP(o)->uri = SP(stmt)->uri;
 		}
 		if(SP(stmt)->stt == STT_ERR) {
 			knh_Stmt_toERR(ctx, o, tm);
@@ -118,7 +118,7 @@ void knh_Stmt_add(Ctx *ctx, Stmt *o, Term *tm)
 		DP(o)->line_end = SP(tk)->line;
 		if(SP(o)->line == 0) {
 			SP(o)->line = DP(o)->line_end;
-			SP(o)->fileid = SP(tk)->fileid;
+			SP(o)->uri = SP(tk)->uri;
 		}
 		if(SP(tk)->tt == TT_ERR) {
 			knh_Stmt_toERR(ctx, o, tm);
@@ -129,7 +129,7 @@ void knh_Stmt_add(Ctx *ctx, Stmt *o, Term *tm)
 /* ======================================================================== */
 /* [visit] */
 
-Stmt *knh_Stmt_tail(Stmt *o)
+Stmt *knh_Stmt_tail(Ctx *ctx, Stmt *o)
 {
 	Stmt *tail = o;
 	while(IS_NOTNULL(DP(tail)->next)) {
@@ -147,7 +147,7 @@ Stmt* knh_StmtNULL_tail_append(Ctx *ctx, Stmt *o, Stmt *stmt)
 		return stmt;
 	}
 	else {
-		Stmt *tail = knh_Stmt_tail(o);
+		Stmt *tail = knh_Stmt_tail(ctx, o);
 		KNH_SETv(ctx, DP(tail)->next, stmt);
 		return o;
 	}
@@ -171,6 +171,10 @@ knh_flag_t knh_StmtMETHOD_flag(Ctx *ctx, Stmt *o)
 		if(IS_NOTNULL(v)) {
 			flag |= KNH_FLAG_MF_VIRTUAL;
 		}
+		v = knh_DictMap_get__b(ctx, DP(o)->metaDictMap, STEXT("Abstract"));
+		if(IS_NOTNULL(v)) {
+			flag |= KNH_FLAG_MF_VIRTUAL;
+		}
 		v = knh_DictMap_get__b(ctx, DP(o)->metaDictMap, STEXT("Private"));
 		if(IS_NOTNULL(v)) {
 			flag |= KNH_FLAG_MF_PRIVATE;
@@ -242,7 +246,7 @@ void knh_Stmt__s(Ctx *ctx, Stmt *o, OutputStream *w, String *m)
 	knh_intptr_t i;
 	knh_putc(ctx, w, '(');
 	if(SP(o)->stt != STT_OP && SP(o)->stt != STT_NEW && SP(o)->stt != STT_CALL ) {
-		knh_write__s(ctx, w, knh_stmt_tochar(SP(o)->stt));
+		knh_write_char(ctx, w, knh_stmt_tochar(SP(o)->stt));
 		if(DP(o)->size > 0) {
 			knh_putc(ctx, w, ' ');
 		}
