@@ -43,21 +43,21 @@ extern "C" {
 /* ======================================================================== */
 /* [ctxkey] */
 
+static int isInit = 0;
 
-#ifdef KNH_USING_THREAD
-static knh_thread_key_t ctxkey;
+#ifdef KNH_USING_NOTHREAD
 static Ctx *curctx = NULL;
 #else
-static Ctx *curctx = NULL;
+static knh_thread_key_t ctxkey;
 #endif
 
 /* ------------------------------------------------------------------------ */
 
 void konoha_init(void)
 {
-	static int isInit = 0;
-#ifdef KNH_USING_THREAD
+#ifndef KNH_USING_NOTHREAD
 	if(isInit == 0) {
+		isInit = 1;
 		knh_thread_key_create(&ctxkey);
 	}
 #endif
@@ -70,11 +70,12 @@ void konoha_init(void)
 
 KNHAPI(Ctx*) knh_beginContext(Ctx *ctx)
 {
-#ifdef KNH_USING_THREAD
+#ifdef KNH_USING_NOTHREAD
+	curctx = ctx;
+#else
 	knh_mutex_lock(ctx->ctxlock);
 	knh_thread_setspecific(ctxkey, ctx);
 #endif
-	curctx = ctx;
 	return ctx;
 }
 
@@ -82,11 +83,12 @@ KNHAPI(Ctx*) knh_beginContext(Ctx *ctx)
 
 KNHAPI(void) knh_endContext(Ctx *ctx)
 {
-#ifdef KNH_USING_THREAD
+#ifdef KNH_USING_NOTHREAD
+	curctx = NULL;
+#else
 	knh_thread_setspecific(ctxkey, NULL);
 	knh_mutex_unlock(ctx->ctxlock);
 #endif
-	curctx = NULL;
 }
 
 /* ------------------------------------------------------------------------ */
@@ -94,13 +96,10 @@ KNHAPI(void) knh_endContext(Ctx *ctx)
 KNHAPI(Ctx*) knh_getCurrentContext(void)
 {
 	Ctx *ctx;
-#ifdef KNH_USING_THREAD
-	ctx = (Ctx*)knh_thread_getspecific(ctxkey);
-	if (ctx == NULL) {
-		ctx = (Ctx *) curctx;
-	}
-#else
+#ifdef KNH_USING_NOTHREAD
 	ctx = curctx;
+#else
+	ctx = (Ctx*)knh_thread_getspecific(ctxkey);
 #endif
 	if(ctx == NULL) {
 		fprintf(stderr, "NOT IN THE CONTEXT OF KONOHA\n");
@@ -115,7 +114,6 @@ KNHAPI(Ctx*) knh_getCurrentContext(void)
 static int userExperienceProgram = 0;
 static int dumpLevel2 = 0;    // this is -d2 debug mode for konoha itself
 static int debugLevel = 0;
-static int toInteractiveMode = 0;
 
 /* ----------------------------------------------------------------------- */
 
@@ -133,13 +131,6 @@ KNHAPI(int) konoha_isSystemDump2(void)
 
 /* ----------------------------------------------------------------------- */
 
-KNHAPI(int) knh_isToInteractiveMode(void)
-{
-	return toInteractiveMode;
-}
-
-/* ----------------------------------------------------------------------- */
-
 int knh_isUserExperienceProgram(Ctx *ctx)
 {
 	return (knh_Context_isInteractive(ctx) && userExperienceProgram == 1);
@@ -186,9 +177,6 @@ KNHAPI(int) konoha_parseopt(konoha_t konoha, int argc, char **argv)
 		else if(t[1] == 'c' && t[2] == 0) {
 			knh_Context_setCompiling(konoha.ctx, 1);
 		}
-		else if(t[1] == 'i' && t[2] == 0) {
-			toInteractiveMode = 1;
-		}
 		else if(t[1] == 'v') {
 			knh_Context_setVerbose(konoha.ctx, 1);
 			if(t[2] == '2') {
@@ -233,10 +221,10 @@ KNHAPI(int) konoha_hasRuntimeError(konoha_t konoha)
 
 /* ------------------------------------------------------------------------ */
 
-void knh_setRuntimeError(Ctx *ctx, knh_String_t *msg)
+void knh_setRuntimeError(Ctx *ctx, String *msg)
 {
-	KNH_SETv(ctx, ((knh_Context_t*)ctx)->msgError, msg);
-	((knh_Context_t*)ctx)->hasError = 1;
+	KNH_SETv(ctx, ((Context*)ctx)->msgError, msg);
+	((Context*)ctx)->hasError = 1;
 }
 
 /* ------------------------------------------------------------------------ */
@@ -258,16 +246,16 @@ KNHAPI(void) konoha_setOutputStreamBuffer(konoha_t konoha, size_t osize, size_t
 	KONOHA_CHECK_(konoha);
 	Ctx *ctx = knh_beginContext(konoha.ctx);
 	if(osize > 0) {
-		KNH_SETv(ctx, ((knh_Context_t*)ctx)->out, new_BytesOutputStream(ctx, new_Bytes(ctx, osize)));
+		KNH_SETv(ctx, ((Context*)ctx)->out, new_BytesOutputStream(ctx, new_Bytes(ctx, osize)));
 	}
 	else {
-		KNH_SETv(ctx, ((knh_Context_t*)ctx)->out, KNH_DEF(ctx, CLASS_OutputStream));
+		KNH_SETv(ctx, ((Context*)ctx)->out, KNH_DEF(ctx, CLASS_OutputStream));
 	}
 	if(esize > 0) {
-		KNH_SETv(ctx, ((knh_Context_t*)ctx)->err, new_BytesOutputStream(ctx, new_Bytes(ctx, esize)));
+		KNH_SETv(ctx, ((Context*)ctx)->err, new_BytesOutputStream(ctx, new_Bytes(ctx, esize)));
 	}
 	else {
-		KNH_SETv(ctx, ((knh_Context_t*)ctx)->err, KNH_DEF(ctx, CLASS_OutputStream));
+		KNH_SETv(ctx, ((Context*)ctx)->err, KNH_DEF(ctx, CLASS_OutputStream));
 	}
 	knh_endContext(ctx);
 }
@@ -278,10 +266,10 @@ KNHAPI(char*) konoha_getStdOutBufferText(konoha_t konoha)
 {
 	KONOHA_CHECK(konoha, "");
 	Ctx *ctx = knh_beginContext(konoha.ctx);
-	knh_OutputStream_t *out = ((knh_Context_t*)ctx)->out;
+	OutputStream *out = ((Context*)ctx)->out;
 	knh_OutputStream_flush(ctx, out);
 	if(IS_Bytes(DP(out)->ba)) {
-		knh_Bytes_t *ba = DP(out)->ba;
+		Bytes *ba = DP(out)->ba;
 		DBG_P("malloc(%zd)", ba->size + 1);
 		char *p = (char*)malloc(ba->size + 1);
 		if(p != NULL) {
@@ -296,14 +284,14 @@ KNHAPI(char*) konoha_getStdOutBufferText(konoha_t konoha)
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(char*) konoha_getStdErrBufferText(konoha_t konoha)
+KNHAPI(char*) knh_getStdErrBufferText(konoha_t konoha)
 {
 	KONOHA_CHECK(konoha, "");
 	Ctx *ctx = knh_beginContext(konoha.ctx);
-	knh_OutputStream_t *out = ((knh_Context_t*)ctx)->err;
+	OutputStream *out = ((Context*)ctx)->err;
 	knh_OutputStream_flush(ctx, out);
 	if(IS_Bytes(DP(out)->ba)) {
-		knh_Bytes_t *ba = DP(out)->ba;
+		Bytes *ba = DP(out)->ba;
 		DBG_P("malloc(%zd)", ba->size + 1);
 		char *p = (char*)malloc(ba->size + 1);
 		if(p != NULL) {
@@ -327,7 +315,7 @@ KNHAPI(void) konoha_evalScript(konoha_t konoha, char *script)
 	knh_Bytes_write(ctx, cwb->ba, B(script));
 	knh_Bytes_putc(ctx, cwb->ba, '\n');
 	{
-		knh_InputStream_t *in = new_BytesInputStream(ctx, cwb->ba, cwb->pos, knh_Bytes_size(cwb->ba));
+		InputStream *in = new_BytesInputStream(ctx, cwb->ba, cwb->pos, knh_Bytes_size(cwb->ba));
 		DP(in)->uri = URI_EVAL;
 		DP(in)->line = 0;
 		knh_NameSpace_load(ctx, ctx->share->mainns, in, 1/*isEval*/,0/*isThrowable*/);
@@ -341,18 +329,17 @@ KNHAPI(void) konoha_evalScript(konoha_t konoha, char *script)
 
 KNHAPI(int) konoha_loadScript(konoha_t konoha, char *fpath)
 {
-	KONOHA_CHECK(konoha, 0);
+	KONOHA_CHECK_(konoha);
 	int res = 0;
 	Ctx *ctx = knh_beginContext(konoha.ctx);
 	knh_sfp_t *lsfp = KNH_LOCAL(ctx);
-	knh_InputStream_t *in = new_ScriptInputStream(ctx, B(fpath), NULL, ctx->share->mainns, 0);
+	InputStream *in = new_ScriptInputStream(ctx, B(fpath), NULL, ctx->share->mainns, 0);
 	KNH_LPUSH(ctx, in);
 	if(!knh_InputStream_isClosed(ctx, in)) {
-		int isEval = (knh_Context_isCompiling(ctx)) ? 0 : 1;
-		knh_NameSpace_load(ctx, ctx->share->mainns, in, isEval, 0/*isThrowable*/);
+		knh_NameSpace_load(ctx, ctx->share->mainns, in, 1/*isEval*/,0/*isThrowable*/);
 	}
 	else {
-		knh_setRuntimeError(ctx, T__("script file doesn't exists"));
+		knh_setRuntimeError(ctx, T("script file doesn't exists"));
 		res = -1;
 	}
 	KNH_LOCALBACK(ctx, lsfp);
@@ -363,64 +350,53 @@ KNHAPI(int) konoha_loadScript(konoha_t konoha, char *fpath)
 
 /* ------------------------------------------------------------------------ */
 
-void knh_setArgv(Ctx* ctx, int argc, char** argv)
-{
-    knh_Array_t *a = new_Array(ctx, CLASS_String, argc);
-    int i;
-    for(i = 0; i < argc; i++) {
-        knh_Array_add(ctx, a, UP(T__(argv[i])));
-    }
-    knh_DictMap_set(ctx, DP(ctx->sys)->props,T__("argv"), UP(a));
-}
-
-/* ------------------------------------------------------------------------ */
-
 KNHAPI(int) konoha_runMain(konoha_t konoha, int argc, char **argv)
 {
-    KONOHA_CHECK(konoha, -1);
-    Ctx *ctx = knh_beginContext(konoha.ctx);
-    knh_sfp_t *lsfp = KNH_LOCAL(ctx);
-    KNH_MOV(ctx, lsfp[0].o, new_ExceptionHandler(ctx));
-    KNH_TRY(ctx, L_CATCH, lsfp, 0);
-    {
-        knh_NameSpace_t *ns = ctx->share->mainns;
-        knh_Script_t *scr = knh_NameSpace_getScript(ctx, ns);
-        knh_Method_t *mtd = knh_Class_getMethod(ctx, knh_Object_cid(scr), METHODN_main);
+	KONOHA_CHECK(konoha, -1);
+	Ctx *ctx = knh_beginContext(konoha.ctx);
+	knh_sfp_t *lsfp = KNH_LOCAL(ctx);
+	KNH_MOV(ctx, lsfp[0].o, new_ExceptionHandler(ctx));
+	KNH_TRY(ctx, L_CATCH, lsfp, 0);
+	{
+		NameSpace *ns = ctx->share->mainns;
+		Script *scr = knh_NameSpace_getScript(ctx, ns);
+		Method *mtd = knh_Class_getMethod(ctx, knh_Object_cid(scr), METHODN_main);
+		if(IS_NULL(mtd)) {
+			goto L_END_TRY;
+		}
+		if(knh_Method_psize(mtd) == 1) {
+			knh_type_t ptype = knh_Method_pztype(mtd, 0);
+			if(ptype != ATYPE_String && ptype != NNATYPE_String) {
+				knh_setRuntimeError(ctx, T("Type!!: main()"));
+				goto L_END_TRY;
+			}
+			else {
+				Array *a = new_Array(ctx, CLASS_String, argc);
+				int i;
+				for(i = 0; i < argc; i++) {
+					knh_Array_add(ctx, a, UP(T(argv[i])));
+				}
+				//KNH_MOV(ctx, lsfp[1].o, mtd);
+				KNH_MOV(ctx, lsfp[2].o, scr);
+				KNH_MOV(ctx, lsfp[3].o, a);
+				KNH_SCALL(ctx, lsfp, 1, mtd, /* args*/ 1);
+			}
+		}
+		else {
+			knh_setRuntimeError(ctx, T("Type!!: main()"));
+			goto L_END_TRY;
+		}
+	}
+	L_END_TRY:
+	knh_Context_clearstack(ctx);
+	knh_endContext(ctx);
+	return (ctx->hasError == 0 ? -1 : 0);
 
-        knh_setArgv(ctx, argc, argv);
-
-        if(IS_NULL(mtd)) {
-            goto L_END_TRY;
-        }
-        if(knh_Method_psize(mtd) == 1) {
-            knh_type_t ptype = knh_Method_pztype(mtd, 0);
-            if(CLASS_type(ptype) != CLASS_String_Ary) {
-                knh_setRuntimeError(ctx, T__("Type!!: main()"));
-                goto L_END_TRY;
-            }
-            else {
-                //KNH_MOV(ctx, lsfp[1].o, mtd);
-                knh_Array_t *a = (knh_Array_t*)knh_getProperty(ctx, B("argv"));
-                KNH_MOV(ctx, lsfp[2].o, scr);
-                KNH_MOV(ctx, lsfp[3].o, a);
-                KNH_SCALL(ctx, lsfp, 1, mtd, /* args*/ 1);
-            }
-        }
-        else {
-            knh_setRuntimeError(ctx, T__("Type!!: main()"));
-            goto L_END_TRY;
-        }
-    }
-L_END_TRY:
-    knh_Context_clearstack(ctx);
-    knh_endContext(ctx);
-    return (ctx->hasError == 0 ? 0 : 1);
-
-    /* catch */
-L_CATCH:;
-        KNH_PRINT_STACKTRACE(ctx, lsfp, 0);
-        knh_endContext(ctx);
-        return (ctx->hasError == 0 ? 0 : 1);
+	/* catch */
+	L_CATCH:;
+	KNH_PRINT_STACKTRACE(ctx, lsfp, 0);
+	knh_endContext(ctx);
+	return (ctx->hasError == 0 ? -1 : 0);
 }
 
 
@@ -506,10 +482,10 @@ KNHAPI(char*) konoha_invokeStringFunc(konoha_t konoha, char *fmt, ...)
 	lsfp = knh_invokeScriptFunc(ctx, fmt, args);
 	va_end(args);
 	if(ctx->hasError == 0 && IS_String(lsfp[0].o)) {
-		knh_String_t *s = lsfp[0].s;
+		String *s = lsfp[0].s;
 		char *p = (char*)malloc(s->size+1);
 		if(p == NULL) {
-			knh_setRuntimeError(ctx, T__("out of memory"));
+			knh_setRuntimeError(ctx, T("out of memory"));
 		}
 		else {
 			knh_memcpy(p, s->str, s->size);
@@ -527,7 +503,7 @@ KNHAPI(int) konoha_setMethodFunc(konoha_t konoha, char *name, knh_fmethod func)
 {
 	KONOHA_CHECK(konoha, -1);
 	Ctx *ctx = knh_beginContext(konoha.ctx);
-	knh_NameSpace_t *ns = ctx->share->mainns;
+	NameSpace *ns = ctx->share->mainns;
 	knh_class_t cid; knh_methodn_t mn;
 	knh_bytes_t n = B(name);
 	knh_index_t loc = knh_bytes_rindex(n, '.');
@@ -546,7 +522,7 @@ KNHAPI(int) konoha_setMethodFunc(konoha_t konoha, char *name, knh_fmethod func)
 	}
 	mn = knh_getmn(ctx, n, METHODN_NONAME);
 	{
-		knh_Method_t *mtd = knh_Class_getMethod(ctx, cid, mn);
+		Method *mtd = knh_Class_getMethod(ctx, cid, mn);
 		if(IS_NULL(mtd)) {
 			goto L_ERROR;
 		}
@@ -556,7 +532,7 @@ KNHAPI(int) konoha_setMethodFunc(konoha_t konoha, char *name, knh_fmethod func)
 	return 0;
 
 	L_ERROR:
-	knh_setRuntimeError(ctx, T__("method not found"));
+	knh_setRuntimeError(ctx, T("method not found"));
 	knh_endContext(ctx);
 	return -1;
 }
@@ -654,8 +630,7 @@ void sigint_action(int signum, siginfo_t *info, void *ctx)
 	}
 	else {
 		//knh_initSIGINT();
-		Ctx *ctx = (Ctx*)shellContext;
-		KNH_THROW__T(ctx, "Interrupt!!");
+		KNH_THROWs((Ctx*)shellContext, "Interrupt!!");
 	}
 }
 #endif
@@ -679,7 +654,7 @@ static void knh_initSIGINT(void)
 
 #if defined(KNH_USING_READLINE)
 
-static volatile Ctx *ctxRL = NULL;
+volatile static Ctx *ctxRL = NULL;
 
 /* ------------------------------------------------------------------------ */
 
@@ -730,7 +705,7 @@ char *knh_rl_stmt(const char *text, int state)
 static
 knh_bytes_t knh_bytes_token(knh_bytes_t t)
 {
-	size_t i;
+	int i;
 	for(i = 0; i < t.len; i++) {
 		if(isalnum(t.buf[i])) {
 			t.buf = t.buf + i + 1;
@@ -746,7 +721,7 @@ knh_bytes_t knh_bytes_token(knh_bytes_t t)
 static
 char *knh_rl_name(const char *text, int state)
 {
-	static size_t index;
+	static int index;
 	static knh_bytes_t t = {(knh_uchar_t*)"", 0};
 	Ctx *ctx = (Ctx*)ctxRL;
 	if(state == 0) {
@@ -754,12 +729,12 @@ char *knh_rl_name(const char *text, int state)
 		index = 0;
 	}
 	if(ctx != NULL) {
-		knh_DictMap_t *symbolDictMap = DP(ctx->kc)->symbolDictMap;
+		DictMap *symbolDictMap = DP(ctx->abr)->symbolDictMap;
 		if(state == 0) {
 			knh_DictMap_sort(symbolDictMap);
 		}
 		for(; index < knh_DictMap_size(symbolDictMap); ) {
-			char *name = __tochar(knh_DictMap_keyAt(symbolDictMap, index));
+			char *name = knh_String_tochar(knh_DictMap_keyAt(symbolDictMap, index));
 			index++;
 			if (strncmp(name, (char*)t.buf, t.len) == 0) {
 				return (dupstr(name));
@@ -769,42 +744,46 @@ char *knh_rl_name(const char *text, int state)
 	return NULL;
 }
 
-static void knh_Context_initSymbolTable(Ctx *ctx)
+/* ------------------------------------------------------------------------ */
+
+static
+void knh_Context_initSymbolTable(Ctx *ctx)
 {
-	knh_DictMap_t *symbolDictMap = DP((ctx)->kc)->symbolDictMap;
-	size_t i, j;
+	knh_Context_initAsm(ctx);
+	DictMap *symbolDictMap = DP((ctx)->abr)->symbolDictMap;
+	int i, j;
 	for(i = 0; i < KNH_TCLASS_SIZE; i++) {
-		knh_String_t *sname = ClassTable(i).sname;
+		String *sname = ClassTable(i).sname;
 		if(sname != NULL) {
 			if(knh_class_isPrivate(i)) continue;
 			knh_DictMap_set(ctx, symbolDictMap, sname, UP(sname));
 		}
 		if(ClassTable(i).constPool != NULL) {
-			knh_DictMap_t *dm = ClassTable(i).constPool;
+			DictMap *dm = ClassTable(i).constPool;
 			for(j = 0; j < knh_DictMap_size(dm); j++) {
-				knh_String_t *s = knh_DictMap_keyAt(dm, j);
+				String *s = knh_DictMap_keyAt(dm, j);
 				knh_DictMap_set(ctx, symbolDictMap, s, UP(s));
 			}
 		}
 	}
 	{
-		knh_Array_t *a = (DP(ctx->sys)->FieldNameDictIdx)->terms;
+		Array *a = (DP(ctx->sys)->FieldNameDictIdx)->terms;
 		for(i = 0; i < knh_Array_size(a); i++) {
-			knh_String_t *s = (knh_String_t*)knh_Array_n(a, i);
+			String *s = (String*)knh_Array_n(a, i);
 			if(s->size > 2) {
 				knh_DictMap_set(ctx, symbolDictMap, s, UP(s));
 			}
 		}
 		a = (DP(ctx->sys)->ResourceDictIdx)->terms;
 		for(i = 0; i < knh_Array_size(a); i++) {
-			knh_String_t *s = (knh_String_t*)knh_Array_n(a, i);
+			String *s = (String*)knh_Array_n(a, i);
 			if(s->size > 2) {
 				knh_DictMap_set(ctx, symbolDictMap, s, UP(s));
 			}
 		}
 	}
 	for(i = 0; keywords[i] != NULL; i++) {
-		knh_String_t *s = T__(keywords[i]);
+		String *s = T(keywords[i]);
 		knh_DictMap_set(ctx, symbolDictMap, s, UP(s));
 	}
 }
@@ -835,7 +814,7 @@ static
 void knh_initScriptLine(Ctx *ctx)
 {
 	DBG2_ASSERT(ctx->lines == NULL);
-	KNH_INITv(((knh_Context_t*)ctx)->lines, new_Array(ctx, CLASS_String, 0));
+	KNH_INITv(((Context*)ctx)->lines, new_Array(ctx, CLASS_String, 0));
 }
 
 /* ------------------------------------------------------------------------ */
@@ -863,11 +842,11 @@ void knh_cancelScriptLine(Ctx *ctx, size_t at)
 
 /* ------------------------------------------------------------------------ */
 
-knh_String_t *knh_Context_getScriptLineNULL(Ctx *ctx, size_t at)
+String *knh_Context_getScriptLineNULL(Ctx *ctx, size_t at)
 {
 	if(ctx->lines != NULL) {
 		DBG2_ASSERT(at < (ctx->lines)->size);
-		return (knh_String_t*)knh_Array_n(ctx->lines, at - 1);
+		return (String*)knh_Array_n(ctx->lines, at - 1);
 	}
 	return NULL;
 }
@@ -875,9 +854,9 @@ knh_String_t *knh_Context_getScriptLineNULL(Ctx *ctx, size_t at)
 /* ------------------------------------------------------------------------ */
 /* @method String[] Script.getLines() @Hidden */
 
-METHOD Script_getLines(Ctx *ctx, knh_sfp_t *sfp)
+METHOD knh__Script_getLines(Ctx *ctx, knh_sfp_t *sfp)
 {
-	knh_Array_t *res = (ctx->lines == NULL)
+	Array *res = (ctx->lines == NULL)
 		? new_Array(ctx, CLASS_String, 0) : ctx->lines;
 	KNH_RETURN(ctx, sfp, res);
 }
@@ -888,89 +867,44 @@ static
 void knh_clearScriptLine(Ctx *ctx)
 {
 	if(ctx->lines != NULL) {
-//#if KONOHA_BUILDID > KONOHA_BUILDID_LATEST
-//		if(knh_getenv("I_LOVE_KONOHA") == NULL) {
-//			char buff[FILEPATH_BUFSIZ];
-//			int i, pid = knh_getpid();
-//			//knh_snprintf(buff, sizeof(buff), "\nDo you want to save 'myscript%d.k'? [Y/n]: ", pid);
-//			fprintf(stdout, "\nThank you for joining 'Konoha User Experience Program'.\n");
-//			fprintf(stdout, "All your cooperation will help the future version of konoha.\n");
-//			knh_snprintf(buff, sizeof(buff), "\nDo you want to save your script? [Y/n]: ");
-//			if(knh_ask(ctx, buff, 1)) {
-//				knh_sfp_t *lsfp = KNH_LOCAL(ctx);
-//				knh_OutputStream_t *w;
-//				knh_snprintf(buff, sizeof(buff), "myscript%d.k", pid);
-//				w = new_FileOutputStream(ctx, B(buff), "w", 0);
-//				KNH_LPUSH(ctx, w);
-//				for(i = 0; i < knh_Array_size(ctx->lines); i++) {
-//					knh_bytes_t line = __tobytes((knh_String_t*)knh_Array_n(ctx->lines, i));
-//					if(knh_bytes_startsWith(line, STEXT("man ")) || knh_bytes_startsWith(line, STEXT("dump "))) {
-//						knh_write(ctx, w, STEXT("// "));
-//					}
-//					knh_println(ctx, w, line);
-//				}
-//				knh_OutputStream_close(ctx, w);
-//				KNH_LOCALBACK(ctx, lsfp);
-//				fprintf(stdout, "More importantly, your script was saved at '%s.'\n", buff);
-//				fprintf(stdout, "We appliciate when you send us your found problems.\n");
-//				fprintf(stdout, "Thank you, again.\n");
-//			}
-//		}
-//#endif
-		KNH_FINALv(ctx, ((knh_Context_t*)ctx)->lines);
-		((knh_Context_t*)ctx)->lines = NULL;
+#if KONOHA_BUILDID > KONOHA_BUILDID_LATEST
+		if(knh_getenv("I_LOVE_KONOHA") == NULL) {
+			char buff[FILEPATH_BUFSIZ];
+			int i, pid = knh_getpid();
+			//knh_snprintf(buff, sizeof(buff), "\nDo you want to save 'myscript%d.k'? [Y/n]: ", pid);
+			fprintf(stdout, "\nThank you for joining 'Konoha User Experience Program'.\n");
+			fprintf(stdout, "All your cooperation will help the future version of konoha.\n");
+			L_AGAIN:;
+			knh_snprintf(buff, sizeof(buff), "\nDo you want to save your script? [Y/n]: ");
+			if(knh_ask(ctx, buff, 1)) {
+				knh_sfp_t *lsfp = KNH_LOCAL(ctx);
+				OutputStream *w;
+				knh_snprintf(buff, sizeof(buff), "myscript%d.k", pid);
+				w = new_FileOutputStream(ctx, B(buff), "w", 0);
+				KNH_LPUSH(ctx, w);
+				for(i = 0; i < knh_Array_size(ctx->lines); i++) {
+					knh_bytes_t line = knh_String_tobytes((String*)knh_Array_n(ctx->lines, i));
+					if(knh_bytes_startsWith(line, STEXT("man ")) || knh_bytes_startsWith(line, STEXT("dump "))) {
+						knh_write(ctx, w, STEXT("// "));
+					}
+					knh_println(ctx, w, line);
+				}
+				knh_OutputStream_close(ctx, w);
+				KNH_LOCALBACK(ctx, lsfp);
+				fprintf(stdout, "More importantly, your script was saved at '%s.'\n", buff);
+				fprintf(stdout, "We appliciate when you send us your found problems.\n");
+				fprintf(stdout, "Thank you, again.\n");
+			}
+		}
+#endif
+		KNH_FINALv(ctx, ((Context*)ctx)->lines);
+		((Context*)ctx)->lines = NULL;
 	}
 }
 
 /* ======================================================================== */
 /* [shell] */
 
-static
-void knh_showWelcome(Ctx *ctx, knh_OutputStream_t *w)
-{
-	knh_printf(ctx, w, "Konoha %s(%s) %s/%s (rev:%d, %s %s)",
-		KONOHA_VERSION, KONOHA_XCODE, KONOHA_DIST, KONOHA_LICENSE,
-		((size_t)KONOHA_REVISION), __DATE__, __TIME__);
-	knh_write_EOL(ctx, w);
-	knh_printf(ctx, w, "[%s] on %s (%d, %s)\n", KONOHA_CC_VERSION, KONOHA_PLATFORM, (knh_intptr_t)(sizeof(void*) * 8), konoha_encoding());
-	knh_write_char(ctx, w, "Options:");
-
-#ifdef KNH_FASTMODE
-	knh_write_char(ctx, w, " fastmode");
-#endif
-
-#ifdef KNH_USING_ICONV
-	knh_write_char(ctx, w, " iconv");
-#endif
-#ifdef KNH_USING_INT32
-	knh_write_char(ctx, w, " int32");
-#endif
-#ifdef KNH_USING_RCGC
-	knh_write_char(ctx, w, " refc");
-#endif
-#ifdef KNH_USING_MARKGC
-	knh_write_char(ctx, w, " m&s");
-#endif
-	knh_write_USING_SQLITE3(ctx, w);
-#ifdef KNH_USING_THREAD
-	knh_write_char(ctx, w, " thread");
-#endif
-	knh_write_USING_REGEX(ctx, w);
-	knh_printf(ctx, w, " used_memory:%d kb", (knh_intptr_t)(ctx->stat->usedMemorySize / 1024));
-	knh_write_EOL(ctx, w);
-#ifdef KONOHA_EXPIRE
-	{
-		knh_uint_t t = knh_time();
-		if(t > KONOHA_EXPIRE) {
-			fprintf(stderr, "Konoha rev:%d was EXPIRED. Do 'svn update' NOW.\n\n", ((int)KONOHA_REVISION));
-			exit(0);
-		}
-	}
-#endif
-}
-
-/* ------------------------------------------------------------------------ */
-
 KNHAPI(void) konoha_shell(konoha_t konoha)
 {
 	KONOHA_CHECK_(konoha);
@@ -984,8 +918,7 @@ KNHAPI(void) konoha_shell(konoha_t konoha)
 	knh_Context_initSymbolTable(ctx);
 	using_history();
 #endif
-	knh_showWelcome(ctx, KNH_STDOUT);
-	knh_check_update(ctx);
+	knh_System__dump(ctx, ctx->sys, KNH_STDOUT, (String*)KNH_NULL);
 	knh_initScriptLine(ctx);
 	{
 		int linenum, linecnt = 0;
@@ -1054,7 +987,7 @@ KNHAPI(void) konoha_shell(konoha_t konoha)
 			//knh_Bytes_putc(ctx, cwb->ba, ';');
 			knh_Bytes_putc(ctx, cwb->ba, '\n');
 			{
-				knh_InputStream_t *in = new_BytesInputStream(ctx, cwb->ba, cwb->pos, knh_Bytes_size(cwb->ba));
+				InputStream *in = new_BytesInputStream(ctx, cwb->ba, cwb->pos, knh_Bytes_size(cwb->ba));
 				KNH_MOV(ctx, lsfp[0].o, in);
 				DP(in)->uri = knh_getResourceId(ctx, STEXT("(shell)"));
 				DP(in)->line = linenum;
