@@ -38,14 +38,33 @@ extern "C" {
 /* ======================================================================== */
 /* [Constant Value] */
 
+int knh_addClassConst(Ctx *ctx, knh_class_t cid, String* name, Object *value)
+{
+	KNH_ASSERT_cid(cid);
+	if(ClassTable(cid).constPool == NULL) {
+		knh_ClassTable_t *t = pClassTable(cid);
+		KNH_INITv(t->constPool, new_DictMap0(ctx, 0));
+	}
+	DictMap *cmap = ClassTable(cid).constPool;
+	KNH_ASSERT(IS_DictMap(cmap));
+	KNH_LOCK(ctx, LOCK_SYSTBL, NULL);
+	int res = knh_DictMap_index__b(cmap, knh_String_tobytes(name));
+	if(res != -1) return 0;
+	knh_DictMap_append(ctx, cmap, name, value);
+	KNH_UNLOCK(ctx, LOCK_SYSTBL, NULL);
+	return 1;
+}
+
+/* ------------------------------------------------------------------------ */
+
 Object *knh_getClassConstNULL(Ctx *ctx, knh_class_t cid, knh_bytes_t name)
 {
-	DBG2_ASSERT_cid(cid);
+	KNH_ASSERT_cid(cid);
 	if(ClassTable(cid).constPool == NULL) return NULL;
-	knh_DictMap_t *cmap = ClassTable(cid).constPool;
+	DictMap *cmap = ClassTable(cid).constPool;
 	Object *value = NULL;
 	KNH_LOCK(ctx, LOCK_SYSTBL, NULL);
-	int res = knh_DictMap_index(cmap, name);
+	int res = knh_DictMap_index__b(cmap, name);
 	if(res != -1) {
 		value = knh_DictMap_valueAt(cmap, res);
 	}
@@ -53,26 +72,6 @@ Object *knh_getClassConstNULL(Ctx *ctx, knh_class_t cid, knh_bytes_t name)
 	return value;
 }
 
-/* ------------------------------------------------------------------------ */
-
-int knh_addClassConst(Ctx *ctx, knh_class_t cid, knh_String_t* name, Object *value)
-{
-	DBG2_ASSERT_cid(cid);
-	if(ClassTable(cid).constPool == NULL) {
-		knh_ClassTable_t *t = pClassTable(cid);
-		KNH_INITv(t->constPool, new_DictMap0(ctx, 0));
-	}
-	knh_DictMap_t *cmap = ClassTable(cid).constPool;
-	KNH_ASSERT(IS_DictMap(cmap));
-	KNH_LOCK(ctx, LOCK_SYSTBL, NULL);
-	int res = knh_DictMap_index(cmap, __tobytes(name));
-	if(res != -1) return 0;
-	knh_DictMap_append(ctx, cmap, name, value);
-	KNH_UNLOCK(ctx, LOCK_SYSTBL, NULL);
-	return 1;
-}
-
-
 /* ------------------------------------------------------------------------ */
 
 static
@@ -80,12 +79,12 @@ void knh_addConstData(Ctx *ctx, char *dname, Object *value)
 {
 	knh_bytes_t n = B(dname);
 	knh_index_t loc = knh_bytes_rindex(n, '.');
-	knh_String_t *name = T__(dname + (loc+1));
+	String *name = T(dname + (loc+1));
 	knh_class_t cid = CLASS_Any;
 	if(loc != -1) {
-		if(ctx->kc != NULL && IS_Gamma(ctx->kc)) {
-			DBG2_P("nsname=%s", __tochar(DP(DP(ctx->kc)->ns)->nsname));
-			cid = knh_NameSpace_getcid(ctx, DP(ctx->kc)->ns, knh_bytes_first(n, loc));
+		if(ctx->abr != NULL && IS_Asm(ctx->abr)) {
+			DBG2_P("nsname=%s", knh_String_tochar(DP(DP(ctx->abr)->ns)->nsname));
+			cid = knh_NameSpace_getcid(ctx, DP(ctx->abr)->ns, knh_bytes_first(n, loc));
 		}
 		else {
 			cid = knh_getcid(ctx, knh_bytes_first(n, loc));
@@ -125,24 +124,55 @@ KNHAPI(void) knh_loadFloatConstData(Ctx *ctx, knh_FloatConstData_t *data)
 KNHAPI(void) knh_loadStringConstData(Ctx *ctx, knh_StringConstData_t *data)
 {
 	while(data->name != NULL) {
-		Object *value = UP(T__(data->value));
+		Object *value = UP(T(data->value));
 		knh_addConstData(ctx, data->name, value);
 		data++;
 	}
 }
 
+/* ------------------------------------------------------------------------ */
+
+void knh_Const__man(Ctx *ctx, knh_class_t cid, OutputStream *w)
+{
+	KNH_ASSERT_cid(cid);
+	if(ClassTable(cid).constPool == NULL) return ;
+	DictMap *tcmap = ClassTable(cid).constPool;
+	size_t i, size = knh_DictMap_size(tcmap);
+	KNH_LOCK(ctx, LOCK_SYSTBL, NULL);
+	int hasCaption = 0;
+	for(i = 0; i < size; i++) {
+		if(!hasCaption) {
+			knh_write_char(ctx, w, _("Const"));
+			knh_write_EOL(ctx, w);
+			hasCaption = 1;
+		}
+		knh_printf(ctx, w, "\t%s.%s: %O\n", CLASSN(cid),
+				knh_String_tochar(knh_DictMap_keyAt(tcmap, i)),
+				knh_DictMap_valueAt(tcmap, i));
+	}
+	KNH_UNLOCK(ctx, LOCK_SYSTBL, NULL);
+}
+
 /* ======================================================================== */
 /* [SystemConst] */
 
-KNHAPI(Object*) knh_getSystemConst(Ctx *ctx, int n)
+#define _KNH_SYS(ctx, n)    knh_getSystemConst(ctx, n)
+#define _KNH_SYS_CTX    0
+#define _KNH_SYS_STDIN  1
+#define _KNH_SYS_STDOUT 2
+#define _KNH_SYS_STDERR 3
+#define _KNH_SYS_OS     4
+#define _KNH_SYS_SCRIPT 5
+
+Object *knh_getSystemConst(Ctx *ctx, int n)
 {
 	switch(n) {
-		case KNH_SYS_CTX:     return (Object*)ctx;
-		case KNH_SYS_STDIN:   return (Object*)((ctx)->in);
-		case KNH_SYS_STDOUT:  return (Object*)((ctx)->out);
-		case KNH_SYS_STDERR:  return (Object*)((ctx)->err);
-		case KNH_SYS_OS:      return (Object*)knh_getClassDefaultValue(ctx, CLASS_System);
-		case KNH_SYS_SCRIPT:  return (Object*)knh_NameSpace_getScript(ctx, (ctx->share)->mainns);
+		case _KNH_SYS_CTX:     return (Object*)ctx;
+		case _KNH_SYS_STDIN:   return (Object*)((ctx)->in);
+		case _KNH_SYS_STDOUT:  return (Object*)((ctx)->out);
+		case _KNH_SYS_STDERR:  return (Object*)((ctx)->err);
+		case _KNH_SYS_OS:      return (Object*)knh_getClassDefaultValue(ctx, CLASS_System);
+		case _KNH_SYS_SCRIPT:  return (Object*)knh_NameSpace_getScript(ctx, (ctx->share)->mainns);
 	}
 	DBG_P("unknown system const n=%d", n);
 	return KNH_NULL;
@@ -197,12 +227,12 @@ knh_fieldn_t knh_System_getfn(Ctx *ctx, knh_bytes_t n, knh_fieldn_t def)
 	}
 	if(idx == -1) {
 		if(def == FIELDN_NEWID) {
-			knh_String_t *s = new_String(ctx, n, NULL);
+			String *s = new_String(ctx, n, NULL);
 			idx = knh_DictIdx_add__fast(ctx, DP(ctx->sys)->FieldNameDictIdx, s);
 			if(!(idx < KNH_FLAG_MN_SETTER)) {  /* Integer overflowed */
 				DBG_P("idx=%d < %d", (int)idx, (int)KNH_FLAG_MN_SETTER);
 				DBG2_ABORT();
-				KNH_THROW__T(ctx, "OutOfMemory!!: Too many field names");
+				KNH_THROWs(ctx, "OutOfMemory!!: Too many field names");
 			}
 		}
 		else {
@@ -270,10 +300,10 @@ char *knh_format_newFMT(char *buf, size_t bufsiz, knh_bytes_t t, int dot, char *
 /* ======================================================================== */
 /* [tfieldn] */
 
-knh_String_t *knh_getFieldName(Ctx *ctx, knh_fieldn_t fn)
+String *knh_getFieldName(Ctx *ctx, knh_fieldn_t fn)
 {
 	KNH_LOCK(ctx, LOCK_SYSTBL, NULL);
-	knh_String_t *s = (knh_String_t*)knh_DictIdx_get__fast(DP(ctx->sys)->FieldNameDictIdx, FIELDN_UNMASK(fn));
+	String *s = (String*)knh_DictIdx_get__fast(DP(ctx->sys)->FieldNameDictIdx, FIELDN_UNMASK(fn));
 	KNH_UNLOCK(ctx, LOCK_SYSTBL, NULL);
 	DBG_(
 	if(IS_NULL(s)) {
@@ -283,6 +313,8 @@ knh_String_t *knh_getFieldName(Ctx *ctx, knh_fieldn_t fn)
 	return s;
 }
 
+#define _FIELDN(fn) knh_String_tochar(knh_getFieldName(ctx, fn))
+
 /* ------------------------------------------------------------------------ */
 
 knh_fieldn_t knh_getfnq(Ctx *ctx, knh_bytes_t tname, knh_fieldn_t def)
@@ -319,12 +351,12 @@ knh_fieldn_t knh_getfnq(Ctx *ctx, knh_bytes_t tname, knh_fieldn_t def)
 		idx = knh_DictIdx_index(ctx, DP(ctx->sys)->FieldNameDictIdx, tname);
 		if(idx == -1) {
 			if(def == FIELDN_NEWID) {
-				knh_String_t *s = new_String(ctx, tname, NULL);
+				String *s = new_String(ctx, tname, NULL);
 				idx = knh_DictIdx_add__fast(ctx, DP(ctx->sys)->FieldNameDictIdx, s);
 				if(!(idx < KNH_FLAG_FN_U2)) {
 					DBG_P("idx=%d < %d", (int)idx, (int)KNH_FLAG_FN_U2);
 					DBG2_ABORT();
-					KNH_THROW__T(ctx, "OutOfMemory!!: Too many field names");
+					KNH_THROWs(ctx, "OutOfMemory!!: Too many field names");
 				}
 			}
 			else {
@@ -339,38 +371,38 @@ knh_fieldn_t knh_getfnq(Ctx *ctx, knh_bytes_t tname, knh_fieldn_t def)
 /* ======================================================================== */
 /* [methodn] */
 
-//String *new_String__mn(Ctx *ctx, knh_methodn_t mn)
-//{
-//	if(METHODN_IS_MOVTEXT(mn)) {
-//		char buf[CLASSNAME_BUFSIZ];
-//		knh_snprintf(buf, sizeof(buf), "%%%s", FIELDN(METHODN_TOFIELDN(mn)));
-//		return new_String(ctx, B(buf), NULL);
-//	}
-//	if(METHODN_IS_GETTER(mn)) {
-//		char buf[CLASSNAME_BUFSIZ];
-//		knh_snprintf(buf, sizeof(buf), "get%s", FIELDN(METHODN_TOFIELDN(mn)));
-//		if(islower(buf[3])) buf[3] = toupper(buf[3]);
-//		return new_String(ctx, B(buf), NULL);
-//	}
-//	if(METHODN_IS_SETTER(mn)) {
-//		char buf[CLASSNAME_BUFSIZ];
-//		knh_snprintf(buf, sizeof(buf), "set%s", FIELDN(METHODN_TOFIELDN(mn)));
-//		if(islower(buf[3])) buf[3] = toupper(buf[3]);
-//		return new_String(ctx, B(buf), NULL);
-//	}
-//
-//	KNH_LOCK(ctx, LOCK_SYSTBL, NULL);
-//	String *s = (String*)knh_DictIdx_get__fast(DP(ctx->sys)->FieldNameDictIdx, FIELDN_UNMASK(mn));
-//	KNH_UNLOCK(ctx, LOCK_SYSTBL, NULL);
-//	if(IS_NULL(s)) {
-//		return TS_EMPTY;
-//	}
-//	return s;
-//}
+String *new_String__mn(Ctx *ctx, knh_methodn_t mn)
+{
+	if(METHODN_IS_MOVTEXT(mn)) {
+		char buf[CLASSNAME_BUFSIZ];
+		knh_snprintf(buf, sizeof(buf), "%%%s", FIELDN(METHODN_TOFIELDN(mn)));
+		return new_String(ctx, B(buf), NULL);
+	}
+	if(METHODN_IS_GETTER(mn)) {
+		char buf[CLASSNAME_BUFSIZ];
+		knh_snprintf(buf, sizeof(buf), "get%s", FIELDN(METHODN_TOFIELDN(mn)));
+		if(islower(buf[3])) buf[3] = toupper(buf[3]);
+		return new_String(ctx, B(buf), NULL);
+	}
+	if(METHODN_IS_SETTER(mn)) {
+		char buf[CLASSNAME_BUFSIZ];
+		knh_snprintf(buf, sizeof(buf), "set%s", FIELDN(METHODN_TOFIELDN(mn)));
+		if(islower(buf[3])) buf[3] = toupper(buf[3]);
+		return new_String(ctx, B(buf), NULL);
+	}
+
+	KNH_LOCK(ctx, LOCK_SYSTBL, NULL);
+	String *s = (String*)knh_DictIdx_get__fast(DP(ctx->sys)->FieldNameDictIdx, FIELDN_UNMASK(mn));
+	KNH_UNLOCK(ctx, LOCK_SYSTBL, NULL);
+	if(IS_NULL(s)) {
+		return TS_EMPTY;
+	}
+	return s;
+}
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(knh_methodn_t) knh_getmn(Ctx *ctx, knh_bytes_t tname, knh_methodn_t def)
+knh_methodn_t knh_getmn(Ctx *ctx, knh_bytes_t tname, knh_methodn_t def)
 {
 	if(tname.len == 0 || tname.len > 64) {
 		DBG2_P("invalid method name '%s'", tname.buf);
@@ -383,13 +415,6 @@ KNHAPI(knh_methodn_t) knh_getmn(Ctx *ctx, knh_bytes_t tname, knh_methodn_t def)
 		}
 		return KNH_FLAG_MN_MOVTEXT | fn;
 	}
-	else if(tname.buf[0] == 'i' && tname.buf[1] == 's') { /* is => get */
-		knh_fieldn_t fn = knh_System_getfn(ctx, knh_bytes_last(tname, 2), def);
-		if(fn == FIELDN_NONAME) {
-			return METHODN_NONAME;
-		}
-		return KNH_FLAG_MN_GETTER | fn;
-	}
 	else if(tname.buf[0] == 'g' && tname.buf[1] == 'e' && tname.buf[2] == 't') {
 		knh_fieldn_t fn = knh_System_getfn(ctx, knh_bytes_last(tname, 3), def);
 		if(fn == FIELDN_NONAME) {
@@ -410,52 +435,52 @@ KNHAPI(knh_methodn_t) knh_getmn(Ctx *ctx, knh_bytes_t tname, knh_methodn_t def)
 	}
 }
 
-///* ------------------------------------------------------------------------ */
-//
-//char *knh_format_methodn(Ctx *ctx, char *buf, size_t bufsiz, knh_methodn_t mn)
-//{
-//	if(METHODN_IS_MOVTEXT(mn)) {
-//		knh_snprintf(buf, bufsiz, "%%%s", FIELDN(METHODN_TOFIELDN(mn)));
-//		return buf;
-//	}
-//	if(METHODN_IS_GETTER(mn)) {
-//		knh_snprintf(buf, bufsiz, "get%s", FIELDN(METHODN_TOFIELDN(mn)));
-//		if(islower(buf[3])) buf[3] = toupper(buf[3]);
-//		return buf;
-//	}
-//	if(METHODN_IS_SETTER(mn)) {
-//		knh_snprintf(buf, bufsiz, "set%s", FIELDN(METHODN_TOFIELDN(mn)));
-//		if(islower(buf[3])) buf[3] = toupper(buf[3]);
-//		return buf;
-//	}
-//	knh_snprintf(buf, bufsiz, "%s", FIELDN(mn));
-//	return buf;
-//}
-//
-///* ------------------------------------------------------------------------ */
-//
-//char *
-//knh_format_cmethodn(Ctx *ctx, char *buf, size_t bufsiz, knh_class_t cid, knh_methodn_t mn)
-//{
-//	if(METHODN_IS_MOVTEXT(mn)) {
-//		knh_snprintf(buf, bufsiz, "%s.%%%s", CLASSN(cid), FIELDN(METHODN_TOFIELDN(mn)));
-//		return buf;
-//	}
-//	if(METHODN_IS_GETTER(mn)) {
-//		int off = knh_strlen(CLASSN(cid))+4;
-//		knh_snprintf(buf, bufsiz, "%s.get%s", CLASSN(cid), FIELDN(METHODN_TOFIELDN(mn)));
-//		if(islower(buf[off])) buf[off] = toupper(buf[off]);
-//		return buf;
-//	}
-//	if(METHODN_IS_SETTER(mn)) {
-//		int off = knh_strlen(CLASSN(cid))+4;
-//		knh_snprintf(buf, bufsiz, "%s.set%s", CLASSN(cid), FIELDN(METHODN_TOFIELDN(mn)));
-//		if(islower(buf[off])) buf[off] = toupper(buf[off]);
-//		return buf;
-//	}
-//	knh_snprintf(buf, bufsiz, "%s.%s", CLASSN(cid), FIELDN(mn));
-//	return buf;
-//}
+/* ------------------------------------------------------------------------ */
+
+char *knh_format_methodn(Ctx *ctx, char *buf, size_t bufsiz, knh_methodn_t mn)
+{
+	if(METHODN_IS_MOVTEXT(mn)) {
+		knh_snprintf(buf, bufsiz, "%%%s", FIELDN(METHODN_TOFIELDN(mn)));
+		return buf;
+	}
+	if(METHODN_IS_GETTER(mn)) {
+		knh_snprintf(buf, bufsiz, "get%s", FIELDN(METHODN_TOFIELDN(mn)));
+		if(islower(buf[3])) buf[3] = toupper(buf[3]);
+		return buf;
+	}
+	if(METHODN_IS_SETTER(mn)) {
+		knh_snprintf(buf, bufsiz, "set%s", FIELDN(METHODN_TOFIELDN(mn)));
+		if(islower(buf[3])) buf[3] = toupper(buf[3]);
+		return buf;
+	}
+	knh_snprintf(buf, bufsiz, "%s", FIELDN(mn));
+	return buf;
+}
+
+/* ------------------------------------------------------------------------ */
+
+char *
+knh_format_cmethodn(Ctx *ctx, char *buf, size_t bufsiz, knh_class_t cid, knh_methodn_t mn)
+{
+	if(METHODN_IS_MOVTEXT(mn)) {
+		knh_snprintf(buf, bufsiz, "%s.%%%s", CLASSN(cid), FIELDN(METHODN_TOFIELDN(mn)));
+		return buf;
+	}
+	if(METHODN_IS_GETTER(mn)) {
+		int off = knh_strlen(CLASSN(cid))+4;
+		knh_snprintf(buf, bufsiz, "%s.get%s", CLASSN(cid), FIELDN(METHODN_TOFIELDN(mn)));
+		if(islower(buf[off])) buf[off] = toupper(buf[off]);
+		return buf;
+	}
+	if(METHODN_IS_SETTER(mn)) {
+		int off = knh_strlen(CLASSN(cid))+4;
+		knh_snprintf(buf, bufsiz, "%s.set%s", CLASSN(cid), FIELDN(METHODN_TOFIELDN(mn)));
+		if(islower(buf[off])) buf[off] = toupper(buf[off]);
+		return buf;
+	}
+	knh_snprintf(buf, bufsiz, "%s.%s", CLASSN(cid), FIELDN(mn));
+	return buf;
+}
 
 /* ======================================================================== */
 /* [uri] */
@@ -465,7 +490,7 @@ knh_uri_t knh_getResourceId(Ctx *ctx, knh_bytes_t t)
 	KNH_LOCK(ctx, LOCK_SYSTBL, NULL);
 	knh_index_t idx = knh_DictIdx_index(ctx, DP(ctx->sys)->ResourceDictIdx, t);
 	if(idx == -1) {
-		knh_String_t *s = new_String(ctx, t, NULL);
+		String *s = new_String(ctx, t, NULL);
 		idx = knh_DictIdx_add__fast(ctx, DP(ctx->sys)->ResourceDictIdx, s);
 	}
 	KNH_UNLOCK(ctx, LOCK_SYSTBL, NULL);
@@ -486,28 +511,30 @@ knh_uri_t knh_cwb_getResourceId(Ctx *ctx, knh_cwb_t *cwb)
 	{
 		knh_index_t idx = knh_DictIdx_index(ctx, DP(ctx->sys)->ResourceDictIdx, t);
 		if(idx == -1) {
-			knh_String_t *s = knh_cwb_newString(ctx, cwb);
+			String *s = new_String__cwb(ctx, cwb);
 			idx = knh_DictIdx_add__fast(ctx, DP(ctx->sys)->ResourceDictIdx, s);
-			t = __tobytes(s);
+			t = knh_String_tobytes(s);
 			knh_cwb_write(ctx, cwb, t);
 		}
 		uri = (knh_uri_t)idx;
 	}
 	KNH_UNLOCK(ctx, LOCK_SYSTBL, NULL);
+	DBG_P("uri=%d,%d '%s'", uri, URI_UNTRUSTED(uri), t.buf);
 	if(!knh_isTrustedPath(ctx, t)) {
 		uri = URI_UNTRUSTED(uri);
 	}
+	DBG_P("uri=%d,%d '%s'", uri, URI_UNTRUSTED(uri), t.buf);
 	return uri;
 }
 
 /* ------------------------------------------------------------------------ */
 
-knh_String_t *knh_getResourceName(Ctx *ctx, knh_uri_t uri)
+String *knh_getResourceName(Ctx *ctx, knh_uri_t uri)
 {
-	knh_String_t *s;
+	String *s;
 	uri = URI_UNMASK(uri);
 	KNH_LOCK(ctx, LOCK_SYSTBL, NULL);
-	s = (knh_String_t*)knh_DictIdx_get__fast(DP(ctx->sys)->ResourceDictIdx, uri);
+	s = (String*)knh_DictIdx_get__fast(DP(ctx->sys)->ResourceDictIdx, uri);
 	KNH_UNLOCK(ctx, LOCK_SYSTBL, NULL);
 	DBG_(
 		if(IS_NULL(s)) {
@@ -515,9 +542,12 @@ knh_String_t *knh_getResourceName(Ctx *ctx, knh_uri_t uri)
 			return TS_EMPTY;
 		}
 	)
+	DBG_P("uri=%d, '%s'", uri, knh_String_tochar(s));
 	return s;
 }
 
+#define _URIDN(uri) knh_String_tochar(knh_getResourceName(ctx, uri))
+
 /* ======================================================================== */
 /* [tPackage] */
 
@@ -561,16 +591,16 @@ knh_drvapi_t *knh_getDriverAPI(Ctx *ctx, int type, knh_bytes_t name)
 /* ------------------------------------------------------------------------ */
 /* [NameSpace] */
 
-knh_NameSpace_t *knh_getNameSpace(Ctx *ctx, knh_bytes_t name)
+NameSpace *knh_getNameSpace(Ctx *ctx, knh_bytes_t name)
 {
 	if(knh_bytes_equals(name, STEXT("main"))) {
 		KNH_ASSERT(IS_NameSpace(ctx->share->mainns));
 		return ctx->share->mainns;
 	}
 	else {
-		knh_NameSpace_t *ns;
+		NameSpace *ns;
 		KNH_LOCK(ctx, LOCK_SYSTBL, NULL);
-		ns = (knh_NameSpace_t*)knh_DictMap_get__b(ctx,  DP(ctx->sys)->NameSpaceTableDictMap, name);
+		ns = (NameSpace*)knh_DictMap_get__b(ctx,  DP(ctx->sys)->NameSpaceTableDictMap, name);
 		KNH_UNLOCK(ctx, LOCK_SYSTBL, NULL);
 		return ns;
 	}
